﻿Type=StaticCode
Version=2.5
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'This module contains file handling functions.
Private Sub Process_Globals
	Private fx As JFX
End Sub

'Read file and return content in bytes array.
Public Sub ReadFile(Dir As String, FileName As String) As Byte()
   Dim Out As OutputStream
   Out.InitializeToBytesArray(100) 'size not really important
   File.Copy2(File.OpenInput(Dir, FileName), Out)
   Return Out.ToBytesArray
End Sub

' Returns the filename from a full path string.
Public Sub getFileName(path As String) As String
  ' Define the JavaObject
  Dim fileO As JavaObject
  ' Get the fileio class
  fileO.InitializeNewInstance("java.io.File", Array As Object(path))
  ' Getthe filename
  Return fileO.RunMethod("getName", Null)
End Sub

' Returns the file path from a full path string.
Public Sub getFilePath(path As String) As String
  ' Define the JavaObject
  Dim fileO As JavaObject
  ' Get the fileio class
  fileO.InitializeNewInstance("java.io.File", Array As Object(path))
  ' Getthe filename and path
  Dim n As String = fileO.RunMethod("getName", Null)
  Dim p As String = fileO.RunMethod("getPath", Null)
  p = p.Replace(n, "")
  Return p
End Sub

'Load Spectrum memory snapshot file (.SNA).
Public Sub LoadSna(OwnerForm As Form)
	Dim FileDialog As FileChooser
	Dim SelectedFile As String
	FileDialog.Initialize
	FileDialog.SetExtensionFilter ("Z80 Snapshot file", Array As String ("*.sna"))
	SelectedFile = FileDialog.ShowOpen(OwnerForm)
	If SelectedFile.ToLowerCase.EndsWith("sna") Then 'Proceed only if a sna-file was chosen
		'Load and interpret the file.
		InitializeSnapshot(getFilePath(SelectedFile), getFileName(SelectedFile)) 
	End If
End Sub
Public Sub InitializeSnapshot(FilePath As String, FileName As String)
	'The specification for the Spectrum Snapshopt format is found here: http://www.worldofspectrum.org/faq/reference/formats.htm
	Dim SNABytes() As Byte
	Dim MemoryStart = 16384 As Int 'Lowest address of RAM.
	Dim i As Int
	'Load data into an array.
	SNABytes = ReadFile(FilePath,FileName)
	'Load array data into memory
	For i = 27 To 49178
		Memory.Write(MemoryStart + i - 27,Bit.AND(SNABytes(i),0xFF)) 'The And-operation is to transform the signed byte to an unsigned Int.
	Next
	'Load Z80 registers.
	Z80.i = Bit.AND(SNABytes(0),255)
	Z80.lPrime = Bit.AND(SNABytes(1),255)
	Z80.hPrime = Bit.AND(SNABytes(2),255)
	Z80.ePrime = Bit.AND(SNABytes(3),255)
	Z80.dPrime = Bit.AND(SNABytes(4),255)
	Z80.cPrime = Bit.AND(SNABytes(5),255)
	Z80.bPrime = Bit.AND(SNABytes(6),255)
	Z80.fPrime = Bit.AND(SNABytes(7),255)
	Z80.aPrime = Bit.AND(SNABytes(8),255)
	Z80.l = Bit.AND(SNABytes(9),255)
	Z80.h = Bit.AND(SNABytes(10),255)
	Z80.e = Bit.AND(SNABytes(11),255)
	Z80.d = Bit.AND(SNABytes(12),255)
	Z80.c = Bit.AND(SNABytes(13),255)
	Z80.b = Bit.AND(SNABytes(14),255)
	Z80.y = Bit.AND(SNABytes(15),255)
	Z80.i2 = Bit.AND(SNABytes(16),255)
	Z80.x = Bit.AND(SNABytes(17),255)
	Z80.i1 = Bit.AND(SNABytes(18),255)
	'The IE-flag at byte 19 is set in bit 1 according to www.worldofspectrum.org, but it seems that other bits can be used instead.
	If SNABytes(19) <> 0 Then Z80.InterruptEnabled = True Else Z80.InterruptEnabled = False
	Z80.r = Bit.AND(SNABytes(20),255)
	Z80.f = Bit.AND(SNABytes(21),255)
	Z80.a = Bit.AND(SNABytes(22),255)
	Z80.SP = Bit.AND(SNABytes(23),255) + 256 * Bit.AND(SNABytes(24),255)
	Z80.InterruptMode = Bit.AND(SNABytes(25),255)
	OutPort.BorderColor = Bit.AND(SNABytes(26),255)
End Sub
