﻿Type=StaticCode
Version=2.5
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'This module handles the signals to the Z80 IN port, including keyboard and joystick.
'Information about Spectrum IN ports can be found here: http://www.worldofspectrum.org/faq/reference/ports.htm.
'How keyboard input affects IN port data is decribed here: http://www.worldofspectrum.org/ZXBasicManual/zxmanchap23.html
Sub Process_Globals
	Private fx As JFX
	Private In31 As Int 'Kempston joystick
	Private In32766, In49150, In57342, In61438, In63486, In64510, In65022, In65278 As Int 'Regular keyboard
	Private In56318, In59390, In59646 As Int 'Alternate keyboard input
	Public Logger As tsLogger
End Sub
Sub Initialize
	' Initialize Keyboard
	In31 = 0 'Kempston joystick interface. R=1,L=2,D=4,U=8,B1=16 (B2 not implemented)
	'Regular keyboard input
	'No key = 0xBF
	In32766 = 0xBF
	In49150 = 0xBF
	In57342 = 0xBF
	In61438 = 0xBF
	In63486 = 0xBF
	In64510 = 0xBF	
	In65022 = 0xBF
	In65278 = 0xBF
	'Alternate keyboard input.
	'No key = 0xFF
	In56318 = 0xFF 'Row Q-P
	In59390 = 0xFF 'Row 1-0
	'Logger
	Logger.Configuration.LogFilename = "SP48.log"
	Logger.Configuration.IsConsoleEnable = True
	Logger.Configuration.IsFileWriterEnable = False
	Logger.Initialize
	Logger.toLevel(0) 'Level 0 = OFF, 1 = TRACE, 2 = DEBUG, 3 = INFO, 4 = WARN, 5 = ERROR, 6 = FATAL
End Sub
Sub In(P As Int) As Int
	'Collect In values for different keyboard ports.
	Dim InValue As Int
	Dim PHigh,PLow As Int
	Dim Temp As Int
	PHigh = P / 256
	PLow = P - PHigh * 256
	If PLow = 0xFE Then 'Keyboard is mapped to addresses with low byte = 0xFE
		Select P
			Case 32766
				InValue = In32766 'Reads the half row SPACE to B
			Case 49150
				InValue = In49150 'Reads the half row ENTER to H
			Case 57342
				InValue = In57342 'Reads the half row P to 7
			Case 61438
				InValue = In61438 'Reads the half row O to 6
			Case 63486
				InValue = In63486 'Reads the half row 1 to 5
			Case 64510
				InValue = In64510 'Reads the half row Q to T
			Case 65022
				InValue = In65022 'Reads the half row A to G
			Case 65278
				InValue = In65278 'Reads the half row CAPS SHIFT to V
			Case 59390
				InValue = In59390 'Reads the row 1 to 0
			Case 56318
				InValue = In56318 'Reads the row Q to P
			Case 59646
				InValue = In59646 'Reads the half rows A to G and CAPS SHIFT to V 
			Case 254
				Temp = Bit.AND(In32766,In49150)
				Temp = Bit.AND(Temp,In57342)
				Temp = Bit.AND(Temp,In61438)
				Temp = Bit.AND(Temp,In63486)
				Temp = Bit.AND(Temp,In64510)
				Temp = Bit.AND(Temp,In65022)
				InValue = Bit.AND(Temp,In65278)
			Case Else
				InValue = 255
		End Select
	Else
		Select PLow
			Case 31 'Kempston joystick interface
				InValue = In31
			Case 255
				'TODO: Handle port FF, see http://www.worldofspectrum.org/faq/reference/48kreference.htm (Arkanoid uses this feature).
				InValue = 1
			Case Else
				InValue = 255
				Logger.logsWARN("Invalid IN port " & PLow)
		End Select
	End If
	Return InValue
End Sub
Sub HandleKeyEvent(e As Event)
	Dim KeyEvent As Reflector
	KeyEvent.Target = e	 ' e is a KeyEvent instance
	Dim KeyCode, EventType As String
	KeyCode = KeyEvent.RunMethod("getCode")
	EventType = KeyEvent.RunMethod("getEventType")
	If EventType = "KEY_PRESSED" Then 
		Select KeyCode
			Case "DIGIT1"
				In63486 = Bit.AND(In63486,254)
				In59390 = Bit.AND(In59390,254)
			Case "DIGIT2"
				In63486 = Bit.AND(In63486,253)
				In59390 = Bit.AND(In59390,253)
			Case "DIGIT3"
				In63486 = Bit.AND(In63486,251)
				In59390 = Bit.AND(In59390,251)
			Case "DIGIT4"
				In63486 = Bit.AND(In63486,247)
				In59390 = Bit.AND(In59390,247)
			Case "DIGIT5"
				In63486 = Bit.AND(In63486,239)
				In59390 = Bit.AND(In59390,239)
			Case "DIGIT0"
				In61438 = Bit.AND(In61438,254)
				In59390 = Bit.AND(In59390,254)
			Case "DIGIT9"
				In61438 = Bit.AND(In61438,253)
				In59390 = Bit.AND(In59390,253)
			Case "DIGIT8"
				In61438 = Bit.AND(In61438,251)
				In59390 = Bit.AND(In59390,251)
			Case "DIGIT7"
				In61438 = Bit.AND(In61438,247)
				In59390 = Bit.AND(In59390,247)
			Case "DIGIT6"
				In61438 = Bit.AND(In61438,239)
				In59390 = Bit.AND(In59390,239)
			Case "Q"
				In64510 = Bit.AND(In64510,254)
				In56318 = Bit.AND(In56318,254)
			Case "W"
				In64510 = Bit.AND(In64510,253)
				In56318 = Bit.AND(In56318,253)
			Case "E"
				In64510 = Bit.AND(In64510,251)
				In56318 = Bit.AND(In56318,251)
			Case "R"
				In64510 = Bit.AND(In64510,247)
				In56318 = Bit.AND(In56318,247)
			Case "T"
				In64510 = Bit.AND(In64510,239)
				In56318 = Bit.AND(In56318,239)
			Case "P"
				In57342 = Bit.AND(In57342,254)
				In56318 = Bit.AND(In56318,254)
			Case "O"
				In57342 = Bit.AND(In57342,253)
				In56318 = Bit.AND(In56318,253)
			Case "I"
				In57342 = Bit.AND(In57342,251)
				In56318 = Bit.AND(In56318,251)
			Case "U"
				In57342 = Bit.AND(In57342,247)
				In56318 = Bit.AND(In56318,247)
			Case "Y"
				In57342 = Bit.AND(In57342,239)
				In56318 = Bit.AND(In56318,239)
			Case "A"
				In65022 = Bit.AND(In65022,254)
				In59646 = Bit.AND(In59646,254)
			Case "S"
				In65022 = Bit.AND(In65022,253)
				In59646 = Bit.AND(In59646,253)
			Case "D"
				In65022 = Bit.AND(In65022,251)
				In59646 = Bit.AND(In59646,251)
			Case "F"
				In65022 = Bit.AND(In65022,247)
				In59646 = Bit.AND(In59646,247)
			Case "G"
				In65022 = Bit.AND(In65022,239)
				In59646 = Bit.AND(In59646,239)
			Case "ENTER"
				In49150 = Bit.AND(In49150,254)
			Case "L"
				In49150 = Bit.AND(In49150,253)
			Case "K"
				In49150 = Bit.AND(In49150,251)
			Case "J"
				In49150 = Bit.AND(In49150,247)
			Case "H"
				In49150 = Bit.AND(In49150,239)
			Case "SHIFT"
				In65278 = Bit.AND(In65278,254)
				In59646 = Bit.AND(In59646,254)
			Case "Z"
				In65278 = Bit.AND(In65278,253)
				In59646 = Bit.AND(In59646,253)
			Case "X"
				In65278 = Bit.AND(In65278,251)
				In59646 = Bit.AND(In59646,251)
			Case "C"
				In65278 = Bit.AND(In65278,247)
				In59646 = Bit.AND(In59646,247)
			Case "V"
				In65278 = Bit.AND(In65278,239)
				In59646 = Bit.AND(In59646,239)
			Case "SPACE"
				In32766 = Bit.AND(In32766,254)
				In31 = Bit.OR(In31,16) 'Space doubles as B1 for Kempston joystick interface
			Case "CONTROL"
				In32766 = Bit.AND(In32766,253)
			Case "M"
				In32766 = Bit.AND(In32766,251)
			Case "N"
				In32766 = Bit.AND(In32766,247)
			Case "B"
				In32766 = Bit.AND(In32766,239)
			Case "ESCAPE" 'Escape triggers extended keyboard mode (CAPS SHIFT + SYMBOL SHIFT)
				In32766 = Bit.AND(In32766,253)
				In65278 = Bit.AND(In65278,254)
			Case "BACK_SPACE"
				In65278 = Bit.AND(In65278,254)
				In61438 = Bit.AND(In61438,254)
			'Kempston joystick interface
			Case "UP"
				In31 = Bit.OR(In31,8)
			Case "DOWN"
				In31 = Bit.OR(In31,4)
			Case "LEFT"
				In31 = Bit.OR(In31,2)
			Case "RIGHT"
				In31 = Bit.OR(In31,1)
			Case "ALT"
				In31 = Bit.OR(In31,32) 'B2
		End Select
	End If
	If EventType = "KEY_RELEASED" Then 
		Select KeyCode
			Case "DIGIT1"
				In63486 = Bit.OR(In63486,1)
				In59390 = Bit.OR(In59390,1)
			Case "DIGIT2"
				In63486 = Bit.OR(In63486,2)
				In59390 = Bit.OR(In59390,2)
			Case "DIGIT3"
				In63486 = Bit.OR(In63486,4)
				In59390 = Bit.OR(In59390,4)
			Case "DIGIT4"
				In63486 = Bit.OR(In63486,8)
				In59390 = Bit.OR(In59390,8)
			Case "DIGIT5"
				In63486 = Bit.OR(In63486,16)
				In59390 = Bit.OR(In59390,16)
			Case "DIGIT0"
				In61438 = Bit.OR(In61438,1)
				In59390 = Bit.OR(In59390,1)
			Case "DIGIT9"
				In61438 = Bit.OR(In61438,2)
				In59390 = Bit.OR(In59390,2)
			Case "DIGIT8"
				In61438 = Bit.OR(In61438,4)
				In59390 = Bit.OR(In59390,4)
			Case "DIGIT7"
				In61438 = Bit.OR(In61438,8)
				In59390 = Bit.OR(In59390,8)
			Case "DIGIT6"
				In61438 = Bit.OR(In61438,16)
				In59390 = Bit.OR(In59390,16)
			Case "Q"
				In64510 = Bit.OR(In64510,1)
				In56318 = Bit.OR(In56318,1)
			Case "W"
				In64510 = Bit.OR(In64510,2)
				In56318 = Bit.OR(In56318,2)
			Case "E"
				In64510 = Bit.OR(In64510,4)
				In56318 = Bit.OR(In56318,4)
			Case "R"
				In64510 = Bit.OR(In64510,8)
				In56318 = Bit.OR(In56318,8)
			Case "T"
				In64510 = Bit.OR(In64510,16)
				In56318 = Bit.OR(In56318,16)
			Case "P"
				In57342 = Bit.OR(In57342,1)
				In56318 = Bit.OR(In56318,1)
			Case "O"
				In57342 = Bit.OR(In57342,2)
				In56318 = Bit.OR(In56318,2)
			Case "I"
				In57342 = Bit.OR(In57342,4)
				In56318 = Bit.OR(In56318,4)
			Case "U"
				In57342 = Bit.OR(In57342,8)
				In56318 = Bit.OR(In56318,8)
			Case "Y"
				In57342 = Bit.OR(In57342,16)
				In56318 = Bit.OR(In56318,16)
			Case "A"
				In65022 = Bit.OR(In65022,1)
				In59646 = Bit.OR(In59646,1)
			Case "S"
				In65022 = Bit.OR(In65022,2)
				In59646 = Bit.OR(In59646,2)
			Case "D"
				In65022 = Bit.OR(In65022,4)
				In59646 = Bit.OR(In59646,4)
			Case "F"
				In65022 = Bit.OR(In65022,8)
				In59646 = Bit.OR(In59646,8)
			Case "G"
				In65022 = Bit.OR(In65022,16)
				In59646 = Bit.OR(In59646,16)
			Case "ENTER"
				In49150 = Bit.OR(In49150,1)
			Case "L"
				In49150 = Bit.OR(In49150,2)
			Case "K"
				In49150 = Bit.OR(In49150,4)
			Case "J"
				In49150 = Bit.OR(In49150,8)
			Case "H"
				In49150 = Bit.OR(In49150,16)
			Case "SHIFT"
				In65278 = Bit.OR(In65278,1)
				In59646 = Bit.OR(In59646,1)
			Case "Z"
				In65278 = Bit.OR(In65278,2)
				In59646 = Bit.OR(In59646,2)
			Case "X"
				In65278 = Bit.OR(In65278,4)
				In59646 = Bit.OR(In59646,4)
			Case "C"
				In65278 = Bit.OR(In65278,8)
				In59646 = Bit.OR(In59646,8)
			Case "V"
				In65278 = Bit.OR(In65278,16)
				In59646 = Bit.OR(In59646,16)
			Case "SPACE"
				In32766 = Bit.OR(In32766,1)	
				In31 = Bit.AND(In31,239) 'Space doubles as B1 for Kempston joystick interface
			Case "CONTROL"
				In32766 = Bit.OR(In32766,2)
			Case "M"
				In32766 = Bit.OR(In32766,4)
			Case "N"
				In32766 = Bit.OR(In32766,8)
			Case "B"
				In32766 = Bit.OR(In32766,16)
			Case "ESCAPE" 'Escape triggers extended keyboard mode (CAPS SHIFT + SYMBOL SHIFT)
				In32766 = Bit.OR(In32766,2)
				In65278 = Bit.OR(In65278,1)
			Case "BACK_SPACE"
				In65278 = Bit.OR(In65278,1)
				In61438 = Bit.OR(In61438,1)
			'Kempston joystick interface
			Case "UP"
				In31 = Bit.AND(In31,247)
			Case "DOWN"
				In31 = Bit.AND(In31,251)
			Case "LEFT"
				In31 = Bit.AND(In31,253)
			Case "RIGHT"
				In31 = Bit.AND(In31,254)
			Case "ALT"
				In31 = Bit.AND(In31,223) 'B2
		End Select
	End If
	If EventType = "CHAR_UNDEFINED" Then
		Logger.logsWARN("Undefined character")
	End If
	e.Consume ' Consume the event to stop it bubbling up the scene graph 	
End Sub

