﻿Type=StaticCode
Version=2.5
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'This module handles the signals from the Z80 OUT port, including border color and sound.
'Border color is just set here in a variable which is handled in the Display routines.
'Sound is a bit tricky. The Spectrum ROM has a "Beeper" routine, which sends alternating on/off signals to
'OUT port 254. Everytime this happens, the speaker gives a pulse, which produces a sound (a beep) with the
'frequency of the on/off signals. Of course timing is essential here so the Spectrum switches off CPU interrupts
'while running the beeper routine. The challenge is to emulate the timing of the sound since in the emulator,
'timing is only checked 50 times a second, when the screen is updated. The CPU operations that take place during
'a 1/50 second are processed faster than on the Spectrum, so the emulator has to wait to the next 1/50
'second to run the next batch of operations.
'The solution - which in practice doesn't work 100 % - is to use information about the exact length of
'every operation in CPU cycles (T-States) to count the CPU cycles between every on/off signal on port 254.
'This information is then used to feed an AudioTrack control with an array of bytes, creating a square wave.
'The problem is that after the signal has been sent to the AudioTrack control, some sort of residual artefacts
'keeps repeating indefinitely. I have solved this in the main program loop by flushing the AudioTrack buffer if
'nothing has been sent to it i a while. This works decently. Another problem is that the signal is broken on some
'computers - it gets chopped up. I have detuned the signal which seems to help a bit.
'So, in conlusion: Sound is not working very well.
Sub Process_Globals
	Private fx As JFX
	Public BorderColor As Int
	Public Beeper As AudioTrack
	Private Const SampleRate As Int = 44000
	Private Const SampleSize As Int = 8
	Private Const Channels As Int = 1
	Private Bit4 As Int
	Private BeepStart, BeepEnd, BeepLength As Int
	Private Samples As Int
	Public LastBeepClick As Long
End Sub
Public Sub Initialize
	LastBeepClick = DateTime.Now
	Beeper.Initialize(SampleRate,SampleSize,Channels)
	Bit4 = 16
	BeepLength = 0
	BeepStart = 0
	Beeper.Start
End Sub
Sub Process(P1 As Int,P2 As Int)
'	TODO: Must handle both mic and speaker bits.
	If P1 = 254 Then
		BorderColor = Bit.AND(P2,7)
		If Bit.AND(P2,16) <> Bit4  Then 'Handle Beep
			Bit4 = Bit.AND(P2,16)
			BeepEnd = Z80.T_States + Z80.Total_T_States
			BeepLength = BeepEnd - BeepStart
			Samples = BeepLength / 70 'Slightly detuned. Should be 79, but this improves sound quality.
			BeepStart = BeepEnd
			If Samples < 800 Then 'Longer periods won't generate sound anyway.
				If Bit4 = 0 Then
					For i = 1 To Samples
						Beeper.Write(Array As Byte(34),0,1)
					Next
				Else
					For i = 1 To Samples
						Beeper.Write(Array As Byte(0),0,1)
					Next
				End If
				LastBeepClick = DateTime.Now
			End If
		End If
	End If
End Sub