﻿Type=StaticCode
Version=2.5
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'This module is the Z80 emulator.
Private Sub Process_Globals
	Private fx As JFX
	'Z80 Registers
	Public a,b,c,d,e,f,h,l,i1,x,i2,y,i,r As Int
	Public aPrime,bPrime,cPrime,dPrime,ePrime,fPrime,hPrime,lPrime As Int
	Public CarryFlag,ZeroFlag,Parity_OverflowFlag,SignFlag,SubstractFlag,HalfCarryFlag As Int
	Public CarryFlagPrime,ZeroFlagPrime,Parity_OverflowFlagPrime,SignFlagPrime,SubstractFlagPrime,HalfCarryFlagPrime As Int
	'Stack pointer and Program counter
	Public SP,PC,PC_Old As Int
	'Variables to keep track of CPU cycles (T-States) and number of processed instructions
	Public T, T_States, Total_T_States,OpCodeCounter As Int
	'Instruction data
	Public OpCode As Int
	Public OpName, OpName_1,Opname_2 As String 'Current instruction name, and the two previous ones. Used for interpreting operations with one or two prefixes.
	Private OpLength As Int 'Number of bytes used by a instruction
	'Interrupt variables
	Public InterruptMode As Int
	Public InterruptEnabled As Boolean
	Public InterruptStarted As Boolean
	Public Logger As tsLogger
	'Debug stuff
	Private PCHistory(10) As Int
	Public AllowInterrupt As Boolean
	Public Break As Boolean
	'A general temp variable
	Private Temp As Int
End Sub
Public Sub Initialize
	ResetControlVariables
	ResetRegisters
	ResetFlags
	InterruptMode = 0
	InterruptEnabled = False
	AllowInterrupt = True
	'Logger
	Logger.Configuration.LogFilename = "SP48.log"
	Logger.Configuration.IsConsoleEnable = True
	Logger.Configuration.IsFileWriterEnable = False
	Logger.Initialize
	Logger.toLevel(0) 'Level 0 = OFF, 1 = TRACE, 2 = DEBUG, 3 = INFO, 4 = WARN, 5 = ERROR, 6 = FATAL
End Sub
Public Sub Reset
	ResetControlVariables
	ResetRegisters
	ResetFlags
	InterruptMode = 0
	InterruptEnabled = False
	AllowInterrupt = True
End Sub
Private Sub ResetControlVariables
	PC = 0
	T = 0
	OpName = ""
	OpName_1 = ""
	Opname_2 = ""
	OpCodeCounter = 0
	T_States = 0
	Total_T_States = 0
	Break = False
End Sub
Public Sub InitializeSnapshot
	RefreshFlags
	OpName = ""
	OpName_1 = ""
	Opname_2 = ""
	OpCodeCounter = 0
	T_States = 0
	Total_T_States = 0
	PC = Pop16
End Sub
Private Sub RefreshFlags
	SignFlag = Bit.AND(f,128) / 128
	ZeroFlag = Bit.AND(f,64) / 64
	HalfCarryFlag = Bit.AND(f,16) / 16
	Parity_OverflowFlag = Bit.AND(f,4) / 4
	SubstractFlag = Bit.AND(f,2) / 2
	CarryFlag = Bit.AND(f,1)
	SignFlagPrime = Bit.AND(fPrime,128) / 128
	ZeroFlagPrime = Bit.AND(fPrime,64) / 64
	HalfCarryFlagPrime = Bit.AND(fPrime,16) / 16
	Parity_OverflowFlagPrime = Bit.AND(fPrime,4) / 4
	SubstractFlagPrime = Bit.AND(fPrime,2) / 2
	CarryFlagPrime = Bit.AND(fPrime,1)
End Sub

'Run Z80 instructions during 69888 CPU cycles (T-States).
'Break at BreakPoint. Set BreakPoint = -1, to avoid break.
Public Sub Run69888TStates(BreakPoint As Int)
	T_States = 0 'Reset the CPU cycle counter.
	InterruptStarted = False 'No interrupt has occured yet.
	Do While T_States < 69884
		If InterruptStarted = False Then Interrupt
		InterpretInstruction
		T_States = T_States + T
		If PC = BreakPoint Then 'Check if the program counter has arrived at the breakpoint.
'			Main.BreakFlag = True
'			Main.MainTimer.Enabled = False
'			Main.DebugWindow.TogglePauseButton
			Break = True
			Exit 'Stop everything
		End If
	Loop
	Total_T_States = Total_T_States + T_States 'Keep track of total number of CPU cycles for the Beeper timing.
End Sub
'This routine interprets and processes on Z80 instruction at a time.
'The instructions are fetched from the memory array, which is managed
'in the Memory module. Every instruction has a OpLength variable which
'is added to the program counter (PC) after the instruction has been
'processed so that the program counter can find the next instruction
'in the array. There is also the T variable which holds the number of
'CPU cycles (T-states) that each instruction needs to be processed.
'There actual code for the instructions is found in separate routines
'with case statements for each instruction. There are routines for
'regular opcodes 0x00-0xFF and for prefixed opcodes. So, for example
'when the regular opcode 0xCB is encountered, the program checks the
'CB routine for the next opcode in the array.
'Z80 uses little Endian; least significant bit first in memory.
'However, double registers have the least significant bit last.
'Multiply most significant bit with 256.
Public Sub InterpretInstruction
'Store the latest PC values In Debug mode.
	If Logger.IsDebugEnabled Then
		PCHistory(9) = PCHistory(8)
		PCHistory(8) = PCHistory(7)
		PCHistory(7) = PCHistory(6)
		PCHistory(6) = PCHistory(5)
		PCHistory(5) = PCHistory(4)
		PCHistory(4) = PCHistory(3)
		PCHistory(3) = PCHistory(2)
		PCHistory(2) = PCHistory(1)
		PCHistory(1) = PCHistory(0)
		PCHistory(0) = PC
	End If
	OpCode = Memory.Read(PC)
	PC_Old = PC
	T = 0
	Select OpName
		Case "CB"
			CBOpCodes
		Case "ED"
			EDOpCodes
		Case "DD"
			DDOpCodes 'IX
		Case "FD"
			FDOpCodes 'IY
		Case Else
			RegularOpCodes
	End Select
	Opname_2 = OpName_1
	OpName_1 = OpName
	'UpdateF_registers
	f = 128 * SignFlag + 64 * ZeroFlag + 16 * HalfCarryFlag + 4 * Parity_OverflowFlag + 2 * SubstractFlag + CarryFlag
	fPrime = 128 * SignFlagPrime + 64 * ZeroFlagPrime + 16 * HalfCarryFlagPrime + 4 * Parity_OverflowFlagPrime + 2 * SubstractFlagPrime + CarryFlagPrime
	PC = PC + OpLength
	OpCodeCounter = OpCodeCounter + 1
End Sub
'Check if an interrupt is allowed. Then handle the different interrupt modes.
'The result of this routine (when an interrupt has been handled) is that the program counter (PC)
'has been redirected to the interrupt code block and the Z80 will continue processing instructions
'there until a return instruction returns processing to where it left off. The flag InterruptStarted
'is set to True to make sure that only one interrupt occurs at the designated frequency.
Public Sub Interrupt
	If InterruptEnabled = True AND AllowInterrupt = True  AND OpName <> "FD" AND OpName <> "CB" AND OpName <> "ED" AND OpName <> "DD" Then
		Select InterruptMode
			Case 0
				'TODO: Find out how IM 0 works. Until then, pretend that it is the same as IM 1.
				If OpName = "HALT" Then PC = PC + 1 'Since HALT waits for an interrupt, count is set to 0 at HALT
				Push16(PC)
				PC = 56
				InterruptStarted = True
			Case 1
				'In IM 1, the processor simply executes an RST 38 instruction if an interrupt is requested.
				'This is the mode the Spectrum is initalised to. In this mode, the processor takes 13 T states
				'to reach 0x0038: a 7 T state M1 cycle to acknowledge the interrupt and decrement SP,
				'a 3 T state M2 cycle to write the high Byte of PC onto the stack and decrement SP again,
				'and finally a 3 T state M3 cycle To write the low Byte onto the stack and to set PC To 0x0038.
				If OpName = "HALT" Then PC = PC + 1 'Since HALT waits for an interrupt, count is set to 0 at HALT
				Push16(PC)
				PC = 56
				InterruptStarted = True
			Case 2
				'The other mode that is commonly used on the Spectrum is IM 2. If an interrupt is requested,
				'the processor first builds a 16-bit address by combining the I register (as the high byte) 
				'with whatever the interrupting device places on the data bus. The processor then fetches the 
				'16-bit address at this interrupt table entry, and finally calls the subroutine at that address.
				'Rodnay Zaks in his book 'Programming the Z80' states that only even bytes are allowed as low index 
				'byte, but that isn't true. The normal Spectrum contains no hardware to place a byte on the bus, 
				'and the bus will therefore always read FF (because the ULA also doesn't read the screen if it 
				'generates an interrupt), so the resulting index address is 256*I+255. However, some not-so-neat 
				'hardware devices put things on the data bus when they shouldn't, so later programs didn't assume 
				'the low index byte was FF. These programs contain a 257 byte table of equal bytes starting at 256*I, 
				'and the interrupt routine is placed at an address that is a multiple of 257. A useful, but not so 
				'much used trick on the Spectrum, is to make the table contain FF's (or use the ROM for this) and put 
				'a byte 18 hex, the opcode for JR, at FFFF. The first byte of the ROM is a DI, F3 hex, so the JR will 
				'jump to FFF4, where a long JP to the actual interrupt routine is put. In IM 2, it takes 19 cycles to 
				'get to the interrupt routine:
				If OpName = "HALT" Then PC = PC + 1 'Since HALT waits for an interrupt, count is set to 0 at HALT
				Push16(PC)
				Logger.logsTrace("Calling interrupt from PC = " & PC)
				Dim Index As Int
				Index = i * 256 + 255
				PC = Memory.Read16(Index)
				InterruptStarted = True
		End Select
	End If
End Sub
Private Sub RegularOpCodes
	IncR 'R is incremented once for every regular opcode.
	Select OpCode
		Case 0
			OpName = "NOP"
			OpLength = 1
			T = 4
		Case 1
			OpName = "LD BC,NN"
			b = Nhigh
			c = N
			OpLength = 3
			T = 10
		Case 2
			OpName = "LD (BC),A"
			Memory.Write(BC,a)
			OpLength = 1
			T = 7
 		Case 3
			OpName = "INC BC"
			Temp = INC16(BC)
			b = HighByte(Temp)
			c = LowByte(Temp)
			OpLength = 1
			T = 6
 	 	Case 4
			OpName = "INC B"
			b = INC(b)
			OpLength = 1
			T = 4
		Case 5
			OpName = "DEC B"
			b = DEC(b)
			OpLength = 1
			T = 4
 	 	Case 6
			OpName = "LD B,N"
			b = N
			OpLength = 2
			T = 7
 	 	Case 7
			OpName = "RLCA" 
			'RLCA rotates the A register To the left one place. The 7th Bit Is
			'put back into the 0 position.  The 7th Bit also goes To the carry
			'flag.
			Temp = Bit.AND(a,128) / 128 'Extract bit 7
			a = Bit.AND(Bit.ShiftLeft(a,1),0xFF) 'Shift A left and drop anything over 0xFF
			a = a + Temp 'Set bit 0 to former bit 7
			CarryFlag = Temp 'Carry = former bit 7
			SubstractFlag = 0
			HalfCarryFlag = 0
			OpLength = 1
			T = 4
 		Case 8
			OpName = "EX AF,AF'"
			Temp = a
			a = aPrime
			aPrime = Temp
			Temp = f
			f = fPrime
			fPrime = Temp
			Temp = SignFlag
			SignFlag = SignFlagPrime
			SignFlagPrime = Temp
			Temp = ZeroFlag
			ZeroFlag = ZeroFlagPrime
			ZeroFlagPrime = Temp
			Temp = HalfCarryFlag
			HalfCarryFlag = HalfCarryFlagPrime
			HalfCarryFlagPrime = Temp
			Temp = Parity_OverflowFlag
			Parity_OverflowFlag = Parity_OverflowFlagPrime
			Parity_OverflowFlagPrime = Temp
			Temp = SubstractFlag
			SubstractFlag = SubstractFlagPrime
			SubstractFlagPrime = Temp
			Temp = CarryFlag
			CarryFlag = CarryFlagPrime
			CarryFlagPrime = Temp
			OpLength = 1
			T = 4
 		Case 9
			OpName = "ADD HL,BC"
			Temp = ADD16(HL, BC)
			h = HighByte(Temp)
			l = LowByte(Temp)
			OpLength = 1
			T = 11
 		Case 10
			OpName = "LD A,(BC)"
			a = Memory.Read(BC)
			OpLength = 1
			T = 7
 		Case 11
			OpName = "DEC BC"
			Temp = DEC16(BC)
			b = HighByte(Temp)
			c = LowByte(Temp)
			OpLength = 1
			T = 6
 		Case 12
			OpName = "INC C"
			c = INC(c)
			OpLength = 1
			T = 4
 		Case 13
			OpName = "DEC C"
			c = DEC(c)
			OpLength = 1
			T = 4
 		Case 14
			OpName = "LD C,N"
			c = N
			OpLength = 2
			T = 7
 		Case 15
			OpName = "RRCA"
			'The  register is shifted right by one,  and the 0 bit goes to the
			'carry flag AND To the 7th Bit.  Flags apart from the carry are As
			'For RLCA.
			Temp = Bit.AND(a,1) 'Extract bit 0
			a = Bit.ShiftRight(a,1) + 128 * Temp 'Shift right and set bit 7 to former bit 0
			CarryFlag = Temp 'Set Carry flag
			SubstractFlag = 0
			HalfCarryFlag = 0
			OpLength = 1
			T = 4
 		Case 16
			OpName = "DJNZ DIS"
			b = Bit.AND(b - 1,255) 'Drop any carry bit
			If b > 0 Then
				Dim Dis As Byte
				Dis = Memory.Read(PC + 1)
				PC = PC + Dis
				T = 8
			End If
			OpLength = 2
			T = T + 4
 		Case 17
			OpName = "LD DE,NN"
			d = Nhigh
			e = N
			OpLength = 3
			T = 10
 		Case 18
			OpName = "LD (DE),A"
			Memory.Write(DE,a)
			OpLength = 1
			T = 7
 		Case 19
			OpName = "INC DE"
			Temp = INC16(DE)
			d = HighByte(Temp)
			e = LowByte(Temp)
			OpLength = 1
			T = 6
 		Case 20
			OpName = "INC D"
			d = INC(d)
			OpLength = 1
			T = 4
 		Case 21
			OpName = "DEC D"
			d = DEC(d)
			OpLength = 1
			T = 4
 		Case 22
			OpName = "LD D,N"
			d = N
			OpLength = 2
			T = 7
 		Case 23
			OpName = "RLA"
			'The RLA instruction rotates left through the carry flag.
			' --- C <---  bbbbbbbb  <--
			'|                         |
			'------------- > ----------
			'The  bits in the register are all rotated left,  the 7th Bit goes
			'to the carry flag AND the carry flag goes To Bit 0.
			Temp = Bit.AND(a,128) / 128 'Extract bit 7
			a = Bit.ShiftLeft(a,1) 'Shift A left
			a = Bit.AND(255,a) + CarryFlag 'Drop anything above 255 and set bit 0 to Carry
			CarryFlag = Temp 'Carry = former bit 7
			SubstractFlag = 0
			HalfCarryFlag = 0
			OpLength = 1
			T = 4
 		Case 24
			OpName = "JR DIS"
			Dim Dis As Byte
			Dis = Memory.Read(PC + 1)
			PC = PC + Dis
			OpLength = 2
			T = 12
 		Case 25
			OpName = "ADD HL,DE"
			Temp = ADD16(HL, DE)
			h = HighByte(Temp)
			l = LowByte(Temp)
			OpLength = 1
			T = 11
 		Case 26
			OpName = "LD A,(DE)"
			a = Memory.Read(DE)
			OpLength = 1
			T = 7
 		Case 27
			OpName = "DEC DE"
			Temp = DEC16(DE)
			d = HighByte(Temp)
			e = LowByte(Temp)
			OpLength = 1
			T = 6
 		Case 28
			OpName = "INC E"
			e = INC(e)
			OpLength = 1
			T = 4
 		Case 29
			OpName = "DEC E"
			e = DEC(e)
			OpLength = 1
			T = 4
 		Case 30
			OpName = "LD E,N"
			e = N
			OpLength = 2
			T = 7
 		Case 31
			OpName = "RRA"
			'The RRA rotates right through the  carry flag
			'The register is shifted right by one, the 0 Bit goes to the carry
			'flag,  and  the carry flag goes To Bit 7.  Flags apart  from  the
			'carry flag are As For RLCA.
			Temp = Bit.AND(a,1) 'Extract bit 0
			a = Bit.ShiftRight(a,1) + 128 * CarryFlag 'Shift right and set bit 7 to Carry
			CarryFlag = Temp 'Set Carry flag
			SubstractFlag = 0
			HalfCarryFlag = 0
			OpLength = 1
			T = 4
 		Case 32
			OpName = "JR NZ, DIS"
			If ZeroFlag = 0 Then
				Dim Dis As Byte
				Dis = Memory.Read(PC + 1)
				PC = PC + Dis
				T = 12
			Else 
				T = 7
			End If
			OpLength = 2
 		Case 33
			OpName = "LD HL,NN"
			h = Nhigh
			l = N
			OpLength = 3
			T = 10
 		Case 34
			OpName = "LD (NN),HL"
			Memory.Write(NN + 1,h)
			Memory.Write(NN,l)
			OpLength = 3
			T = 16
 		Case 35
			OpName = "INC HL"
			Temp = INC16(HL)
			h = HighByte(Temp)
			l = LowByte(Temp)
			OpLength = 1
			T = 6
 		Case 36
			OpName = "INC H"
			h = INC(h)
			OpLength = 1
			T = 4
 		Case 37
			OpName = "DEC H"
			h = DEC(h)
			OpLength = 1
			T = 4
 		Case 38
			OpName = "LD H,N"
			h = N
			OpLength = 2
			T = 7
 		Case 39
			OpName = "DAA"
			'According to http://www.worldofspectrum.org/faq/reference/z80reference.htm#DAA
			Dim Temp As Int
			Dim CorrectionFactor As Byte
			CorrectionFactor = 0
			Temp = a
			If Bit.AND(a,0xF0) > 0x90 OR CarryFlag = 1 Then
				CorrectionFactor = 0x60
				CarryFlag = 1
			End If
			If Bit.AND(a,0x0F) > 0x09 OR HalfCarryFlag = 1 Then
				CorrectionFactor = CorrectionFactor + 0x06
			End If
			If SubstractFlag = 0 Then
				a = Bit.AND(a + CorrectionFactor,0xFF)
			Else
				a = Bit.AND(a - CorrectionFactor,0xFF)
			End If
			Parity_OverflowFlag = Parity(a)
			HalfCarryFlag = Bit.AND(0x10,Bit.Xor(Temp,a)) / 0x10
			SignFlag = Bit.AND(a,128)/128 'Set Sign flag if bit 7 is set
			OpLength = 1
			T = 4
 		Case 40
			OpName = "JR Z,DIS"
			If ZeroFlag = 1 Then
				Dim Dis As Byte
				Dis = Memory.Read(PC + 1)
				PC = PC + Dis
				T = 12
			Else 
				T = 7
			End If
			OpLength = 2
 		Case 41
			OpName = "ADD HL,HL"
			Temp = ADD16(HL,HL)
			h = HighByte(Temp)
			l = LowByte(Temp)
			OpLength = 1
			T = 4
 		Case 42
			OpName = "LD HL,(NN)"
			h = Memory.Read(NN + 1)
			l = Memory.Read(NN)
			OpLength = 3
			T = 4
 		Case 43
			OpName = "DEC HL"
			Temp = DEC16(HL)
			h = HighByte(Temp)
			l = LowByte(Temp)
			OpLength = 1
			T = 4
 		Case 44
			OpName = "INC L"
			l = INC(l)
			OpLength = 1
			T = 4
 		Case 45
			OpName = "DEC L"
			l = DEC(l)
			OpLength = 1
			T = 4
 		Case 46
			OpName = "LD L,N"
			L = N
			OpLength = 2
			T = 7
 		Case 47
			OpName = "CPL"
			a = Bit.AND(Bit.Not(a),0xFF)
			SubstractFlag = 1
			HalfCarryFlag = 1
			OpLength = 1
			T = 4
 		Case 48
			OpName = "JR NC,DIS"
			If CarryFlag = 0 Then
				Dim Dis As Byte
				Dis = N
				PC = PC + Dis
				T = 12
			Else 
				T = 7
			End If
			OpLength = 2
 		Case 49
			OpName = "LD SP,NN"
			SP = NN
			OpLength = 3
			T = 10
 		Case 50
			OpName = "LD (NN),A"
			Memory.Write(NN,a)
			OpLength = 3
			T = 13
 		Case 51
			OpName = "INC SP"
			SP = INC16(SP)
			OpLength = 1
			T = 6
 		Case 52
			OpName = "INC (HL)"
			Memory.Write(HL,INC(Memory.Read(HL)))
			OpLength = 1
			T = 11
 		Case 53
			OpName = "DEC (HL)"
			Memory.Write(HL,DEC(Memory.Read(HL)))
			OpLength = 1
			T = 11
 		Case 54
			OpName = "LD (HL),N"
			Memory.Write(HL,N)
			OpLength = 2
			T = 10
 		Case 55
			OpName = "SCF"
			CarryFlag = 1
			HalfCarryFlag = 0
			SubstractFlag = 0
			OpLength = 1
			T = 4
 		Case 56
			OpName = "JR C,DIS"
			If CarryFlag = 1 Then
				Dim Dis As Byte
				Dis = N
				PC = PC + Dis
				T = 12
			Else 
				T = 7
			End If
			OpLength = 2
 		Case 57
			OpName = "ADD HL,SP"
			Temp = ADD16(HL, SP)
			h = HighByte(Temp)
			l = LowByte(Temp)
			OpLength = 1
			T = 11
 		Case 58
			OpName = "LD A,(NN)"
			a = Memory.Read(NN)
			OpLength = 3
			T = 13
 		Case 59
			OpName = "DEC SP"
			SP = DEC16(SP)
			OpLength = 1
			T = 6
 		Case 60
			OpName = "INC A"
			a = INC(a)
			OpLength = 1
			T = 4
 		Case 61
			OpName = "DEC A"
			a = DEC(a)
			OpLength = 1
			T = 4
 		Case 62
			OpName = "LD A,N"
			a = N
			OpLength = 2
			T = 7
 		Case 63 'OK
			OpName = "CCF"
			HalfCarryFlag = CarryFlag
			If CarryFlag = 0 Then CarryFlag = 1 Else CarryFlag = 0
			SubstractFlag = 0
			OpLength = 1
			T = 4
 		Case 64
			OpName = "LD B,B"
			b = b
			OpLength = 1
			T = 4
 		Case 65
			OpName = "LD B,C"
			b = c
			OpLength = 1
			T = 4
 		Case 66
			OpName = "LD B,D"
			b = d
			OpLength = 1
			T = 4
 		Case 67
			OpName = "LD B,E"
			b = e
			OpLength = 1
			T = 4
 		Case 68
			OpName = "LD B,H"
			b = h
			OpLength = 1
			T = 4
 		Case 69
			OpName = "LD B,L"
			b = l
			OpLength = 1
			T = 4
 		Case 70
			OpName = "LD B,(HL)"
			b = Memory.Read(HL)
			OpLength = 1
			T = 7
 		Case 71
			OpName = "LD B,A"
			b = a
			OpLength = 1
			T = 4
 		Case 72
			OpName = "LD C,B"
			c = b
			OpLength = 1
			T = 4
 		Case 73
			OpName = "LD C,C"
			c = c
			OpLength = 1
			T = 4
 		Case 74
			OpName = "LD C,D"
			c = d
			OpLength = 1
			T = 4
 		Case 75
			OpName = "LD C,E"
			c = e
			OpLength = 1
			T = 4
 		Case 76
			OpName = "LD C,H"
			c = h
			OpLength = 1
			T = 4
 		Case 77
			OpName = "LD C,L"
			c = l
			OpLength = 1
			T = 4
 		Case 78
			OpName = "LD C,(HL)"
			c = Memory.Read(HL)
			OpLength = 1
			T = 7
 		Case 79
			OpName = "LD C,A"
			c = a
			OpLength = 1
			T = 4
 		Case 80
			OpName = "LD D,B"
			d = b
			OpLength = 1
			T = 4
 		Case 81
			OpName = "LD D,C"
			d = c
			OpLength = 1
			T = 4
 		Case 82
			OpName = "LD D,D"
			d = d
			OpLength = 1
			T = 4
 		Case 83
			OpName = "LD D,E"
			d = e
			OpLength = 1
			T = 4
 		Case 84
			OpName = "LD D,H"
			d = h
			OpLength = 1
			T = 4
 		Case 85
			OpName = "LD D,L"
			d = l
			OpLength = 1
			T = 4
 		Case 86
			OpName = "LD D,(HL)"
			d = Memory.Read(HL)
			OpLength = 1
			T = 7
 		Case 87
			OpName = "LD D,A"
			d = a
			OpLength = 1
			T = 4
 		Case 88
			OpName = "LD E,B"
			e = b
			OpLength = 1
			T = 4
 		Case 89
			OpName = "LD E,C"
			e = c
			OpLength = 1
			T = 4
 		Case 90
			OpName = "LD E,D"
			e = d
			OpLength = 1
			T = 4
 		Case 91
			OpName = "LD E,E"
			e = e
			OpLength = 1
			T = 4
 		Case 92
			OpName = "LD E,H"
			e = h
			OpLength = 1
			T = 4
 		Case 93
			OpName = "LD E,L"
			e = l
			OpLength = 1
			T = 4
 		Case 94
			OpName = "LD E,(HL)"
			e = Memory.Read(HL)
			OpLength = 1
			T = 7
 		Case 95
			OpName = "LD E,A"
			e = a
			OpLength = 1
			T = 4
 		Case 96
			OpName = "LD H,B"
			h = b
			OpLength = 1
			T = 4
 		Case 97
			OpName = "LD H,C"
			h = c
			OpLength = 1
			T = 4
 		Case 98
			OpName = "LD H,D"
			h = d
			OpLength = 1
			T = 4
 		Case 99
			OpName = "LD H,E"
			h = e
			OpLength = 1
			T = 4
 		Case 100
			OpName = "LD H,H"
			h = h
			OpLength = 1
			T = 4
 		Case 101
			OpName = "LD H,L"
			h = l
			OpLength = 1
			T = 4
 		Case 102
			OpName = "LD H,(HL)"
			h = Memory.Read(HL)
			OpLength = 1
			T = 7
 		Case 103
			OpName = "LD H,A"
			h = a
			OpLength = 1
			T = 4
 		Case 104
			OpName = "LD L,B"
			l = b
			OpLength = 1
			T = 4
 		Case 105
			OpName = "LD L,C"
			l = c
			OpLength = 1
			T = 4
 		Case 106
			OpName = "LD L,D"
			l = d
			OpLength = 1
			T = 4
 		Case 107
			OpName = "LD L,E"
			l = e
			OpLength = 1
			T = 4
 		Case 108
			OpName = "LD L,H"
			l = h
			OpLength = 1
			T = 4
 		Case 109
			OpName = "LD L,L"
			l = l
			OpLength = 1
			T = 4
 		Case 110
			OpName = "LD L,(HL)"
			l = Memory.Read(HL)
			OpLength = 1
			T = 7
 		Case 111
			OpName = "LD L,A"
			l = a
			OpLength = 1
			T = 4
 		Case 112
			OpName = "LD (HL),B"
			Memory.Write(HL,b)
			OpLength = 1
			T = 7
 		Case 113
			OpName = "LD (HL),C"
			Memory.Write(HL,c)
			OpLength = 1
			T = 7
 		Case 114
			OpName = "LD (HL),D"
			Memory.Write(HL,d)
			OpLength = 1
			T = 7
 		Case 115
			OpName = "LD (HL),E"
			Memory.Write(HL,e)
			OpLength = 1
			T = 7
 		Case 116
			OpName = "LD (HL),H"
			Memory.Write(HL,h)
			OpLength = 1
			T = 7
 		Case 117
			OpName = "LD (HL),L"
			Memory.Write(HL,l)
			OpLength = 1
			T = 7
 		Case 118
			OpName = "HALT"
			'The HALT instruction suspends CPU operation until a interrupt or reset is received.
			OpLength = 0
			T = 4
 		Case 119
			OpName = "LD (HL),A"
			Memory.Write(HL,a)
			OpLength = 1
			T = 7
 		Case 120
			OpName = "LD A,B"
			a = b
			OpLength = 1
			T = 4
 		Case 121
			OpName = "LD A,C"
			a = c
			OpLength = 1
			T = 4
 		Case 122
			OpName = "LD A,D"
			a = d
			OpLength = 1
			T = 4
 		Case 123
			OpName = "LD A,E"
			a = e
			OpLength = 1
			T = 4
 		Case 124
			OpName = "LD A,H"
			a = h
			OpLength = 1
			T = 4
 		Case 125
			OpName = "LD A,L"
			a = l
			OpLength = 1
			T = 4
 		Case 126
			OpName = "LD A,(HL)"
			a = Memory.Read(HL)
			OpLength = 1
			T = 7
 		Case 127
			OpName = "LD A,A"
			a = a
			OpLength = 1
			T = 4
 		Case 128
			OpName = "ADD A,B"
			a = ADD(a,b)
			OpLength = 1
			T = 4
 		Case 129
			OpName = "ADD A,C"
			a = ADD(a,c)
			OpLength = 1
			T = 4
 		Case 130
			OpName = "ADD A,D"
			a = ADD(a,d)
			OpLength = 1
			T = 4
 		Case 131
			OpName = "ADD A,E"
			a = ADD(a,e)
			OpLength = 1
			T = 4
 		Case 132
			OpName = "ADD A,H"
			a = ADD(a,h)
			OpLength = 1
			T = 4
 		Case 133
			OpName = "ADD A,L"
			a = ADD(a,l)
			OpLength = 1
			T = 4
 		Case 134
			OpName = "ADD A,(HL)"
			a = ADD(a,Memory.Read(HL))
			OpLength = 1
			T = 7
 		Case 135
			OpName = "ADD A,A"
			a = ADD(a,a)
			OpLength = 1
			T = 4
 		Case 136
			OpName = "ADC A,B"
			a = ADC(a,b)
			OpLength = 1
			T = 4
 		Case 137
			OpName = "ADC A,C"
			a = ADC(a,c)
			OpLength = 1
			T = 4
 		Case 138
			OpName = "ADC A,D"
			a = ADC(a,d)
			OpLength = 1
			T = 4
 		Case 139
			OpName = "ADC A,E"
			a = ADC(a,e)
			OpLength = 1
			T = 4
 		Case 140
			OpName = "ADC A,H"
			a = ADC(a,h)
			OpLength = 1
			T = 4
 		Case 141
			OpName = "ADC A,L"
			a = ADC(a,l)
			OpLength = 1
			T = 4
 		Case 142
			OpName = "ADC A,(HL)"
			a = ADC(a,Memory.Read(HL))
			OpLength = 1
			T = 7
 		Case 143
			OpName = "ADC A,A"
			a = ADC(a,a)
			OpLength = 1
			T = 4
 		Case 144
			OpName = "SUB B"
			a = SUB_(a,b)
			OpLength = 1
			T = 4
 		Case 145
			OpName = "SUB C"
			a = SUB_(a,c)
			OpLength = 1
			T = 4
 		Case 146
			OpName = "SUB D"
			a = SUB_(a,d)				
			OpLength = 1
			T = 4
 		Case 147
			OpName = "SUB E"
			a = SUB_(a,e)				
			OpLength = 1
			T = 4
 		Case 148
			OpName = "SUB H"
			a = SUB_(a,h)				
			OpLength = 1
			T = 4
 		Case 149
			OpName = "SUB L"
			a = SUB_(a,l)				
			OpLength = 1
			T = 4
 		Case 150
			OpName = "SUB (HL)"
			a = SUB_(a,Memory.Read(HL))
			OpLength = 1
			T = 7
 		Case 151
			OpName = "SUB A"
			a = SUB_(a,a)
			OpLength = 1
			T = 4
 		Case 152
			OpName = "SBC A,B"
			a = SBC(a,b)
			OpLength = 1
			T = 4
 		Case 153
			OpName = "SBC A,C"
			a = SBC(a,c)
			OpLength = 1
			T = 4
 		Case 154
			OpName = "SBC A,D"
			a = SBC(a,d)				
			OpLength = 1
			T = 4
 		Case 155
			OpName = "SBC A,E"
			a = SBC(a,e)				
			OpLength = 1
			T = 4
 		Case 156
			OpName = "SBC A,H"
			a = SBC(a,h)				
			OpLength = 1
			T = 4
 		Case 157
			OpName = "SBC A,L"
			a = SBC(a,l)				
			OpLength = 1
			T = 4
 		Case 158
			OpName = "SBC A,(HL)"
			a = SBC(a,Memory.Read(HL))
			OpLength = 1
			T = 7
 		Case 159
			OpName = "SBC A,A"
			a = SBC(a,a)
			OpLength = 1
			T = 4
 		Case 160
			OpName = "AND B"
			a = AND_(a,b)
			OpLength = 1
			T = 4
 		Case 161
			OpName = "AND C"
			a = AND_(a,c)
			OpLength = 1
			T = 4
 		Case 162
			OpName = "AND D"
			a = AND_(a,d)				
			OpLength = 1
			T = 4
 		Case 163
			OpName = "AND E"
			a = AND_(a,e)				
			OpLength = 1
			T = 4
 		Case 164
			OpName = "AND H"
			a = AND_(a,h)				
			OpLength = 1
			T = 4
 		Case 165
			OpName = "AND L"
			a = AND_(a,l)				
			OpLength = 1
			T = 4
 		Case 166
			OpName = "AND (HL)"
			a = AND_(a,Memory.Read(HL))
			OpLength = 1
			T = 7
 		Case 167
			OpName = "AND A"
			a = AND_(a,a)
			OpLength = 1
			T = 4
 		Case 168
			OpName = "XOR B"
			a = XOR_(a,b)
			OpLength = 1
			T = 4
 		Case 169
			OpName = "XOR C"
			a = XOR_(a,c)
			OpLength = 1
			T = 4
 		Case 170
			OpName = "XOR D"
			a = XOR_(a,d)				
			OpLength = 1
			T = 4
 		Case 171
			OpName = "XOR E"
			a = XOR_(a,e)				
			OpLength = 1
			T = 4
 		Case 172
			OpName = "XOR H"
			a = XOR_(a,h)				
			OpLength = 1
			T = 4
 		Case 173
			OpName = "XOR L"
			a = XOR_(a,l)				
			OpLength = 1
			T = 4
 		Case 174
			OpName = "XOR (HL)"
			a = XOR_(a,Memory.Read(HL))
			OpLength = 1
			T = 7
 		Case 175
			OpName = "XOR A"
			a = XOR_(a,a)
			OpLength = 1
			T = 4
 		Case 176
			OpName = "OR B"
			a = OR_(a,b)
			OpLength = 1
			T = 4
 		Case 177
			OpName = "OR C"
			a = OR_(a,c)
			OpLength = 1
			T = 4
 		Case 178
			OpName = "OR D"
			a = OR_(a,d)				
			OpLength = 1
			T = 4
 		Case 179
			OpName = "OR E"
			a = OR_(a,e)				
			OpLength = 1
			T = 4
 		Case 180
			OpName = "OR H"
			a = OR_(a,h)				
			OpLength = 1
			T = 4
 		Case 181
			OpName = "OR L"
			a = OR_(a,l)				
			OpLength = 1
			T = 4
 		Case 182
			OpName = "OR (HL)"
			a = OR_(a,Memory.Read(HL))
			OpLength = 1
			T = 7
 		Case 183
			OpName = "OR A"
			a = OR_(a,a)
			OpLength = 1
			T = 4
 		Case 184
			OpName = "CP B"
			SUB_(a,b)
			OpLength = 1
			T = 4
 		Case 185
			OpName = "CP C"
			SUB_(a,c)
			OpLength = 1
			T = 4
 		Case 186
			OpName = "CP D"
			SUB_(a,d)				
			OpLength = 1
			T = 4
 		Case 187
			OpName = "CP E"
			SUB_(a,e)				
			OpLength = 1
			T = 4
 		Case 188
			OpName = "CP H"
			SUB_(a,h)				
			OpLength = 1
			T = 4
 		Case 189
			OpName = "CP L"
			SUB_(a,l)				
			OpLength = 1
			T = 4
 		Case 190
			OpName = "CP (HL)"
			SUB_(a,Memory.Read(HL))
			OpLength = 1
			T = 7
 		Case 191
			OpName = "CP A"
			SUB_(a,a)
			OpLength = 1
			T = 4
 		Case 192
			OpName = "RET NZ"
			If ZeroFlag = 0 Then
				PC = Pop16
				OpLength = 0
				T = 11
			Else
				OpLength = 1
				T = 5
			End If
 		Case 193
			OpName = "POP BC"
			Temp = Pop16
			b = HighByte(Temp)
			c = LowByte(Temp)
			OpLength = 1			
			T = 10
 		Case 194
			OpName = "JP NZ,NN"
			If ZeroFlag = 0 Then
				PC = NN
				OpLength = 0
			Else
				OpLength = 3
			End If
			T = 10
 		Case 195
			OpName = "JUMP NN"
			PC = NN
			OpLength = 0
			T = 10
 		Case 196
			OpName = "CALL NZ,NN"
			If ZeroFlag = 0 Then
				Push16(PC + 3)
				PC = NN
				OpLength = 0
				T = 17
			Else
				OpLength = 3
				T = 10
			End If
 		Case 197
			OpName = "PUSH BC"
			Push16(BC)
			OpLength = 1
			T = 4
 		Case 198
			OpName = "ADD A,N"
			a = ADD(a,N)
			OpLength = 2
			T = 7
 		Case 199
			OpName = "RST 0"
			Push16(PC + 1)
			PC = 0x0000
			OpLength = 0
			T = 11
 		Case 200
			OpName = "RET Z"
			If ZeroFlag = 1 Then
				PC = Pop16
				OpLength = 0
				T = 11
			Else
				OpLength = 1
				T = 5
			End If
 		Case 201
			OpName = "RET"
			PC = Pop16
			OpLength = 0
			T = 10
 		Case 202
			OpName = "JP Z,NN"
			If ZeroFlag = 1 Then
				PC = NN
				OpLength = 0
			Else
				OpLength = 3
			End If
			T = 10
 		Case 203
			OpName = "CB"
			OpLength = 1
 		Case 204
			OpName = "CALL Z,NN"
			If ZeroFlag = 1 Then
				Push16(PC + 3)
				PC = NN
				OpLength = 0
				T = 17
			Else
				OpLength = 3
				T = 19
			End If
 		Case 205
			OpName = "CALL NN"
			Push16(PC + 3)
			PC = NN
			OpLength = 0
			T = 17
 		Case 206
			OpName = "ADC A,N"
			a = ADC(a,N)
			OpLength = 2
			T = 7
 		Case 207
			OpName = "RST 8"
			Push16(PC + 1)
			PC = 8
			OpLength = 0
			T = 11
 		Case 208
			OpName = "RET NC"
			If CarryFlag = 0 Then
				PC = Pop16
				OpLength = 0
			Else
				OpLength = 1
			End If
			T = 5
 		Case 209
			OpName = "POP DE"
			Temp = Pop16
			d = HighByte(Temp)
			e = LowByte(Temp)
			OpLength = 1
			T = 10
 		Case 210
			OpName = "JPNC,NN"
			If CarryFlag = 0 Then
				PC = NN
				OpLength = 0
			Else
				OpLength = 3
			End If
			T = 10
 		Case 211
			OpName = "Out(N),a"
			OutPort.Process(N,a)
			OpLength = 2
			T = 11
 		Case 212
			OpName = "CALL NC,NN"
			If CarryFlag = 0 Then
				Push16(PC + 3)
				PC = NN
				OpLength = 0
				T = 17
			Else
				OpLength = 3
				T = 10
			End If
			T = 4
 		Case 213
			OpName = "PUSH DE"
			Push16(DE)
			OpLength = 1
			T = 11
 		Case 214
			OpName = "SUB N"
			a = SUB_(a,N)
			OpLength = 2
			T = 7
 		Case 215
			OpName = "RST 16"
			Push16(PC + 1)
			PC = 16
			OpLength = 0
			T = 11
 		Case 216
			OpName = "RET C"
			If CarryFlag = 1 Then
				PC = Pop16
				OpLength = 0
			Else
				OpLength = 1
			End If
			T = 5
 		Case 217
			OpName = "EXX"
			Temp = b
			b = bPrime
			bPrime = Temp
			Temp = c
			c = cPrime
			cPrime = Temp
			Temp = d
			d = dPrime
			dPrime = Temp
			Temp = e
			e = ePrime
			ePrime = Temp
			Temp = h
			h = hPrime
			hPrime = Temp
			Temp = l
			l = lPrime
			lPrime = Temp
			OpLength = 1
			T = 4
 		Case 218
			OpName = "JPC,NN"
			If CarryFlag = 1 Then
				PC = NN
				OpLength = 0
			Else
				OpLength = 3
			End If
			T = 10
 		Case 219
			OpName = "IN A,(N)"
			'The high byte of the port address is fetched from A.
			'No flags are affected.
			a = InPort.In(a * 256 + N)
			OpLength = 2
			T = 11
 		Case 220
			OpName = "CALL C,NN"
			If CarryFlag = 1 Then
				Push16(PC + 3)
				PC = NN
				OpLength = 0
				T = 17
			Else
				OpLength = 3
				T = 10
			End If
 		Case 221
			OpName = "DD"
			OpLength = 1
 		Case 222
			OpName = "SBC A,N"
			a = SBC(a,N)
			OpLength = 2
			T = 7
 		Case 223
			OpName = "RST 24"
			Push16(PC + 1)
			PC = 24
			OpLength = 0
			T = 11
 		Case 224
			OpName = "RET PO"
			If Parity_OverflowFlag = 0 Then
				PC = Pop16
				OpLength = 0
			Else
				OpLength = 1
			End If
			T = 4
 		Case 225
			OpName = "POP HL"
			Temp = Pop16
			h = HighByte(Temp)
			l = LowByte(Temp)
			OpLength = 1
			T = 4
 		Case 226
			OpName = "JP PO,NN"
			If Parity_OverflowFlag = 0 Then
				PC = NN
				OpLength = 0
				T = 11
			Else
				OpLength = 3
				T = 5
			End If
 		Case 227
			OpName = "EX (SP),HL"
			Temp = Memory.Read(SP)
			Memory.Write(SP,l)
			l = Temp
			Temp = Memory.Read(SP + 1)
			Memory.Write(SP + 1,h)
			h = Temp
			OpLength = 1
			T = 19
 		Case 228
			OpName = "CALL PO,NN"
			If Parity_OverflowFlag = 0 Then
				Push16(PC + 3)
				PC = NN
				OpLength = 0
				T = 17
			Else
				OpLength = 3
				T = 109
			End If
 		Case 229
			OpName = "PUSH HL"
			Push16(HL)
			OpLength = 1
			T = 11
 		Case 230
			OpName = "AND N"
			a = AND_(a,N)
			OpLength = 2
			T = 4
 		Case 231
			OpName = "RST 32"
			Push16(PC + 1)
			PC = 32
			OpLength = 0
			T = 11
 		Case 232
			OpName = "RET PE"
			If Parity_OverflowFlag = 1 Then
				PC = Pop16
				OpLength = 0
				T = 11
			Else
				OpLength = 1
				T = 5
			End If
 		Case 233
			OpName = "JP (HL)"
			PC = HL 'JP (HL) is actually JP HL
			OpLength = 0
			T = 4
 		Case 234
			OpName = "JP PE,NN"
			If Parity_OverflowFlag = 1 Then
				PC = NN
				OpLength = 0
			Else
				OpLength = 3
			End If
			T = 10
 		Case 235
			OpName = "EX DE,HL"
			Temp = d
			d = h
			h = Temp
			Temp = e
			e = l
			l = Temp
			OpLength = 1
			T = 4
 		Case 236
			OpName = "CALL PE,NN"
			If Parity_OverflowFlag = 1 Then
				Push16(PC + 3)
				PC = NN
				OpLength = 0
				T = 17
			Else
				OpLength = 3
				T = 10
			End If
 		Case 237
			OpName = "ED"
			OpLength = 1
 		Case 238
			OpName = "XOR N"
			a = XOR_(a,N)
			OpLength = 2
			T = 7
 		Case 239
			OpName = "RST 40"
			Push16(PC + 1)
			PC = 40
			OpLength = 0
			T = 11
 		Case 240
			OpName = "RET P"
			If SignFlag = 0 Then
				PC = Pop16
				OpLength = 0
				T = 11
			Else
				OpLength = 1
				T = 5
			End If
 		Case 241
			OpName = "POP AF"
			Temp = Pop16
			a = HighByte(Temp)
			f = LowByte(Temp)
			SignFlag = Bit.AND(f,128) / 128
			ZeroFlag = Bit.AND(f,64) / 64
			HalfCarryFlag = Bit.AND(f,4) / 4
			Parity_OverflowFlag = Bit.AND(f,2) / 2
			CarryFlag = Bit.AND(f,1) / 1			
			OpLength = 1
			T = 10
 		Case 242
			OpName = "JP P,NN"
			If SignFlag = 0 Then
				PC = NN
				OpLength = 0
			Else
				OpLength = 3
			End If
			T = 10
 		Case 243
			OpName = "DI"
			InterruptEnabled = False
			OpLength = 1
			T = 4
 		Case 244
			OpName = "CALL P,NN"
			If SignFlag = 0 Then
				Push16(PC + 3)
				PC = NN
				OpLength = 0
				T = 17
			Else
				OpLength = 3
				T = 10
			End If
 		Case 245
			OpName = "PUSH AF"
			Push16 (a * 256 + f)
			OpLength = 1
			T = 11
 		Case 246
			OpName = "OR N"
			a = OR_(a,N)
			OpLength = 2
			T = 4
 		Case 247
			OpName = "RST 48"
			Push16(PC + 1)
			PC = 48
			OpLength = 0
			T = 11
 		Case 248
			OpName = "RET M"
			If SignFlag = 1 Then
				PC = Pop16
				OpLength = 0
				T = 11
			Else
				OpLength = 1
				T = 5
			End If
 		Case 249
			OpName = "LD SP,HL"
			SP = HL
			OpLength = 1
			T = 6
 		Case 250
			OpName = "JP M,NN"
			If SignFlag = 1 Then
				PC = NN
				OpLength = 0
			Else
				OpLength = 3
			End If
			T = 10
 		Case 251
			OpName = "EI"
			InterruptEnabled = True
			OpLength = 1
			T = 4
 		Case 252
			OpName = "CALL M,NN"
			If SignFlag = 1 Then
				Push16(PC + 3)
				PC = NN
				OpLength = 0
				T = 17
			Else
				OpLength = 3
				T = 10
			End If
			T = 4
 		Case 253
			OpName = "FD"
			OpLength = 1
 		Case 254
			OpName = "CP N"
			SUB_(a,N)
			OpLength = 2
			T = 7
 		Case 255
			OpName = "RST 56"
			Push16(PC + 1)
			PC = 56
			OpLength = 0
			T = 11
 		Case Else
			OpName = ""
			Logger.logsWARN("Missing OpCode " & OpCode)
	End Select
End Sub
Private Sub CBOpCodes
	Select Opname_2
		Case "DD"
			CBAfterDDOpCodes
		Case "FD"
			CBAfterFDOpCodes
		Case Else
			CBRegularOpCodes
		End Select
End Sub
Private Sub CBRegularOpCodes
	IncR 'R is incremented twice for every CB opcode. This is the second increment.
	Select OpCode
		#Region Shift
		Case 0
			OpName = "RLC B"
			b = RLC(b)
			OpLength = 1
			T = 8
 		Case 1
			OpName = "RLC C"
			c = RLC(c)
			OpLength = 1
			T = 8
 		Case 2
			OpName = "RLC D"
			d = RLC(d)
			OpLength = 1
			T = 8
 		Case 3
			OpName = "RLC E"
			e = RLC(e)
			OpLength = 1
			T = 8
 		Case 4
			OpName = "RLC H"
			h = RLC(h)
			OpLength = 1
			T = 8
 		Case 5
			OpName = "RLC L"
			l = RLC(l)
			OpLength = 1
			T = 8
 		Case 6
			OpName = "RLC (HL)"
			Memory.Write(HL,RLC(Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 7
			OpName = "RLC A"
			a = RLC(a)
			OpLength = 1
			T = 8
 		Case 8
			OpName = "RRC B"
			b = RRC(b)
			OpLength = 1
			T = 8
 		Case 9
			OpName = "RRC C"
			c = RRC(c)
			OpLength = 1
			T = 8
 		Case 10
			OpName = "RRC D"
			d = RRC(d)
			OpLength = 1
			T = 8
 		Case 11
			OpName = "RRC E"
			e = RRC(e)
			OpLength = 1
			T = 8
 		Case 12
			OpName = "RRC H"
			h = RRC(h)
			OpLength = 1
			T = 8
 		Case 13
			OpName = "RRC L"
			l = RRC(l)
			OpLength = 1
			T = 8
 		Case 14
			OpName = "RRC (HL)"
			Memory.Write(HL,RRC(Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 15
			OpName = "RRC A"
			a = RRC(a)
			OpLength = 1
			T = 8
 		Case 16
			OpName = "RL B"
			b = RL(b)
			OpLength = 1
			T = 8
 		Case 17
			OpName = "RL C"
			c = RL(c)
			OpLength = 1
			T = 8
 		Case 18
			OpName = "RL D"
			d = RL(d)
			OpLength = 1
			T = 8
 		Case 19
			OpName = "RL E"
			e = RL(e)
			OpLength = 1
			T = 8
 		Case 20
			OpName = "RL H"
			h = RL(h)
			OpLength = 1
			T = 8
 		Case 21
			OpName = "RL L"
			l = RL(l)
			OpLength = 1
			T = 8
 		Case 22
			OpName = "RL (HL)"
			Memory.Write(HL,RL(Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 23
			OpName = "RL A"
			a = RL(a)
			OpLength = 1
			T = 8
 		Case 24
			OpName = "RR B"
			b = RR(b)
			OpLength = 1
			T = 8
 		Case 25
			OpName = "RR C"
			c = RR(c)
			OpLength = 1
			T = 8
 		Case 26
			OpName = "RR D"
			d = RR(d)
			OpLength = 1
			T = 8
 		Case 27
			OpName = "RRE"
			e = RR(e)
			OpLength = 1
			T = 8
 		Case 28
			OpName = "RR H"
			h = RR(h)
			OpLength = 1
			T = 8
 		Case 29
			OpName = "RR L"
			l = RR(l)
			OpLength = 1
			T = 8
 		Case 30
			OpName = "RR (HL)"
			Memory.Write(HL,RR(Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 31
			OpName = "RR A"
			a = RR(a)
			OpLength = 1
			T = 8
 		Case 32
			OpName = "SLA B"
			b = SLA(b)
			OpLength = 1
			T = 8
 		Case 33
			OpName = "SLA C"
			c = SLA(c)
			OpLength = 1
			T = 8
 		Case 34
			OpName = "SLA D"
			d = SLA(d)
			OpLength = 1
			T = 8
 		Case 35
			OpName = "SLA E"
			e = SLA(e)
			OpLength = 1
			T = 8
 		Case 36
			OpName = "SLA H"
			h = SLA(h)
			OpLength = 1
			T = 8
 		Case 37
			OpName = "SLA L"
			l = SLA(l)
			OpLength = 1
			T = 8
 		Case 38
			OpName = "SLA (HL)"
			Memory.Write(HL,SLA(Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 39
			OpName = "SLA A"
			a = SLA(a)
			OpLength = 1
			T = 8
 		Case 40
			OpName = "SRA B"
			b = SRA(b)
			OpLength = 1
			T = 8
 		Case 41
			OpName = "SRA C"
			c = SRA(c)
			OpLength = 1
			T = 8
 		Case 42
			OpName = "SRA D"
			d = SRA(d)
			OpLength = 1
			T = 8
 		Case 43
			OpName = "SRA E"
			e = SRA(e)
			OpLength = 1
			T = 8
 		Case 44
			OpName = "SRA H"
			h = SRA(h)
			OpLength = 1
			T = 8
 		Case 45
			OpName = "SRA L"
			l = SRA(l)
			OpLength = 1
			T = 8
 		Case 46
			OpName = "SRA (HL)"
			Memory.Write(HL,SRA(Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 47
			OpName = "SRA A"
			a = SRA(a)
			OpLength = 1
			T = 8
 		Case 56
			OpName = "SRL B"
			b = SRL(b)
			OpLength = 1
			T = 8
 		Case 57
			OpName = "SRL C"
			c = SRL(c)
			OpLength = 1
			T = 8
 		Case 58
			OpName = "SRL D"
			d = SRL(d)
			OpLength = 1
			T = 8
 		Case 59
			OpName = "SRL E"
			e = SRL(e)
			OpLength = 1
			T = 8
 		Case 60
			OpName = "SRL H"
			h = SRL(h)
			OpLength = 1
			T = 8
 		Case 61
			OpName = "SRL L"
			l = SRL(l)
			OpLength = 1
			T = 8
 		Case 62
			OpName = "SRL (HL)"
			Memory.Write(HL,SRL(Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 63
			OpName = "SRL A"
			a = SRL(a)
			OpLength = 1
			T = 8
		#End Region
		#Region Bit Check
			T = 8
 		Case 64
			OpName = "BIT 0,B"
			BitCheck(0,b)
			OpLength = 1
			T = 8
 		Case 65
			OpName = "BIT 0,C"
			BitCheck(0,c)
			OpLength = 1
			T = 8
 		Case 66
			OpName = "BIT 0,D"
			BitCheck(0,d)
			OpLength = 1
			T = 8
 		Case 67
			OpName = "BIT 0,E"
			BitCheck(0,e)
			OpLength = 1
			T = 8
 		Case 68
			OpName = "BIT 0,H"
			BitCheck(0,h)
			OpLength = 1
			T = 8
 		Case 69
			OpName = "BIT 0,L"
			BitCheck(0,l)
			OpLength = 1
			T = 8
 		Case 70
			OpName = "BIT 0,(HL)"
			BitCheck(0,Memory.Read(HL))
			OpLength = 1
			T = 12
 		Case 71
			OpName = "BIT 0,A"
			BitCheck(0,a)
			OpLength = 1
			T = 8
 		Case 72
			OpName = "BIT 1,B"
			BitCheck(1,b)
			OpLength = 1
			T = 8
 		Case 73
			OpName = "BIT 1,C"
			BitCheck(1,c)
			OpLength = 1
			T = 8
 		Case 74
			OpName = "BIT 1,D"
			BitCheck(1,d)
			OpLength = 1
			T = 8
 		Case 75
			OpName = "BIT 1,E"
			BitCheck(1,e)
			OpLength = 1
			T = 8
 		Case 76
			OpName = "BIT 1,H"
			BitCheck(1,h)
			OpLength = 1
			T = 8
 		Case 77
			OpName = "BIT 1,L"
			BitCheck(1,l)
			OpLength = 1
			T = 8
 		Case 78
			OpName = "BIT 1,(HL)"
			BitCheck(1,Memory.Read(HL))
			OpLength = 1
			T = 12
 		Case 79
			OpName = "BIT 1,A"
			BitCheck(1,a)
			OpLength = 1
			T = 8
 		Case 80
			OpName = "BIT 2,B"
			BitCheck(2,b)
			OpLength = 1
			T = 8
 		Case 81
			OpName = "BIT 2,C"
			BitCheck(2,c)
			OpLength = 1
			T = 8
 		Case 82
			OpName = "BIT 2,D"
			BitCheck(2,d)
			OpLength = 1
			T = 8
 		Case 83
			OpName = "BIT 2,E"
			BitCheck(2,e)
			OpLength = 1
			T = 8
 		Case 84
			OpName = "BIT 2,H"
			BitCheck(2,h)
			OpLength = 1
			T = 8
 		Case 85
			OpName = "BIT 2,L"
			BitCheck(2,l)
			OpLength = 1
			T = 8
 		Case 86
			OpName = "BIT 2,(HL)"
			BitCheck(2,Memory.Read(HL))
			OpLength = 1
			T = 12
 		Case 87
			OpName = "BIT 2,A"
			BitCheck(2,a)
			OpLength = 1
			T = 8
 		Case 88
			OpName = "BIT 3,B"
			BitCheck(3,b)
			OpLength = 1
			T = 8
 		Case 89
			OpName = "BIT 3,C"
			BitCheck(3,c)
			OpLength = 1
			T = 8
 		Case 90
			OpName = "BIT 3,D"
			BitCheck(3,d)
			OpLength = 1
			T = 8
 		Case 91
			OpName = "BIT 3,E"
			BitCheck(3,e)
			OpLength = 1
			T = 8
 		Case 92
			OpName = "BIT 3,H"
			BitCheck(3,h)
			OpLength = 1
			T = 8
 		Case 93
			OpName = "BIT 3,L"
			BitCheck(3,l)
			OpLength = 1
			T = 8
 		Case 94
			OpName = "BIT 3,(HL)"
			BitCheck(3,Memory.Read(HL))
			OpLength = 1
			T = 12
 		Case 95
			OpName = "BIT 3,A"
			BitCheck(3,a)
			OpLength = 1
			T = 8
 		Case 96
			OpName = "BIT 4,B"
			BitCheck(4,b)
			OpLength = 1
			T = 8
 		Case 97
			OpName = "BIT 4,C"
			BitCheck(4,c)
			OpLength = 1
			T = 8
 		Case 98
			OpName = "BIT 4,D"
			BitCheck(4,d)
			OpLength = 1
			T = 8
 		Case 99
			OpName = "BIT 4,E"
			BitCheck(4,e)
			OpLength = 1
			T = 8
 		Case 100
			OpName = "BIT 4,H"
			BitCheck(4,h)
			OpLength = 1
			T = 8
 		Case 101
			OpName = "BIT 4,L"
			BitCheck(4,l)
			OpLength = 1
			T = 8
 		Case 102
			OpName = "BIT 4,(HL)"
			BitCheck(4,Memory.Read(HL))
			OpLength = 1
			T = 12
 		Case 103
			OpName = "BIT 4,A"
			BitCheck(4,a)
			OpLength = 1
			T = 8
 		Case 104
			OpName = "BIT 5,B"
			BitCheck(5,b)
			OpLength = 1
			T = 8
 		Case 105
			OpName = "BIT 5,C"
			BitCheck(5,c)
			OpLength = 1
			T = 8
 		Case 106
			OpName = "BIT 5,D"
			BitCheck(5,d)
			OpLength = 1
			T = 8
 		Case 107
			OpName = "BIT 5,E"
			BitCheck(5,e)
			OpLength = 1
			T = 8
 		Case 108
			OpName = "BIT 5,H"
			BitCheck(5,h)
			OpLength = 1
			T = 8
 		Case 109
			OpName = "BIT 5,L"
			BitCheck(5,l)
			OpLength = 1
			T = 8
 		Case 110
			OpName = "BIT 5,(HL)"
			BitCheck(5,Memory.Read(HL))
			OpLength = 1
			T = 12
 		Case 111
			OpName = "BIT 5,A"
			BitCheck(5,a)
			OpLength = 1
			T = 8
 		Case 112
			OpName = "BIT 6,B"
			BitCheck(6,b)
			OpLength = 1
			T = 8
 		Case 113
			OpName = "BIT 6,C"
			BitCheck(6,c)
			OpLength = 1
			T = 8
 		Case 114
			OpName = "BIT 6,D"
			BitCheck(6,d)
			OpLength = 1
			T = 8
 		Case 115
			OpName = "BIT 6,E"
			BitCheck(6,e)
			OpLength = 1
			T = 8
 		Case 116
			OpName = "BIT 6,H"
			BitCheck(6,h)
			OpLength = 1
			T = 8
 		Case 117
			OpName = "BIT 6,L"
			BitCheck(6,l)
			OpLength = 1
			T = 8
 		Case 118
			OpName = "BIT 6,(HL)"
			BitCheck(6,Memory.Read(HL))
			OpLength = 1
			T = 12
 		Case 119
			OpName = "BIT 6,A"
			BitCheck(6,a)
			OpLength = 1
			T = 8
 		Case 120
			OpName = "BIT 7,B"
			BitCheck(7,b)
			OpLength = 1
			T = 8
 		Case 121
			OpName = "BIT 7,C"
			BitCheck(7,c)
			OpLength = 1
			T = 8
 		Case 122
			OpName = "BIT 7,D"
			BitCheck(7,d)
			OpLength = 1
			T = 8
 		Case 123
			OpName = "BIT 7,E"
			BitCheck(7,e)
			OpLength = 1
			T = 8
 		Case 124
			OpName = "BIT 7,H"
			BitCheck(7,h)
			OpLength = 1
			T = 8
 		Case 125
			OpName = "BIT 7,L"
			BitCheck(7,l)
			OpLength = 1
			T = 8
 		Case 126
			OpName = "BIT 7,(HL)"
			BitCheck(7,Memory.Read(HL))
			OpLength = 1
			T = 12
 		Case 127
			OpName = "BIT 7,A"
			BitCheck(7,a)
			OpLength = 1
			T = 8
		#End Region
		#Region Bit Reset
 		Case 128
			OpName = "RES 0,B"
			b = BitReset(0,b)
			OpLength = 1
			T = 8
 		Case 129
			OpName = "RES 0,C"
			c = BitReset(0,c)
			OpLength = 1
			T = 8
 		Case 130
			OpName = "RES 0,D"
			d = BitReset(0,d)
			OpLength = 1
			T = 8
 		Case 131
			OpName = "RES 0,E"
			e = BitReset(0,e)
			OpLength = 1
			T = 8
 		Case 132
			OpName = "RES 0,H"
			h = BitReset(0,h)
			OpLength = 1
			T = 8
 		Case 133
			OpName = "RES 0,L"
			l = BitReset(0,l)
			OpLength = 1
			T = 8
 		Case 134
			OpName = "RES 0,(HL)"
			Memory.Write(HL,BitReset(0,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 135
			OpName = "RES 0,A"
			a = BitReset(0,a)
			OpLength = 1
			T = 8
 		Case 136
			OpName = "RES 1,B"
			b = BitReset(1,b)
			OpLength = 1
			T = 8
 		Case 137
			OpName = "RES 1,C"
			c = BitReset(1,c)
			OpLength = 1
			T = 8
 		Case 138
			OpName = "RES 1,D"
			d = BitReset(1,d)
			OpLength = 1
			T = 8
 		Case 139
			OpName = "RES 1,E"
			e = BitReset(1,e)
			OpLength = 1
			T = 8
 		Case 140
			OpName = "RES 1,H"
			h = BitReset(1,h)
			OpLength = 1
			T = 8
 		Case 141
			OpName = "RES 1,L"
			l = BitReset(1,l)
			OpLength = 1
			T = 8
 		Case 142
			OpName = "RES 1,(HL)"
			Memory.Write(HL,BitReset(1,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 143
			OpName = "RES 1,A"
			a = BitReset(1,a)
			OpLength = 1
			T = 8
 		Case 144
			OpName = "RES 2,B"
			b = BitReset(2,b)
			OpLength = 1
			T = 8
 		Case 145
			OpName = "RES 2,C"
			c = BitReset(2,c)
			OpLength = 1
			T = 8
 		Case 146
			OpName = "RES 2,D"
			d = BitReset(2,d)
			OpLength = 1
			T = 8
 		Case 147
			OpName = "RES 2,E"
			e = BitReset(2,e)
			OpLength = 1
			T = 8
 		Case 148
			OpName = "RES 2,H"
			h = BitReset(2,h)
			OpLength = 1
			T = 8
 		Case 149
			OpName = "RES 2,L"
			l = BitReset(2,l)
			OpLength = 1
			T = 8
 		Case 150
			OpName = "RES 2,(HL)"
			Memory.Write(HL,BitReset(2,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 151
			OpName = "RES 2,A"
			a = BitReset(2,a)
			OpLength = 1
			T = 8
 		Case 152
			OpName = "RES 3,B"
			b = BitReset(3,b)
			OpLength = 1
			T = 8
 		Case 153
			OpName = "RES 3,C"
			c = BitReset(3,c)
			OpLength = 1
			T = 8
 		Case 154
			OpName = "RES 3,D"
			d = BitReset(3,d)
			OpLength = 1
			T = 8
 		Case 155
			OpName = "RES 3,E"
			e = BitReset(3,e)
			OpLength = 1
			T = 8
 		Case 156
			OpName = "RES 3,H"
			h = BitReset(3,h)
			OpLength = 1
			T = 8
 		Case 157
			OpName = "RES 3,L"
			l = BitReset(3,l)
			OpLength = 1
			T = 8
 		Case 158
			OpName = "RES 3,(HL)"
			Memory.Write(HL,BitReset(3,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 159
			OpName = "RES 3,A"
			a = BitReset(3,a)
			OpLength = 1
			T = 8
 		Case 160
			OpName = "RES 4,B"
			b = BitReset(4,b)
			OpLength = 1
			T = 8
 		Case 161
			OpName = "RES 4,C"
			c = BitReset(4,c)
			OpLength = 1
			T = 8
 		Case 162
			OpName = "RES 4,D"
			d = BitReset(4,d)
			OpLength = 1
			T = 8
 		Case 163
			OpName = "RES 4,E"
			e = BitReset(4,e)
			OpLength = 1
			T = 8
 		Case 164
			OpName = "RES 4,H"
			h = BitReset(4,h)
			OpLength = 1
			T = 8
 		Case 165
			OpName = "RES 4,L"
			l = BitReset(4,l)
			OpLength = 1
			T = 8
 		Case 166
			OpName = "RES 4,(HL)"
			Memory.Write(HL,BitReset(4,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 167
			OpName = "RES 4,A"
			a = BitReset(4,a)
			OpLength = 1
			T = 8
 		Case 168
			OpName = "RES 5,B"
			b = BitReset(5,b)
			OpLength = 1
			T = 8
 		Case 169
			OpName = "RES 5,C"
			c = BitReset(5,c)
			OpLength = 1
			T = 8
 		Case 170
			OpName = "RES 5,D"
			d = BitReset(5,d)
			OpLength = 1
			T = 8
 		Case 171
			OpName = "RES 5,E"
			e = BitReset(5,e)
			OpLength = 1
			T = 8
 		Case 172
			OpName = "RES 5,H"
			h = BitReset(5,h)
			OpLength = 1
			T = 8
 		Case 173
			OpName = "RES 5,L"
			l = BitReset(5,l)
			OpLength = 1
			T = 8
 		Case 174
			OpName = "RES 5,(HL)"
			Memory.Write(HL,BitReset(5,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 175
			OpName = "RES 5,A"
			a = BitReset(5,a)
			OpLength = 1
			T = 8
 		Case 176
			OpName = "RES 6,B"
			b = BitReset(6,b)
			OpLength = 1
			T = 8
 		Case 177
			OpName = "RES 6,C"
			c = BitReset(6,c)
			OpLength = 1
			T = 8
 		Case 178
			OpName = "RES 6,D"
			d = BitReset(6,d)
			OpLength = 1
			T = 8
 		Case 179
			OpName = "RES 6,E"
			e = BitReset(6,e)
			OpLength = 1
			T = 8
 		Case 180
			OpName = "RES 6,H"
			h = BitReset(6,h)
			OpLength = 1
			T = 8
 		Case 181
			OpName = "RES 6,L"
			l = BitReset(6,l)
			OpLength = 1
			T = 8
 		Case 182
			OpName = "RES 6,(HL)"
			Memory.Write(HL,BitReset(6,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 183
			OpName = "RES 6,A"
			a = BitReset(6,a)
			OpLength = 1	
			T = 8
 		Case 184
			OpName = "RES 7,B"
			b = BitReset(7,b)
			OpLength = 1
			T = 8
 		Case 185
			OpName = "RES 7,C"
			c = BitReset(7,c)
			OpLength = 1
			T = 8
 		Case 186
			OpName = "RES 7,D"
			d = BitReset(7,d)
			OpLength = 1
			T = 8
 		Case 187
			OpName = "RES 7,E"
			e = BitReset(7,e)
			OpLength = 1
			T = 8
 		Case 188
			OpName = "RES 7,H"
			h = BitReset(7,h)
			OpLength = 1
			T = 8
 		Case 189
			OpName = "RES 7,L"
			l = BitReset(7,l)
			OpLength = 1
			T = 8
 		Case 190
			OpName = "RES 7,(HL)"
			Memory.Write(HL,BitReset(7,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 191
			OpName = "RES 7,A"
			a = BitReset(7,a)
			OpLength = 1
			T = 8
		#End Region
		#Region Bit Set
			T = 8
 		Case 192
			OpName = "SET 0,B"
			b = BitSet(0,b)
			OpLength = 1
			T = 8
 		Case 193
			OpName = "SET 0,C"
			c = BitSet(0,c)
			OpLength = 1
			T = 8
 		Case 194
			OpName = "SET 0,D"
			d = BitSet(0,d)
			OpLength = 1
			T = 8
 		Case 195
			OpName = "SET 0,E"
			e = BitSet(0,e)
			OpLength = 1
			T = 8
 		Case 196
			OpName = "SET 0,H"
			h = BitSet(0,h)
			OpLength = 1
			T = 8
 		Case 197
			OpName = "SET 0,L"
			l = BitSet(0,l)
			OpLength = 1
			T = 8
 		Case 198
			OpName = "SET 0,(HL)"
			Memory.Write(HL,BitSet(0,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 199
			OpName = "SET 0,A"
			a = BitSet(0,a)
			OpLength = 1
			T = 8
 		Case 200
			OpName = "SET 1,B"
			b = BitSet(1,b)
			OpLength = 1
			T = 8
 		Case 201
			OpName = "SET 1,C"
			c = BitSet(1,c)
			OpLength = 1
			T = 8
 		Case 202
			OpName = "SET 1,D"
			d = BitSet(1,d)
			OpLength = 1
			T = 8
 		Case 203
			OpName = "SET 1,E"
			e = BitSet(1,e)
			OpLength = 1
			T = 8
 		Case 204
			OpName = "SET 1,H"
			h = BitSet(1,h)
			OpLength = 1
			T = 8
 		Case 205
			OpName = "SET 1,L"
			l = BitSet(1,l)
			OpLength = 1
			T = 8
 		Case 206
			OpName = "SET 1,(HL)"
			Memory.Write(HL,BitSet(1,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 207
			OpName = "SET 1,A"
			a = BitSet(1,a)
			OpLength = 1
			T = 8
 		Case 208
			OpName = "SET 2,B"
			b = BitSet(2,b)
			OpLength = 1
			T = 8
 		Case 209
			OpName = "SET 2,C"
			c = BitSet(2,c)
			OpLength = 1
			T = 8
 		Case 210
			OpName = "SET 2,D"
			d = BitSet(2,d)
			OpLength = 1
			T = 8
 		Case 211
			OpName = "SET 2,E"
			e = BitSet(2,e)
			OpLength = 1
			T = 8
 		Case 212
			OpName = "SET 2,H"
			h = BitSet(2,h)
			OpLength = 1
			T = 8
 		Case 213
			OpName = "SET 2,L"
			l = BitSet(2,l)
			OpLength = 1
			T = 8
 		Case 214
			OpName = "SET 2,(HL)"
			Memory.Write(HL,BitSet(2,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 215
			OpName = "SET 2,A"
			a = BitSet(2,a)
			OpLength = 1
			T = 8
 		Case 216
			OpName = "SET 3,B"
			b = BitSet(3,b)
			OpLength = 1
			T = 8
 		Case 217
			OpName = "SET 3,C"
			c = BitSet(3,c)
			OpLength = 1
			T = 8
 		Case 218
			OpName = "SET 3,D"
			d = BitSet(3,d)
			OpLength = 1
			T = 8
 		Case 219
			OpName = "SET 3,E"
			e = BitSet(3,e)
			OpLength = 1
			T = 8
 		Case 220
			OpName = "SET 3,H"
			h = BitSet(3,h)
			OpLength = 1
			T = 8
 		Case 221
			OpName = "SET 3,L"
			l = BitSet(3,l)
			OpLength = 1
			T = 8
 		Case 222
			OpName = "SET 3,(HL)"
			Memory.Write(HL,BitSet(3,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 223
			OpName = "SET 3,A"
			a = BitSet(3,a)
			OpLength = 1
			T = 8
 		Case 224
			OpName = "SET 4,B"
			b = BitSet(4,b)
			OpLength = 1
			T = 8
 		Case 225
			OpName = "SET 4,C"
			c = BitSet(4,c)
			OpLength = 1
			T = 8
 		Case 226
			OpName = "SET 4,D"
			d = BitSet(4,d)
			OpLength = 1
			T = 8
 		Case 227
			OpName = "SET 4,E"
			e = BitSet(4,e)
			OpLength = 1
			T = 8
 		Case 228
			OpName = "SET 4,H"
			h = BitSet(4,h)
			OpLength = 1
			T = 8
 		Case 229
			OpName = "SET 4,L"
			l = BitSet(4,l)
			OpLength = 1
			T = 8
 		Case 230
			OpName = "SET 4,(HL)"
			Memory.Write(HL,BitSet(4,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 231
			OpName = "SET 4,A"
			a = BitSet(4,a)
			OpLength = 1
			T = 8
 		Case 232
			OpName = "SET 5,B"
			b = BitSet(5,b)
			OpLength = 1
			T = 8
 		Case 233
			OpName = "SET 5,C"
			c = BitSet(5,c)
			OpLength = 1
			T = 8
 		Case 234
			OpName = "SET 5,D"
			d = BitSet(5,d)
			OpLength = 1
			T = 8
 		Case 235
			OpName = "SET 5,E"
			e = BitSet(5,e)
			OpLength = 1
			T = 8
 		Case 236
			OpName = "SET 5,H"
			h = BitSet(5,h)
			OpLength = 1
			T = 8
 		Case 237
			OpName = "SET 5,L"
			l = BitSet(5,l)
			OpLength = 1
			T = 8
 		Case 238
			OpName = "SET 5,(HL)"
			Memory.Write(HL,BitSet(5,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 239
			OpName = "SET 5,A"
			a = BitSet(5,a)
			OpLength = 1
			T = 8
 		Case 240
			OpName = "SET 6,B"
			b = BitSet(6,b)
			OpLength = 1
			T = 8
 		Case 241
			OpName = "SET 6,C"
			c = BitSet(6,c)
			OpLength = 1
			T = 8
 		Case 242
			OpName = "SET 6,D"
			d = BitSet(6,d)
			OpLength = 1
			T = 8
 		Case 243
			OpName = "SET 6,E"
			e = BitSet(6,e)
			OpLength = 1
			T = 8
 		Case 244
			OpName = "SET 6,H"
			h = BitSet(6,h)
			OpLength = 1
			T = 8
 		Case 245
			OpName = "SET 6,L"
			l = BitSet(6,l)
			OpLength = 1
			T = 8
 		Case 246
			OpName = "SET 6,(HL)"
			Memory.Write(HL,BitSet(6,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 247
			OpName = "SET 6,A"
			a = BitSet(6,a)
			OpLength = 1	
			T = 8
 		Case 248
			OpName = "SET 7,B"
			b = BitSet(7,b)
			OpLength = 1
			T = 8
 		Case 249
			OpName = "SET 7,C"
			c = BitSet(7,c)
			OpLength = 1
			T = 8
 		Case 250
			OpName = "SET 7,D"
			d = BitSet(7,d)
			OpLength = 1
			T = 8
 		Case 251
			OpName = "SET 7,E"
			e = BitSet(7,e)
			OpLength = 1
			T = 8
 		Case 252
			OpName = "SET 7,H"
			h = BitSet(7,h)
			OpLength = 1
			T = 8
 		Case 253
			OpName = "SET 7,L"
			l = BitSet(7,l)
			OpLength = 1
			T = 8
 		Case 254
			OpName = "SET 7,(HL)"
			Memory.Write(HL,BitSet(7,Memory.Read(HL)))
			OpLength = 1
			T = 15
 		Case 255
			OpName = "SET 7,A"
			a = BitSet(7,a)
			OpLength = 1
			T = 8
		#End Region
 		Case Else
			OpName = ""
			Logger.logsWARN("Missing OpCode CB + " & OpCode)
	End Select
End Sub
Private Sub CBAfterDDOpCodes 'TODO: Fix all undocumented cases
	'R is incremented twice for every DDCB opcode. This has already been done when we get here
	Dim DDCBOpcode As Int
	DDCBOpcode = Memory.Read(PC + 1) 'The actual opode is at PC + 1, while the d-index is at PC.
	Select DDCBOpcode
		#Region Shift
		Case 0
			OpName = "RLC B"
			b = RLC(b)
			OpLength = 1
			T = 8
 		Case 1
			'TODO: Check this!
			OpName = "RLC (IX+d)> C"
			Memory.Write(IX + dIndex_1,RLC(Memory.Read(IX + dIndex_1)))
			c = Memory.Read(IX + dIndex_1)
			OpLength = 2
			T = 8
 		Case 2
			OpName = "RLC D"
			d = RLC(d)
			OpLength = 1
			T = 8
 		Case 3
			OpName = "RLC E"
			e = RLC(e)
			OpLength = 1
			T = 8
 		Case 4
			OpName = "RLC H"
			h = RLC(h)
			OpLength = 1
			T = 8
 		Case 5
			OpName = "RLC L"
			l = RLC(l)
			OpLength = 1
			T = 8
 		Case 6
			OpName = "RLC (IX+d)"
			Memory.Write(IX + dIndex_1,RLC(Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
 		Case 7
			OpName = "RLC A"
			a = RLC(a)
			OpLength = 1
			T = 8
 		Case 8
			OpName = "RRC B"
			b = RRC(b)
			OpLength = 1
			T = 8
 		Case 9
			OpName = "RRC C"
			c = RRC(c)
			OpLength = 1
			T = 8
 		Case 10
			OpName = "RRC D"
			d = RRC(d)
			OpLength = 1
			T = 8
 		Case 11
			OpName = "RRC E"
			e = RRC(e)
			OpLength = 1
			T = 8
 		Case 12
			OpName = "RRC H"
			h = RRC(h)
			OpLength = 1
			T = 8
 		Case 13
			OpName = "RRC L"
			l = RRC(l)
			OpLength = 1
			T = 8
 		Case 14
			OpName = "RRC (IX + d)"
			Memory.Write(IX + dIndex_1,RRC(Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
 		Case 15
			OpName = "RRC A"
			a = RRC(a)
			OpLength = 1
			T = 8
 		Case 16
			OpName = "RL B"
			b = RL(b)
			OpLength = 1
			T = 8
 		Case 17
			OpName = "RL C"
			c = RL(c)
			OpLength = 1
			T = 8
 		Case 18
			OpName = "RL D"
			d = RL(d)
			OpLength = 1
			T = 8
 		Case 19
			OpName = "RL E"
			e = RL(e)
			OpLength = 1
			T = 8
 		Case 20
			OpName = "RL H"
			h = RL(h)
			OpLength = 1
			T = 8
 		Case 21
			OpName = "RL L"
			l = RL(l)
			OpLength = 1
			T = 8
 		Case 22
			OpName = "RL (IX+d)"
			Memory.Write(IX + dIndex_1,RL(Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
 		Case 23
			OpName = "RL A"
			a = RL(a)
			OpLength = 1
			T = 8
 		Case 24
			OpName = "RR B"
			b = RR(b)
			OpLength = 1
			T = 8
 		Case 25
			OpName = "RR C"
			c = RR(c)
			OpLength = 1
			T = 8
 		Case 26
			OpName = "RR D"
			d = RR(d)
			OpLength = 1
			T = 8
 		Case 27
			OpName = "RRE"
			e = RR(e)
			OpLength = 1
			T = 8
 		Case 28
			OpName = "RR H"
			h = RR(h)
			OpLength = 1
			T = 8
 		Case 29
			OpName = "RR L"
			l = RR(l)
			OpLength = 1
			T = 8
 		Case 30
			OpName = "RR (IX+d)"
			Memory.Write(IX + dIndex_1,RR(Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
 		Case 31
			OpName = "RR A"
			a = RR(a)
			OpLength = 1
			T = 8
 		Case 32
			OpName = "SLA B"
			b = SLA(b)
			OpLength = 1
			T = 8
 		Case 33
			OpName = "SLA C"
			c = SLA(c)
			OpLength = 1
			T = 8
 		Case 34
			OpName = "SLA D"
			d = SLA(d)
			OpLength = 1
			T = 8
 		Case 35
			OpName = "SLA E"
			e = SLA(e)
			OpLength = 1
			T = 8
 		Case 36
			OpName = "SLA H"
			h = SLA(h)
			OpLength = 1
			T = 8
 		Case 37
			OpName = "SLA L"
			l = SLA(l)
			OpLength = 1
			T = 8
 		Case 38
			OpName = "SLA (IX+d)"
			Memory.Write(IX + dIndex_1,SLA(Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
 		Case 39
			OpName = "SLA A"
			a = SLA(a)
			OpLength = 1
			T = 8
 		Case 40
			OpName = "SRA B"
			b = SRA(b)
			OpLength = 1
			T = 8
 		Case 41
			OpName = "SRA C"
			c = SRA(c)
			OpLength = 1
			T = 8
 		Case 42
			OpName = "SRA D"
			d = SRA(d)
			OpLength = 1
			T = 8
 		Case 43
			OpName = "SRA E"
			e = SRA(e)
			OpLength = 1
			T = 8
 		Case 44
			OpName = "SRA H"
			h = SRA(h)
			OpLength = 1
			T = 8
 		Case 45
			OpName = "SRA L"
			l = SRA(l)
			OpLength = 1
			T = 8
 		Case 46
			OpName = "SRA (IX+d)"
			Memory.Write(IX + dIndex_1,SRA(Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
 		Case 47
			OpName = "SRA A"
			a = SRA(a)
			OpLength = 1
			T = 8
 		Case 56
			OpName = "SRL B"
			b = SRL(b)
			OpLength = 1
			T = 8
 		Case 57
			OpName = "SRL C"
			c = SRL(c)
			OpLength = 1
			T = 8
 		Case 58
			OpName = "SRL D"
			d = SRL(d)
			OpLength = 1
			T = 8
 		Case 59
			OpName = "SRL E"
			e = SRL(e)
			OpLength = 1
			T = 8
 		Case 60
			OpName = "SRL H"
			h = SRL(h)
			OpLength = 1
			T = 8
 		Case 61
			OpName = "SRL L"
			l = SRL(l)
			OpLength = 1
			T = 8
 		Case 62
			OpName = "SRL (IX + d)"
			Memory.Write(IX + dIndex_1,SRL(Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
 		Case 63
			OpName = "SRL A"
			a = SRL(a)
			OpLength = 1
			T = 8
		#End Region
		#Region Bit Check
		Case 64
			OpName = "BIT 0,B"
			BitCheck(0,b)
			OpLength = 1
			T = 8
		Case 65
			OpName = "BIT 0,C"
			BitCheck(0,c)
			OpLength = 1
			T = 8
		Case 66
			OpName = "BIT 0,D"
			BitCheck(0,d)
			OpLength = 1
			T = 8
		Case 67
			OpName = "BIT 0,E"
			BitCheck(0,e)
			OpLength = 1
			T = 8
		Case 68
			OpName = "BIT 0,H"
			BitCheck(0,h)
			OpLength = 1
			T = 8
		Case 69
			OpName = "BIT 0,L"
			BitCheck(0,l)
			OpLength = 1
			T = 8
		Case 70
			OpName = "BIT 0,(IX+d)"
			BitCheck(0,Memory.Read(IX + dIndex_1))
			OpLength = 2
			T = 20
		Case 71
			OpName = "BIT 0,A"
			BitCheck(0,a)
			OpLength = 1
			T = 8
		Case 72
			OpName = "BIT 1,B"
			BitCheck(1,b)
			OpLength = 1
			T = 8
		Case 73
			OpName = "BIT 1,C"
			BitCheck(1,c)
			OpLength = 1
		Case 74
			OpName = "BIT 1,D"
			BitCheck(1,d)
			OpLength = 1
			T = 8
		Case 75
			OpName = "BIT 1,E"
			BitCheck(1,e)
			OpLength = 1
			T = 8
		Case 76
			OpName = "BIT 1,H"
			BitCheck(1,h)
			OpLength = 1
			T = 8
		Case 77
			OpName = "BIT 1,L"
			BitCheck(1,l)
			OpLength = 1
			T = 8
		Case 78
			OpName = "BIT 1,(IX+d)"
			BitCheck(1,Memory.Read(IX + dIndex_1))
			OpLength = 2
			T = 20
		Case 79
			OpName = "BIT 1,A"
			BitCheck(1,a)
			OpLength = 1
			T = 8
		Case 80
			OpName = "BIT 2,B"
			BitCheck(2,b)
			OpLength = 1
			T = 8
		Case 81
			OpName = "BIT 2,C"
			BitCheck(2,c)
			OpLength = 1
			T = 8
		Case 82
			OpName = "BIT 2,D"
			BitCheck(2,d)
			OpLength = 1
			T = 8
		Case 83
			OpName = "BIT 2,E"
			BitCheck(2,e)
			OpLength = 1
			T = 8
		Case 84
			OpName = "BIT 2,H"
			BitCheck(2,h)
			OpLength = 1
			T = 8
		Case 85
			OpName = "BIT 2,L"
			BitCheck(2,l)
			OpLength = 1
			T = 8
		Case 86
			OpName = "BIT 2,(IX+d)"
			BitCheck(2,Memory.Read(IX + dIndex_1))
			OpLength = 2
			T = 20
		Case 87
			OpName = "BIT 2,A"
			BitCheck(2,a)
			OpLength = 1
			T = 8
		Case 88
			OpName = "BIT 3,B"
			BitCheck(3,b)
			OpLength = 1
			T = 8
		Case 89
			OpName = "BIT 3,C"
			BitCheck(3,c)
			OpLength = 1
			T = 8
		Case 90
			OpName = "BIT 3,D"
			BitCheck(3,d)
			OpLength = 1
			T = 8
		Case 91
			OpName = "BIT 3,E"
			BitCheck(3,e)
			OpLength = 1
			T = 8
		Case 92
			OpName = "BIT 3,H"
			BitCheck(3,h)
			OpLength = 1
			T = 8
		Case 93
			OpName = "BIT 3,L"
			BitCheck(3,l)
			OpLength = 1
			T = 8
		Case 94
			OpName = "BIT 3,(IX+d)"
			BitCheck(3,Memory.Read(IX + dIndex_1))
			OpLength = 2
			T = 20
		Case 95
			OpName = "BIT 3,A"
			BitCheck(3,a)
			OpLength = 1
			T = 8
		Case 96
			OpName = "BIT 4,B"
			BitCheck(4,b)
			OpLength = 1
			T = 8
		Case 97
			OpName = "BIT 4,C"
			BitCheck(4,c)
			OpLength = 1
			T = 8
		Case 98
			OpName = "BIT 4,D"
			BitCheck(4,d)
			OpLength = 1
			T = 8
		Case 99
			OpName = "BIT 4,E"
			BitCheck(4,e)
			OpLength = 1
			T = 8
		Case 100
			OpName = "BIT 4,H"
			BitCheck(4,h)
			OpLength = 1
			T = 8
		Case 101
			OpName = "BIT 4,L"
			BitCheck(4,l)
			OpLength = 1
			T = 8
		Case 102
			OpName = "BIT 4,(IX+d)"
			BitCheck(4,Memory.Read(IX + dIndex_1))
			OpLength = 2
			T = 20
		Case 103
			OpName = "BIT 4,A"
			BitCheck(4,a)
			OpLength = 1
			T = 8
		Case 104
			OpName = "BIT 5,B"
			BitCheck(5,b)
			OpLength = 1
			T = 8
		Case 105
			OpName = "BIT 5,C"
			BitCheck(5,c)
			OpLength = 1
			T = 8
		Case 106
			OpName = "BIT 5,D"
			BitCheck(5,d)
			OpLength = 1
			T = 8
		Case 107
			OpName = "BIT 5,E"
			BitCheck(5,e)
			OpLength = 1
			T = 8
		Case 108
			OpName = "BIT 5,H"
			BitCheck(5,h)
			OpLength = 1
			T = 8
		Case 109
			OpName = "BIT 5,L"
			BitCheck(5,l)
			OpLength = 1
			T = 8
		Case 110
			OpName = "BIT 5,(IX+d)"
			BitCheck(5,Memory.Read(IX + dIndex_1))
			OpLength = 2
			T = 20
		Case 111
			OpName = "BIT 5,A"
			BitCheck(5,a)
			OpLength = 1
			T = 8
		Case 112
			OpName = "BIT 6,B"
			BitCheck(6,b)
			OpLength = 1
			T = 8
		Case 113
			OpName = "BIT 6,C"
			BitCheck(6,c)
			OpLength = 1
			T = 8
		Case 114
			OpName = "BIT 6,D"
			BitCheck(6,d)
			OpLength = 1
			T = 8
		Case 115
			OpName = "BIT 6,E"
			BitCheck(6,e)
			OpLength = 1
			T = 8
		Case 116
			OpName = "BIT 6,H"
			BitCheck(6,h)
			OpLength = 1
			T = 8
		Case 117
			OpName = "BIT 6,L"
			BitCheck(6,l)
			OpLength = 1
			T = 8
		Case 118
			OpName = "BIT 6,(IX+d)"
			BitCheck(6,Memory.Read(IX + dIndex_1))
			OpLength = 2
			T = 20
		Case 119
			OpName = "BIT 6,A"
			BitCheck(6,a)
			OpLength = 1
			T = 8
		Case 120
			OpName = "BIT 7,B"
			BitCheck(7,b)
			OpLength = 1
			T = 8
		Case 121
			OpName = "BIT 7,C"
			BitCheck(7,c)
			OpLength = 1
			T = 8
		Case 122
			OpName = "BIT 7,D"
			BitCheck(7,d)
			OpLength = 1
			T = 8
		Case 123
			OpName = "BIT 7,E"
			BitCheck(7,e)
			OpLength = 1
			T = 8
		Case 124
			OpName = "BIT 7,H"
			BitCheck(7,h)
			OpLength = 1
			T = 8
		Case 125
			OpName = "BIT 7,L"
			BitCheck(7,l)
			OpLength = 1
			T = 8
		Case 126
			OpName = "BIT 7,(IX+d)"
			BitCheck(7,Memory.Read(IX + dIndex_1))
			OpLength = 2
			T = 20
		Case 127
			OpName = "BIT 7,A"
			BitCheck(7,a)
			OpLength = 1
			T = 8
		#End Region
		#Region Bit Reset
		Case 128
			OpName = "RES 0,B"
			b = BitReset(0,b)
			OpLength = 1
			T = 8
		Case 129
			OpName = "RES 0,C"
			c = BitReset(0,c)
			OpLength = 1
			T = 8
		Case 130
			OpName = "RES 0,D"
			d = BitReset(0,d)
			OpLength = 1
			T = 8
		Case 131
			OpName = "RES 0,E"
			e = BitReset(0,e)
			OpLength = 1
			T = 8
		Case 132
			OpName = "RES 0,H"
			h = BitReset(0,h)
			OpLength = 1
			T = 8
		Case 133
			OpName = "RES 0,L"
			l = BitReset(0,l)
			OpLength = 1
			T = 8
		Case 134
			OpName = "RES 0,(IX+d)"
			Memory.Write(IX + dIndex_1,BitReset(0,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 135
			OpName = "RES 0,A"
			a = BitReset(0,a)
			OpLength = 1
			T = 8
		Case 136
			OpName = "RES 1,B"
			b = BitReset(1,b)
			OpLength = 1
			T = 8
		Case 137
			OpName = "RES 1,C"
			c = BitReset(1,c)
			OpLength = 1
			T = 8
		Case 138
			OpName = "RES 1,D"
			d = BitReset(1,d)
			OpLength = 1
			T = 8
		Case 139
			OpName = "RES 1,E"
			e = BitReset(1,e)
			OpLength = 1
			T = 8
		Case 140
			OpName = "RES 1,H"
			h = BitReset(1,h)
			OpLength = 1
			T = 8
		Case 141
			OpName = "RES 1,L"
			l = BitReset(1,l)
			OpLength = 1
			T = 8
		Case 142
			OpName = "RES 1,(IX+d)"
			Memory.Write(IX + dIndex_1,BitReset(1,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 143
			OpName = "RES 1,A"
			a = BitReset(1,a)
			OpLength = 1
			T = 8
		Case 144
			OpName = "RES 2,B"
			b = BitReset(2,b)
			OpLength = 1
			T = 8
		Case 145
			OpName = "RES 2,C"
			c = BitReset(2,c)
			OpLength = 1
		Case 146
			OpName = "RES 2,D"
			d = BitReset(2,d)
			OpLength = 1
			T = 8
		Case 147
			OpName = "RES 2,E"
			e = BitReset(2,e)
			OpLength = 1
			T = 8
		Case 148
			OpName = "RES 2,H"
			h = BitReset(2,h)
			OpLength = 1
			T = 8
		Case 149
			OpName = "RES 2,L"
			l = BitReset(2,l)
			OpLength = 1
			T = 8
		Case 150
			OpName = "RES 2,(IX+2)"
			Memory.Write(IX + dIndex_1,BitReset(2,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 151
			OpName = "RES 2,A"
			a = BitReset(2,a)
			OpLength = 1
			T = 8
		Case 152
			OpName = "RES 3,B"
			b = BitReset(3,b)
			OpLength = 1
			T = 8
		Case 153
			OpName = "RES 3,C"
			c = BitReset(3,c)
			OpLength = 1
			T = 8
		Case 154
			OpName = "RES 3,D"
			d = BitReset(3,d)
			OpLength = 1
			T = 8
		Case 155
			OpName = "RES 3,E"
			e = BitReset(3,e)
			OpLength = 1
			T = 8
		Case 156
			OpName = "RES 3,H"
			h = BitReset(3,h)
			OpLength = 1
			T = 8
		Case 157
			OpName = "RES 3,L"
			l = BitReset(3,l)
			OpLength = 1
			T = 8
		Case 158
			OpName = "RES 3,(IX+d)"
			Memory.Write(IX + dIndex_1,BitReset(3,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 159
			OpName = "RES 3,A"
			a = BitReset(3,a)
			OpLength = 1
			T = 8
		Case 160
			OpName = "RES 4,B"
			b = BitReset(4,b)
			OpLength = 1
			T = 8
		Case 161
			OpName = "RES 4,C"
			c = BitReset(4,c)
			OpLength = 1
			T = 8
		Case 162
			OpName = "RES 4,D"
			d = BitReset(4,d)
			OpLength = 1
			T = 8
		Case 163
			OpName = "RES 4,E"
			e = BitReset(4,e)
			OpLength = 1
			T = 8
		Case 164
			OpName = "RES 4,H"
			h = BitReset(4,h)
			OpLength = 1
			T = 8
		Case 165
			OpName = "RES 4,L"
			l = BitReset(4,l)
			OpLength = 1
			T = 8
		Case 166
			OpName = "RES 4,(IX+d)"
			Memory.Write(IX + dIndex_1,BitReset(4,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 167
			OpName = "RES 4,A"
			a = BitReset(4,a)
			OpLength = 1
			T = 8
		Case 168
			OpName = "RES 5,B"
			b = BitReset(5,b)
			OpLength = 1
			T = 8
		Case 169
			OpName = "RES 5,C"
			c = BitReset(5,c)
			OpLength = 1
			T = 8
		Case 170
			OpName = "RES 5,D"
			d = BitReset(5,d)
			OpLength = 1
			T = 8
		Case 171
			OpName = "RES 5,E"
			e = BitReset(5,e)
			OpLength = 1
			T = 8
		Case 172
			OpName = "RES 5,H"
			h = BitReset(5,h)
			OpLength = 1
			T = 8
		Case 173
			OpName = "RES 5,L"
			l = BitReset(5,l)
			OpLength = 1
			T = 8
		Case 174
			OpName = "RES 5,(IX+d)"
			Memory.Write(IX + dIndex_1,BitReset(5,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 175
			OpName = "RES 5,A"
			a = BitReset(5,a)
			OpLength = 1
			T = 8
		Case 176
			OpName = "RES 6,B"
			b = BitReset(6,b)
			OpLength = 1
			T = 8
		Case 177
			OpName = "RES 6,C"
			c = BitReset(6,c)
			OpLength = 1
			T = 8
		Case 178
			OpName = "RES 6,D"
			d = BitReset(6,d)
			OpLength = 1
			T = 8
		Case 179
			OpName = "RES 6,E"
			e = BitReset(6,e)
			OpLength = 1
			T = 8
		Case 180
			OpName = "RES 6,H"
			h = BitReset(6,h)
			OpLength = 1
			T = 8
		Case 181
			OpName = "RES 6,L"
			l = BitReset(6,l)
			OpLength = 1
			T = 8
		Case 182
			OpName = "RES 6,(IX+d)"
			Memory.Write(IX + dIndex_1,BitReset(6,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 183
			OpName = "RES 6,A"
			a = BitReset(6,a)
			OpLength = 1
			T = 8
		Case 184
			OpName = "RES 7,B"
			b = BitReset(7,b)
			OpLength = 1
			T = 8
		Case 185
			OpName = "RES 7,C"
			c = BitReset(7,c)
			OpLength = 1
			T = 8
		Case 186
			OpName = "RES 7,D"
			d = BitReset(7,d)
			OpLength = 1
			T = 8
		Case 187
			OpName = "RES 7,E"
			e = BitReset(7,e)
			OpLength = 1
			T = 8
		Case 188
			OpName = "RES 7,H"
			h = BitReset(7,h)
			OpLength = 1
			T = 8
		Case 189
			OpName = "RES 7,L"
			l = BitReset(7,l)
			OpLength = 1
			T = 8
		Case 190
			OpName = "RES 7,(IX+d)"
			Memory.Write(IX + dIndex_1,BitReset(7,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 191
			OpName = "RES 7,A"
			a = BitReset(7,a)
			OpLength = 1	
			T = 8
		#End Region
		#Region Bit Set
		Case 192
			OpName = "SET 0,B"
			b = BitSet(0,b)
			OpLength = 1
			T = 8
		Case 193
			OpName = "SET 0,C"
			c = BitSet(0,c)
			OpLength = 1
			T = 8
		Case 194
			OpName = "SET 0,D"
			d = BitSet(0,d)
			OpLength = 1
			T = 8
		Case 195
			OpName = "SET 0,E"
			e = BitSet(0,e)
			OpLength = 1
			T = 8
		Case 196
			OpName = "SET 0,H"
			h = BitSet(0,h)
			OpLength = 1
			T = 8
		Case 197
			OpName = "SET 0,L"
			l = BitSet(0,l)
			OpLength = 1
			T = 8
		Case 198
			OpName = "SET 0,(IX+d)"
			Memory.Write(IX + dIndex_1,BitSet(0,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 199
			OpName = "SET 0,A"
			a = BitSet(0,a)
			OpLength = 1
			T = 8
		Case 200
			OpName = "SET 1,B"
			b = BitSet(1,b)
			OpLength = 1
			T = 8
		Case 201
			OpName = "SET 1,C"
			c = BitSet(1,c)
			OpLength = 1
			T = 8
		Case 202
			OpName = "SET 1,D"
			d = BitSet(1,d)
			OpLength = 1
			T = 8
		Case 203
			OpName = "SET 1,E"
			e = BitSet(1,e)
			OpLength = 1
			T = 8
		Case 204
			OpName = "SET 1,H"
			h = BitSet(1,h)
			OpLength = 1
			T = 8
		Case 205
			OpName = "SET 1,L"
			l = BitSet(1,l)
			OpLength = 1
			T = 8
		Case 206
			OpName = "SET 1,(IX+d)"
			Memory.Write(IX + dIndex_1,BitSet(1,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 207
			OpName = "SET 1,A"
			a = BitSet(1,a)
			OpLength = 1
			T = 8
		Case 208
			OpName = "SET 2,B"
			b = BitSet(2,b)
			OpLength = 1
			T = 8
		Case 209
			OpName = "SET 2,C"
			c = BitSet(2,c)
			OpLength = 1
			T = 8
		Case 210
			OpName = "SET 2,D"
			d = BitSet(2,d)
			OpLength = 1
			T = 8
		Case 211
			OpName = "SET 2,E"
			e = BitSet(2,e)
			OpLength = 1
			T = 8
		Case 212
			OpName = "SET 2,H"
			h = BitSet(2,h)
			OpLength = 1
			T = 8
		Case 213
			OpName = "SET 2,L"
			l = BitSet(2,l)
			OpLength = 1
			T = 8
		Case 214
			OpName = "SET 2,(IX+d)"
			Memory.Write(IX + dIndex_1,BitSet(2,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 215
			OpName = "SET 2,A"
			a = BitSet(2,a)
			OpLength = 1
			T = 8
		Case 216
			OpName = "SET 3,B"
			b = BitSet(3,b)
			OpLength = 1
			T = 8
		Case 217
			OpName = "SET 3,C"
			c = BitSet(3,c)
			OpLength = 1
			T = 8
		Case 218
			OpName = "SET 3,D"
			d = BitSet(3,d)
			OpLength = 1
			T = 8
		Case 219
			OpName = "SET 3,E"
			e = BitSet(3,e)
			OpLength = 1
			T = 8
		Case 220
			OpName = "SET 3,H"
			h = BitSet(3,h)
			OpLength = 1
			T = 8
		Case 221
			OpName = "SET 3,L"
			l = BitSet(3,l)
			OpLength = 1
			T = 8
		Case 222
			OpName = "SET 3,(IX+d)"
			Memory.Write(IX + dIndex_1,BitSet(3,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 223
			OpName = "SET 3,A"
			a = BitSet(3,a)
			OpLength = 1
			T = 8
		Case 224
			OpName = "SET 4,B"
			b = BitSet(4,b)
			OpLength = 1
			T = 8
		Case 225
			OpName = "SET 4,C"
			c = BitSet(4,c)
			OpLength = 1
			T = 8
		Case 226
			OpName = "SET 4,D"
			d = BitSet(4,d)
			OpLength = 1
			T = 8
		Case 227
			OpName = "SET 4,E"
			e = BitSet(4,e)
			OpLength = 1
			T = 8
		Case 228
			OpName = "SET 4,H"
			h = BitSet(4,h)
			OpLength = 1
			T = 8
		Case 229
			OpName = "SET 4,L"
			l = BitSet(4,l)
			OpLength = 1
			T = 8
		Case 230
			OpName = "SET 4,(IX+d)"
			Memory.Write(IX + dIndex_1,BitSet(4,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 231
			OpName = "SET 4,A"
			a = BitSet(4,a)
			OpLength = 1
			T = 8
		Case 232
			OpName = "SET 5,B"
			b = BitSet(5,b)
			OpLength = 1
			T = 8
		Case 233
			OpName = "SET 5,C"
			c = BitSet(5,c)
			OpLength = 1
			T = 8
		Case 234
			OpName = "SET 5,D"
			d = BitSet(5,d)
			OpLength = 1
			T = 8
		Case 235
			OpName = "SET 5,E"
			e = BitSet(5,e)
			OpLength = 1
			T = 8
		Case 236
			OpName = "SET 5,H"
			h = BitSet(5,h)
			OpLength = 1
			T = 8
		Case 237
			OpName = "SET 5,L"
			l = BitSet(5,l)
			OpLength = 1
			T = 8
		Case 238
			OpName = "SET 5,(IX+d)"
			Memory.Write(IX + dIndex_1,BitSet(5,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 239
			OpName = "SET 5,A"
			a = BitSet(5,a)
			OpLength = 1
			T = 8
		Case 240
			OpName = "SET 6,B"
			b = BitSet(6,b)
			OpLength = 1
			T = 8
		Case 241
			OpName = "SET 6,C"
			c = BitSet(6,c)
			OpLength = 1
			T = 8
		Case 242
			OpName = "SET 6,D"
			d = BitSet(6,d)
			OpLength = 1
			T = 8
		Case 243
			OpName = "SET 6,E"
			e = BitSet(6,e)
			OpLength = 1
			T = 8
		Case 244
			OpName = "SET 6,H"
			h = BitSet(6,h)
			OpLength = 1
			T = 8
		Case 245
			OpName = "SET 6,L"
			l = BitSet(6,l)
			OpLength = 1
			T = 8
		Case 246
			OpName = "SET 6,(IX+d)"
			Memory.Write(IX + dIndex_1,BitSet(6,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 247
			OpName = "SET 6,A"
			a = BitSet(6,a)
			OpLength = 1	
			T = 8
		Case 248
			OpName = "SET 7,B"
			b = BitSet(7,b)
			OpLength = 1
			T = 8
		Case 249
			OpName = "SET 7,C"
			c = BitSet(7,c)
			OpLength = 1
			T = 8
		Case 250
			OpName = "SET 7,D"
			d = BitSet(7,d)
			OpLength = 1
			T = 8
		Case 251
			OpName = "SET 7,E"
			e = BitSet(7,e)
			OpLength = 1
			T = 8
		Case 252
			OpName = "SET 7,H"
			h = BitSet(7,h)
			OpLength = 1
			T = 8
		Case 253
			OpName = "SET 7,L"
			l = BitSet(7,l)
			OpLength = 1
			T = 8
		Case 254
			OpName = "SET 7,(IX+d)"
			Memory.Write(IX + dIndex_1,BitSet(7,Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 255
			OpName = "SET 7,A"
			a = BitSet(7,a)
			OpLength = 1
			T = 8
		#End Region
		Case Else
			OpName = ""
			Logger.logsWARN("Missing OpCode DDCB + " & OpCode)
		End Select
End Sub
Private Sub CBAfterFDOpCodes 'TODO: Fix all undocumented cases
	'R is incremented twice for every FDCB opcode. This has already been done when we get here
	Dim FDCBOpcode As Int
	FDCBOpcode = Memory.Read(PC + 1) 'The actual opode is at PC + 1, while the d-index is at PC.
	Select FDCBOpcode
		#Region Shift
		Case 0
			OpName = "RLC B"
			b = RLC(b)
			OpLength = 1
			T = 8
		Case 1
			OpName = "RLC (IY+d)> C"
			Memory.Write(IX + dIndex_1,RLC(Memory.Read(IX + dIndex_1)))
			c = Memory.Read(IY + dIndex_1)
			OpLength = 2
			T = 23
		Case 2
			OpName = "RLC D"
			d = RLC(d)
			OpLength = 1
			T = 8
		Case 3
			OpName = "RLC E"
			e = RLC(e)
			OpLength = 1
			T = 8
		Case 4
			OpName = "RLC H"
			h = RLC(h)
			OpLength = 1
			T = 8
		Case 5
			OpName = "RLC L"
			l = RLC(l)
			OpLength = 1
			T = 8
		Case 6
			OpName = "RLC (IY+d)"
			Memory.Write(IX + dIndex_1,RLC(Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 7
			OpName = "RLC A"
			a = RLC(a)
			OpLength = 1
			T = 8
		Case 8
			OpName = "RRC B"
			b = RRC(b)
			OpLength = 1
			T = 8
		Case 9
			OpName = "RRC C"
			c = RRC(c)
			OpLength = 1
			T = 8
		Case 10
			OpName = "RRC D"
			d = RRC(d)
			OpLength = 1
			T = 8
		Case 11
			OpName = "RRC E"
			e = RRC(e)
			OpLength = 1
			T = 8
		Case 12
			OpName = "RRC H"
			h = RRC(h)
			OpLength = 1
			T = 8
		Case 13
			OpName = "RRC L"
			l = RRC(l)
			OpLength = 1
			T = 8
		Case 14
			OpName = "RRC (IY + d)"
			Memory.Write(IX + dIndex_1,RRC(Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 15
			OpName = "RRC A"
			a = RRC(a)
			OpLength = 1
			T = 8
		Case 16
			OpName = "RL B"
			b = RL(b)
			OpLength = 1
			T = 8
		Case 17
			OpName = "RL C"
			c = RL(c)
			OpLength = 1
			T = 8
		Case 18
			OpName = "RL D"
			d = RL(d)
			OpLength = 1
			T = 8
		Case 19
			OpName = "RL E"
			e = RL(e)
			OpLength = 1
			T = 8
		Case 20
			OpName = "RL H"
			h = RL(h)
			OpLength = 1
			T = 8
		Case 21
			OpName = "RL L"
			l = RL(l)
			OpLength = 1
			T = 8
		Case 22
			OpName = "RL (IY+d)"
			Memory.Write(IX + dIndex_1,RL(Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 23
			OpName = "RL A"
			a = RL(a)
			OpLength = 1
			T = 8
		Case 24
			OpName = "RR B"
			b = RR(b)
			OpLength = 1
			T = 8
		Case 25
			OpName = "RR C"
			c = RR(c)
			OpLength = 1
			T = 8
		Case 26
			OpName = "RR D"
			d = RR(d)
			OpLength = 1
			T = 8
		Case 27
			OpName = "RR E"
			e = RR(e)
			OpLength = 1
			T = 8
		Case 28
			OpName = "RR H"
			h = RR(h)
			OpLength = 1
			T = 8
		Case 29
			OpName = "RR L"
			l = RR(l)
			OpLength = 1
			T = 8
		Case 30
			OpName = "RR (IY+d)"
			Memory.Write(IX + dIndex_1,RR(Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 31
			OpName = "RR A"
			a = RR(a)
			OpLength = 1
			T = 8
		Case 32
			OpName = "SLA B"
			b = SLA(b)
			OpLength = 1
			T = 8
		Case 33
			OpName = "SLA C"
			c = SLA(c)
			OpLength = 1
			T = 8
		Case 34
			OpName = "SLA D"
			d = SLA(d)
			OpLength = 1
			T = 8
		Case 35
			OpName = "SLA E"
			e = SLA(e)
			OpLength = 1
			T = 8
		Case 36
			OpName = "SLA H"
			h = SLA(h)
			OpLength = 1
			T = 8
		Case 37
			OpName = "SLA L"
			l = SLA(l)
			OpLength = 1
			T = 8
		Case 38
			OpName = "SLA (IY+d)"
			Memory.Write(IX + dIndex_1,SLA(Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 39
			OpName = "SLA A"
			a = SLA(a)
			OpLength = 1
			T = 8
		Case 40
			OpName = "SRA B"
			b = SRA(b)
			OpLength = 1
			T = 8
		Case 41
			OpName = "SRA C"
			c = SRA(c)
			OpLength = 1
			T = 8
		Case 42
			OpName = "SRA D"
			d = SRA(d)
			OpLength = 1
			T = 8
		Case 43
			OpName = "SRA E"
			e = SRA(e)
			OpLength = 1
			T = 8
		Case 44
			OpName = "SRA H"
			h = SRA(h)
			OpLength = 1
			T = 8
		Case 45
			OpName = "SRA L"
			l = SRA(l)
			OpLength = 1
			T = 8
		Case 46
			OpName = "SRA (IY+d)"
			Memory.Write(IX + dIndex_1,SRA(Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 47
			OpName = "SRA A"
			a = SRA(a)
			OpLength = 1
			T = 8
		Case 56
			OpName = "SRL B"
			b = SRL(b)
			OpLength = 1
			T = 8
		Case 57
			OpName = "SRL C"
			c = SRL(c)
			OpLength = 1
			T = 8
		Case 58
			OpName = "SRL D"
			d = SRL(d)
			OpLength = 1
			T = 8
		Case 59
			OpName = "SRL E"
			e = SRL(e)
			OpLength = 1
			T = 8
		Case 60
			OpName = "SRL H"
			h = SRL(h)
			OpLength = 1
			T = 8
		Case 61
			OpName = "SRL L"
			l = SRL(l)
			OpLength = 1
			T = 8
		Case 62
			OpName = "SRL (IY + d)"
			Memory.Write(IX + dIndex_1,SRL(Memory.Read(IX + dIndex_1)))
			OpLength = 2
			T = 23
		Case 63
			OpName = "SRL A"
			a = SRL(a)
			OpLength = 1
			T = 8
		#End Region
		#Region Bit Check
		Case 64
			OpName = "BIT 0,B"
			BitCheck(0,b)
			OpLength = 1
			T = 8
		Case 65
			OpName = "BIT 0,C"
			BitCheck(0,c)
			OpLength = 1
			T = 8
		Case 66
			OpName = "BIT 0,D"
			BitCheck(0,d)
			OpLength = 1
			T = 8
		Case 67
			OpName = "BIT 0,E"
			BitCheck(0,e)
			OpLength = 1
			T = 8
		Case 68
			OpName = "BIT 0,H"
			BitCheck(0,h)
			OpLength = 1
			T = 8
		Case 69
			OpName = "BIT 0,L"
			BitCheck(0,l)
			OpLength = 1
			T = 8
		Case 70
			OpName = "BIT 0,(IY+d)"
			BitCheck(0,Memory.Read(IY + dIndex_1))
			OpLength = 2
			T = 20
		Case 71
			OpName = "BIT 0,A"
			BitCheck(0,a)
			OpLength = 1
			T = 8
		Case 72
			OpName = "BIT 1,B"
			BitCheck(1,b)
			OpLength = 1
		Case 73
			OpName = "BIT 1,C"
			BitCheck(1,c)
			OpLength = 1
			T = 8
		Case 74
			OpName = "BIT 1,D"
			BitCheck(1,d)
			OpLength = 1
			T = 8
		Case 75
			OpName = "BIT 1,E"
			BitCheck(1,e)
			OpLength = 1
			T = 8
		Case 76
			OpName = "BIT 1,H"
			BitCheck(1,h)
			OpLength = 1
			T = 8
		Case 77
			OpName = "BIT 1,L"
			BitCheck(1,l)
			OpLength = 1
			T = 8
		Case 78
			OpName = "BIT 1,(IY+d)"
			BitCheck(1,Memory.Read(IY + dIndex_1))
			OpLength = 2
			T = 20
		Case 79
			OpName = "BIT 1,A"
			BitCheck(1,a)
			OpLength = 1
			T = 8
		Case 80
			OpName = "BIT 2,B"
			BitCheck(2,b)
			OpLength = 1
			T = 8
		Case 81
			OpName = "BIT 2,C"
			BitCheck(2,c)
			OpLength = 1
			T = 8
		Case 82
			OpName = "BIT 2,D"
			BitCheck(2,d)
			OpLength = 1
			T = 8
		Case 83
			OpName = "BIT 2,E"
			BitCheck(2,e)
			OpLength = 1
			T = 8
		Case 84
			OpName = "BIT 2,H"
			BitCheck(2,h)
			OpLength = 1
			T = 8
		Case 85
			OpName = "BIT 2,L"
			BitCheck(2,l)
			OpLength = 1
			T = 8
		Case 86
			OpName = "BIT 2,(IY+d)"
			BitCheck(2,Memory.Read(IY + dIndex_1))
			OpLength = 2
			T = 20
		Case 87
			OpName = "BIT 2,A"
			BitCheck(2,a)
			OpLength = 1
			T = 8
		Case 88
			OpName = "BIT 3,B"
			BitCheck(3,b)
			OpLength = 1
			T = 8
		Case 89
			OpName = "BIT 3,C"
			BitCheck(3,c)
			OpLength = 1
			T = 8
		Case 90
			OpName = "BIT 3,D"
			BitCheck(3,d)
			OpLength = 1
			T = 8
		Case 91
			OpName = "BIT 3,E"
			BitCheck(3,e)
			OpLength = 1
			T = 8
		Case 92
			OpName = "BIT 3,H"
			BitCheck(3,h)
			OpLength = 1
			T = 8
		Case 93
			OpName = "BIT 3,L"
			BitCheck(3,l)
			OpLength = 1
			T = 8
		Case 94
			OpName = "BIT 3,(IY+d)"
			BitCheck(3,Memory.Read(IY + dIndex_1))
			OpLength = 2
			T = 20
		Case 95
			OpName = "BIT 3,A"
			BitCheck(3,a)
			OpLength = 1
			T = 8
		Case 96
			OpName = "BIT 4,B"
			BitCheck(4,b)
			OpLength = 1
			T = 8
		Case 97
			OpName = "BIT 4,C"
			BitCheck(4,c)
			OpLength = 1
			T = 8
		Case 98
			OpName = "BIT 4,D"
			BitCheck(4,d)
			OpLength = 1
			T = 8
		Case 99
			OpName = "BIT 4,E"
			BitCheck(4,e)
			OpLength = 1
			T = 8
		Case 100
			OpName = "BIT 4,H"
			BitCheck(4,h)
			OpLength = 1
			T = 8
		Case 101
			OpName = "BIT 4,L"
			BitCheck(4,l)
			OpLength = 1
			T = 8
		Case 102
			OpName = "BIT 4,(IY+d)"
			BitCheck(4,Memory.Read(IY + dIndex_1))
			OpLength = 2
			T = 20
		Case 103
			OpName = "BIT 4,A"
			BitCheck(4,a)
			OpLength = 1
			T = 8
		Case 104
			OpName = "BIT 5,B"
			BitCheck(5,b)
			OpLength = 1
			T = 8
		Case 105
			OpName = "BIT 5,C"
			BitCheck(5,c)
			OpLength = 1
			T = 8
		Case 106
			OpName = "BIT 5,D"
			BitCheck(5,d)
			OpLength = 1
			T = 8
		Case 107
			OpName = "BIT 5,E"
			BitCheck(5,e)
			OpLength = 1
			T = 8
		Case 108
			OpName = "BIT 5,H"
			BitCheck(5,h)
			OpLength = 1
			T = 8
		Case 109
			OpName = "BIT 5,L"
			BitCheck(5,l)
			OpLength = 1
			T = 8
		Case 110
			OpName = "BIT 5,(IY+d)"
			BitCheck(5,Memory.Read(IY + dIndex_1))
			OpLength = 2
			T = 20
		Case 111
			OpName = "BIT 5,A"
			BitCheck(5,a)
			OpLength = 1
			T = 8
		Case 112
			OpName = "BIT 6,B"
			BitCheck(6,b)
			OpLength = 1
			T = 8
		Case 113
			OpName = "BIT 6,C"
			BitCheck(6,c)
			OpLength = 1
			T = 8
		Case 114
			OpName = "BIT 6,D"
			BitCheck(6,d)
			OpLength = 1
			T = 8
		Case 115
			OpName = "BIT 6,E"
			BitCheck(6,e)
			OpLength = 1
			T = 8
		Case 116
			OpName = "BIT 6,H"
			BitCheck(6,h)
			OpLength = 1
			T = 8
		Case 117
			OpName = "BIT 6,L"
			BitCheck(6,l)
			OpLength = 1
			T = 8
		Case 118
			OpName = "BIT 6,(IY+d)"
			BitCheck(6,Memory.Read(IY + dIndex_1))
			OpLength = 2
			T = 20
		Case 119
			OpName = "BIT 6,A"
			BitCheck(6,a)
			OpLength = 1
			T = 8
		Case 120
			OpName = "BIT 7,B"
			BitCheck(7,b)
			OpLength = 1
			T = 8
		Case 121
			OpName = "BIT 7,C"
			BitCheck(7,c)
			OpLength = 1
			T = 8
		Case 122
			OpName = "BIT 7,D"
			BitCheck(7,d)
			OpLength = 1
			T = 8
		Case 123
			OpName = "BIT 7,E"
			BitCheck(7,e)
			OpLength = 1
			T = 8
		Case 124
			OpName = "BIT 7,H"
			BitCheck(7,h)
			OpLength = 1
			T = 8
		Case 125
			OpName = "BIT 7,L"
			BitCheck(7,l)
			OpLength = 1
			T = 8
		Case 126
			OpName = "BIT 7,(IY+d)"
			BitCheck(7,Memory.Read(IY + dIndex_1))
			OpLength = 2
			T = 20
		Case 127
			OpName = "BIT 7,A"
			BitCheck(7,a)
			OpLength = 1
			T = 8
		#End Region
		#Region Bit Reset
		Case 128
			OpName = "RES 0,B"
			b = BitReset(0,b)
			OpLength = 1
			T = 8
		Case 129
			OpName = "RES 0,C"
			c = BitReset(0,c)
			OpLength = 1
			T = 8
		Case 130
			OpName = "RES 0,D"
			d = BitReset(0,d)
			OpLength = 1
			T = 8
		Case 131
			OpName = "RES 0,E"
			e = BitReset(0,e)
			OpLength = 1
			T = 8
		Case 132
			OpName = "RES 0,H"
			h = BitReset(0,h)
			OpLength = 1
			T = 8
		Case 133
			OpName = "RES 0,L"
			l = BitReset(0,l)
			OpLength = 1
			T = 8
		Case 134
			OpName = "RES 0,(IY+d)"
			Memory.Write(IY + dIndex_1,BitReset(0,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 135
			OpName = "RES 0,A"
			a = BitReset(0,a)
			OpLength = 1
			T = 8
		Case 136
			OpName = "RES 1,B"
			b = BitReset(1,b)
			OpLength = 1
			T = 8
		Case 137
			OpName = "RES 1,C"
			c = BitReset(1,c)
			OpLength = 1
			T = 8
		Case 138
			OpName = "RES 1,D"
			d = BitReset(1,d)
			OpLength = 1
			T = 8
		Case 139
			OpName = "RES 1,E"
			e = BitReset(1,e)
			OpLength = 1
			T = 8
		Case 140
			OpName = "RES 1,H"
			h = BitReset(1,h)
			OpLength = 1
			T = 8
		Case 141
			OpName = "RES 1,L"
			l = BitReset(1,l)
			OpLength = 1
			T = 8
		Case 142
			OpName = "RES 1,(IY+d)"
			Memory.Write(IY + dIndex_1,BitReset(1,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 143
			OpName = "RES 1,A"
			a = BitReset(1,a)
			OpLength = 1
			T = 8
		Case 144
			OpName = "RES 2,B"
			b = BitReset(2,b)
			OpLength = 1
			T = 8
		Case 145
			OpName = "RES 2,C"
			c = BitReset(2,c)
			OpLength = 1
			T = 8
		Case 146
			OpName = "RES 2,D"
			d = BitReset(2,d)
			OpLength = 1
			T = 8
		Case 147
			OpName = "RES 2,E"
			e = BitReset(2,e)
			OpLength = 1
			T = 8
		Case 148
			OpName = "RES 2,H"
			h = BitReset(2,h)
			OpLength = 1
			T = 8
		Case 149
			OpName = "RES 2,L"
			l = BitReset(2,l)
			OpLength = 1
			T = 8
		Case 150
			OpName = "RES 2,(IY+2)"
			Memory.Write(IY + dIndex_1,BitReset(2,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 151
			OpName = "RES 2,A"
			a = BitReset(2,a)
			OpLength = 1
			T = 8
		Case 152
			OpName = "RES 3,B"
			b = BitReset(3,b)
			OpLength = 1
			T = 8
		Case 153
			OpName = "RES 3,C"
			c = BitReset(3,c)
			OpLength = 1
			T = 8
		Case 154
			OpName = "RES 3,D"
			d = BitReset(3,d)
			OpLength = 1
			T = 8
		Case 155
			OpName = "RES 3,E"
			e = BitReset(3,e)
			OpLength = 1
			T = 8
		Case 156
			OpName = "RES 3,H"
			h = BitReset(3,h)
			OpLength = 1
			T = 8
		Case 157
			OpName = "RES 3,L"
			l = BitReset(3,l)
			OpLength = 1
			T = 8
		Case 158
			OpName = "RES 3,(IY+d)"
			Memory.Write(IY + dIndex_1,BitReset(3,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 159
			OpName = "RES 3,A"
			a = BitReset(3,a)
			OpLength = 1
			T = 8
		Case 160
			OpName = "RES 4,B"
			b = BitReset(4,b)
			OpLength = 1
			T = 8
		Case 161
			OpName = "RES 4,C"
			c = BitReset(4,c)
			OpLength = 1
			T = 8
		Case 162
			OpName = "RES 4,D"
			d = BitReset(4,d)
			OpLength = 1
			T = 8
		Case 163
			OpName = "RES 4,E"
			e = BitReset(4,e)
			OpLength = 1
			T = 8
		Case 164
			OpName = "RES 4,H"
			h = BitReset(4,h)
			OpLength = 1
			T = 8
		Case 165
			OpName = "RES 4,L"
			l = BitReset(4,l)
			OpLength = 1
			T = 8
		Case 166
			OpName = "RES 4,(IY+d)"
			Memory.Write(IY + dIndex_1,BitReset(4,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 167
			OpName = "RES 4,A"
			a = BitReset(4,a)
			OpLength = 1
			T = 8
		Case 168
			OpName = "RES 5,B"
			b = BitReset(5,b)
			OpLength = 1
			T = 8
		Case 169
			OpName = "RES 5,C"
			c = BitReset(5,c)
			OpLength = 1
			T = 8
		Case 170
			OpName = "RES 5,D"
			d = BitReset(5,d)
			OpLength = 1
			T = 8
		Case 171
			OpName = "RES 5,E"
			e = BitReset(5,e)
			OpLength = 1
			T = 8
		Case 172
			OpName = "RES 5,H"
			h = BitReset(5,h)
			OpLength = 1
			T = 8
		Case 173
			OpName = "RES 5,L"
			l = BitReset(5,l)
			OpLength = 1
			T = 8
		Case 174
			OpName = "RES 5,(IY+d)"
			Memory.Write(IY + dIndex_1,BitReset(5,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 175
			OpName = "RES 5,A"
			a = BitReset(5,a)
			OpLength = 1
			T = 8
		Case 176
			OpName = "RES 6,B"
			b = BitReset(6,b)
			OpLength = 1
			T = 8
		Case 177
			OpName = "RES 6,C"
			c = BitReset(6,c)
			OpLength = 1
			T = 8
		Case 178
			OpName = "RES 6,D"
			d = BitReset(6,d)
			OpLength = 1
			T = 8
		Case 179
			OpName = "RES 6,E"
			e = BitReset(6,e)
			OpLength = 1
			T = 8
		Case 180
			OpName = "RES 6,H"
			h = BitReset(6,h)
			OpLength = 1
			T = 8
		Case 181
			OpName = "RES 6,L"
			l = BitReset(6,l)
			OpLength = 1
			T = 8
		Case 182
			OpName = "RES 6,(IY+d)"
			Memory.Write(IY + dIndex_1,BitReset(6,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 183
			OpName = "RES 6,A"
			a = BitReset(6,a)
			OpLength = 1
			T = 8
		Case 184
			OpName = "RES 7,B"
			b = BitReset(7,b)
			OpLength = 1
			T = 8
		Case 185
			OpName = "RES 7,C"
			c = BitReset(7,c)
			OpLength = 1
			T = 8
		Case 186
			OpName = "RES 7,D"
			d = BitReset(7,d)
			OpLength = 1
			T = 8
		Case 187
			OpName = "RES 7,E"
			e = BitReset(7,e)
			OpLength = 1
			T = 8
		Case 188
			OpName = "RES 7,H"
			h = BitReset(7,h)
			OpLength = 1
			T = 8
		Case 189
			OpName = "RES 7,L"
			l = BitReset(7,l)
			OpLength = 1
			T = 8
		Case 190
			OpName = "RES 7,(IY+d)"
			Memory.Write(IY + dIndex_1,BitReset(7,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 191
			OpName = "RES 7,A"
			a = BitReset(7,a)
			OpLength = 1
			T = 8
		#End Region
		#Region Bit Set
		Case 192
			OpName = "SET 0,B"
			b = BitSet(0,b)
			OpLength = 1
			T = 8
		Case 193
			OpName = "SET 0,C"
			c = BitSet(0,c)
			OpLength = 1
			T = 8
		Case 194
			OpName = "SET 0,D"
			d = BitSet(0,d)
			OpLength = 1
			T = 8
		Case 195
			OpName = "SET 0,E"
			e = BitSet(0,e)
			OpLength = 1
			T = 8
		Case 196
			OpName = "SET 0,H"
			h = BitSet(0,h)
			OpLength = 1
			T = 8
		Case 197
			OpName = "SET 0,L"
			l = BitSet(0,l)
			OpLength = 1
			T = 8
		Case 198
			OpName = "SET 0,(IY+d)"
			Memory.Write(IY + dIndex_1,BitSet(0,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 199
			OpName = "SET 0,A"
			a = BitSet(0,a)
			OpLength = 1
			T = 8
		Case 200
			OpName = "SET 1,B"
			b = BitSet(1,b)
			OpLength = 1
			T = 8
		Case 201
			OpName = "SET 1,C"
			c = BitSet(1,c)
			OpLength = 1
			T = 8
		Case 202
			OpName = "SET 1,D"
			d = BitSet(1,d)
			OpLength = 1
			T = 8
		Case 203
			OpName = "SET 1,E"
			e = BitSet(1,e)
			OpLength = 1
			T = 8
		Case 204
			OpName = "SET 1,H"
			h = BitSet(1,h)
			OpLength = 1
			T = 8
		Case 205
			OpName = "SET 1,L"
			l = BitSet(1,l)
			OpLength = 1
			T = 8
		Case 206
			OpName = "SET 1,(IY+d)"
			Memory.Write(IY + dIndex_1,BitSet(1,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 207
			OpName = "SET 1,A"
			a = BitSet(1,a)
			OpLength = 1
			T = 8
		Case 208
			OpName = "SET 2,B"
			b = BitSet(2,b)
			OpLength = 1
			T = 8
		Case 209
			OpName = "SET 2,C"
			c = BitSet(2,c)
			OpLength = 1
			T = 8
		Case 210
			OpName = "SET 2,D"
			d = BitSet(2,d)
			OpLength = 1
			T = 8
		Case 211
			OpName = "SET 2,E"
			e = BitSet(2,e)
			OpLength = 1
			T = 8
		Case 212
			OpName = "SET 2,H"
			h = BitSet(2,h)
			OpLength = 1
			T = 8
		Case 213
			OpName = "SET 2,L"
			l = BitSet(2,l)
			OpLength = 1
			T = 8
		Case 214
			OpName = "SET 2,(IY+d)"
			Memory.Write(IY + dIndex_1,BitSet(2,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 215
			OpName = "SET 2,A"
			a = BitSet(2,a)
			OpLength = 1
			T = 8
		Case 216
			OpName = "SET 3,B"
			b = BitSet(3,b)
			OpLength = 1
			T = 8
		Case 217
			OpName = "SET 3,C"
			c = BitSet(3,c)
			OpLength = 1
			T = 8
		Case 218
			OpName = "SET 3,D"
			d = BitSet(3,d)
			OpLength = 1
			T = 8
		Case 219
			OpName = "SET 3,E"
			e = BitSet(3,e)
			OpLength = 1
			T = 8
		Case 220
			OpName = "SET 3,H"
			h = BitSet(3,h)
			OpLength = 1
			T = 8
		Case 221
			OpName = "SET 3,L"
			l = BitSet(3,l)
			OpLength = 1
			T = 8
		Case 222
			OpName = "SET 3,(IY+d)"
			Memory.Write(IY + dIndex_1,BitSet(3,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 223
			OpName = "SET 3,A"
			a = BitSet(3,a)
			OpLength = 1
			T = 8
		Case 224
			OpName = "SET 4,B"
			b = BitSet(4,b)
			OpLength = 1
			T = 8
		Case 225
			OpName = "SET 4,C"
			c = BitSet(4,c)
			OpLength = 1
			T = 8
		Case 226
			OpName = "SET 4,D"
			d = BitSet(4,d)
			OpLength = 1
			T = 8
		Case 227
			OpName = "SET 4,E"
			e = BitSet(4,e)
			OpLength = 1
			T = 8
		Case 228
			OpName = "SET 4,H"
			h = BitSet(4,h)
			OpLength = 1
			T = 8
		Case 229
			OpName = "SET 4,L"
			l = BitSet(4,l)
			OpLength = 1
			T = 8
		Case 230
			OpName = "SET 4,(IY+d)"
			Memory.Write(IY + dIndex_1,BitSet(4,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 231
			OpName = "SET 4,A"
			a = BitSet(4,a)
			OpLength = 1
			T = 8
		Case 232
			OpName = "SET 5,B"
			b = BitSet(5,b)
			OpLength = 1
			T = 8
		Case 233
			OpName = "SET 5,C"
			c = BitSet(5,c)
			OpLength = 1
			T = 8
		Case 234
			OpName = "SET 5,D"
			d = BitSet(5,d)
			OpLength = 1
			T = 8
		Case 235
			OpName = "SET 5,E"
			e = BitSet(5,e)
			OpLength = 1
			T = 8
		Case 236
			OpName = "SET 5,H"
			h = BitSet(5,h)
			OpLength = 1
			T = 8
		Case 237
			OpName = "SET 5,L"
			l = BitSet(5,l)
			OpLength = 1
			T = 8
		Case 238
			OpName = "SET 5,(IY+d)"
			Memory.Write(IY + dIndex_1,BitSet(5,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 239
			OpName = "SET 5,A"
			a = BitSet(5,a)
			OpLength = 1
			T = 8
		Case 240
			OpName = "SET 6,B"
			b = BitSet(6,b)
			OpLength = 1
			T = 8
		Case 241
			OpName = "SET 6,C"
			c = BitSet(6,c)
			OpLength = 1
			T = 8
		Case 242
			OpName = "SET 6,D"
			d = BitSet(6,d)
			OpLength = 1
			T = 8
		Case 243
			OpName = "SET 6,E"
			e = BitSet(6,e)
			OpLength = 1
			T = 8
		Case 244
			OpName = "SET 6,H"
			h = BitSet(6,h)
			OpLength = 1
			T = 8
		Case 245
			OpName = "SET 6,L"
			l = BitSet(6,l)
			OpLength = 1
			T = 8
		Case 246
			OpName = "SET 6,(IY+d)"
			Memory.Write(IY + dIndex_1,BitSet(6,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 247
			OpName = "SET 6,A"
			a = BitSet(6,a)
			OpLength = 1
			T = 8
		Case 248
			OpName = "SET 7,B"
			b = BitSet(7,b)
			OpLength = 1
			T = 8
		Case 249
			OpName = "SET 7,C"
			c = BitSet(7,c)
			OpLength = 1
			T = 8
		Case 250
			OpName = "SET 7,D"
			d = BitSet(7,d)
			OpLength = 1
			T = 8
		Case 251
			OpName = "SET 7,E"
			e = BitSet(7,e)
			OpLength = 1
			T = 8
		Case 252
			OpName = "SET 7,H"
			h = BitSet(7,h)
			OpLength = 1
			T = 8
		Case 253
			OpName = "SET 7,L"
			l = BitSet(7,l)
			OpLength = 1
			T = 8
		Case 254
			OpName = "SET 7,(IY+d)"
			Memory.Write(IY + dIndex_1,BitSet(7,Memory.Read(IY + dIndex_1)))
			OpLength = 2
			T = 23
		Case 255
			OpName = "SET 7,A"
			a = BitSet(7,a)
			OpLength = 1
			T = 8
		#End Region
		Case Else
			OpName = ""
			Logger.logsWARN("Missing OpCode FDCB + " & OpCode)
	End Select
End Sub

Private Sub EDOpCodes
	IncR 'R is incremented twice for every ED opcode. This is the second increment.
	Select OpCode
		Case 64
			OpName = "IN B,(C)"
			b = In(c)
			OpLength = 1
			T = 12
		Case 65
			OpName = "OUT (C),B"
			OutPort.Process(c,b)
			OpLength = 1
			T = 12
		Case 66
			OpName = "SBC HL,BC"
			Temp = SBC16(HL, BC)
			h = HighByte(Temp)
			l = LowByte(Temp)
			OpLength = 1
			T = 15
		Case 67
			OpName = "LD (NN),BC"
			Memory.Write(NN + 1,b)
			Memory.Write(NN,c)
			OpLength = 3
			T = 20
		Case 68
			OpName = "NEG"
			Temp = 0 - a
			SubstractFlag = 1
			SetCarryFlag(Temp)
			SetSignFlag(Temp)
			Temp = Bit.AND(Temp,255) 'Drop Carry bit (bit 8)
			SetZeroFlag(Temp)
			'Overflow after a SUB is when the result has the wrong sign bit (bit 7)
			'A negative number substracted from a positive number can't give a negative result.
			'A positive number substracted from a negative number can't give a positive result.
			If Bit.AND(128,(Bit.Xor(0,a))) = 128 Then 'There may be oveflow if the operands have different values at Bit 7
				If Bit.AND(128,(Bit.Xor(0,Temp))) = 128 Then 'There is overflow if the left operand and the result have different values at bit 7
					Parity_OverflowFlag = 1
				Else
					Parity_OverflowFlag = 0
				End If
			Else
				Parity_OverflowFlag = 0
			End If
			HalfCarryFlag = Bit.AND(16,(Bit.AND(0,15) - Bit.AND(a,15)))/16
			a = Temp
			OpLength = 1
			T = 8
		Case 69
			OpName = "RETN"
			'TODO Return from non maskable interrupt
			PC = Pop16
			OpLength = 0
			T = 14
		Case 70
			OpName = "IM 0"
			InterruptMode = 0
			OpLength = 1
			T = 8
		Case 71
			OpName = "LD I,A"
			i = a
			OpLength = 1
			T = 4
		Case 72
			OpName = "IN C,(C)"
			c = In(c)
			OpLength = 1
			T = 12
		Case 73
			OpName = "OUT (C),C"
			OutPort.Process(c,c)
			OpLength = 1
			T = 12
		Case 74
			OpName = "ADC HL,BC"
			Temp = ADC16(HL,BC)
			h = HighByte(Temp)
			l = LowByte(Temp)
			OpLength = 1
			T = 15
		Case 75
			OpName = "LD BC,(NN)"
			b = Memory.Read(NN + 1)
			c = Memory.Read(NN)
			OpLength = 3
			T = 20
		Case 77
			OpName = "RETI"
			PC = Pop16
			OpLength = 0
			T = 14
		Case 79
			OpName = "LD R,A"
			r = a
			OpLength = 1
			T = 9
		Case 80
			OpName = "IN D,(C)"
			d = In(c)
			OpLength = 1
			T = 12
		Case 81
			OpName = "OUT (C),D"
			OutPort.Process(c,d)
			OpLength = 1
			T = 12
		Case 82
			OpName = "SBC HL,DE"
			Temp = SBC16(HL,DE)
			h = HighByte(Temp)
			l = LowByte(Temp)
			OpLength = 1
			T = 15
		Case 83
			OpName = "LD (NN),DE"
			Memory.Write(NN + 1,d)
			Memory.Write(NN,e)
			OpLength = 3
			T = 20
		Case 86
			OpName = "IM 1"
			InterruptMode = 1
			OpLength = 1
			T = 8
		Case 87
			OpName = "LD A,I"
			a = i
			OpLength = 1
			T = 9
		Case 88
			OpName = "IN E,(C)"
			e = In(c)
			OpLength = 1
			T = 12
		Case 89
			OpName = "OUT (C),E"
			OutPort.Process(c,e)
			OpLength = 1
			T = 12
		Case 90
			OpName = "ADC HL,DE"
			Temp = ADC16(HL,DE)
			h = HighByte(Temp)
			l = LowByte(Temp)
			OpLength = 1
			T = 15
		Case 91
			OpName = "LD DE,(NN)"
			d = Memory.Read(NN + 1)
			e = Memory.Read(NN)
			OpLength = 3
			T = 20
		Case 94
			OpName = "IM 2"
			InterruptMode = 2
			OpLength = 1
			T = 8
		Case 95
			OpName = "LD A,R"
			a = r
			OpLength = 1
			T = 9
		Case 96
			OpName = "IN H,(C)"
			h = In(c)
			OpLength = 1
			T = 12
		Case 97
			OpName = "OUT (C),H"
			OutPort.Process(c,h)
			OpLength = 1
			T = 12
		Case 98
			OpName = "SBC HL,HL"
			Temp = SBC16(HL,HL)
			h = HighByte(Temp)
			l = LowByte(Temp)
			OpLength = 1
			T = 4
		Case 99
			OpName = "LD (NN),HL"
			Memory.Write(NN + 1,h)
			Memory.Write(NN,l)
			OpLength = 3
			T = 15
		Case 103
			OpName = "RRD"
			'Verified
			'	--------<----------
			'1	2 ----->---- 3 >-- 4
			'   A             (HL)
			Dim alow, ahigh,inhl As Int
			inhl = Memory.Read(HL)
			ahigh = Bit.AND(a,0xF0)
			alow = Bit.AND(a,0x0F)

			a = Bit.OR(ahigh,Bit.AND(inhl,0xF))
			inhl = Bit.AND(Bit.OR(Bit.ShiftLeft(alow,4),Bit.ShiftRight(inhl,4)),0xFF)
			Memory.Write(HL,inhl)
			
			'Note that flags are set on result in a
			'CarryFlag is unaffected
			SetZeroFlag(a)
			Parity_OverflowFlag = Parity(a)
			SetSignFlag(a)
			SubstractFlag = 0
			HalfCarryFlag = 0
			T = 18
		Case 104
			OpName = "IN L,(C)"
			l = In(c)
			OpLength = 1
			T = 12
		Case 105
			OpName = "OUT (C),L"
			OutPort.Process(c,l)
			OpLength = 1
			T = 12
		Case 106
			OpName = "ADC HL,HL"
			Temp = ADC16(HL, HL)
			h = HighByte(Temp)
			l = LowByte(Temp)
			OpLength = 1
			T = 15
		Case 107
			OpName = "LD HL,(NN)"
			h = Memory.Read(NN + 1)
			l = Memory.Read(NN)
			OpLength = 3
			T = 16
		Case 111
			OpName = "RLD"
			'Verified
			'	-------->----------
			'1	2 -----<---- 3 <-- 4
			'   A             (HL)
			Dim alow, ahigh,inhl As Int
			inhl = Memory.Read(HL)
			ahigh = Bit.AND(a,0xF0)
			alow = Bit.AND(a,0x0F)
			a = Bit.OR(ahigh,Bit.ShiftRight(inhl,4))
			inhl = Bit.AND(Bit.OR(Bit.ShiftLeft(inhl,4),alow),0xFF)
			Memory.Write(HL,inhl)
			'Note that flags are set on result in a
			'CarryFlag is unaffected
			SetZeroFlag(a)
			Parity_OverflowFlag = Parity(a)
			SetSignFlag(a)
			SubstractFlag = 0
			HalfCarryFlag = 0
			OpLength = 1
			T = 18
		Case 112
			OpName = "IN F,(C)"
			f = In(c)
			OpLength = 1
			T = 12
		Case 114
			OpName = "SBC HL,SP"
			Temp = SBC16(HL,SP)
			h = HighByte(Temp)
			l = LowByte(Temp)
			OpLength = 1
			T = 15
		Case 115
			OpName = "LD (NN),SP"
			Memory.Write(NN + 1,HighByte(SP))
			Memory.Write(NN,LowByte(SP))
			OpLength = 3
			T = 20
		Case 120
			OpName = "IN A,(C)"
			a = In(c)
			OpLength = 1
			T = 12
		Case 121
			OpName = "OUT (C),A"
			OutPort.Process(c,a)
			OpLength = 1
			T = 12
		Case 122
			OpName = "ADC HL,SP"
			Temp = ADC16(HL,SP)
			h = HighByte(Temp)
			l = LowByte(Temp)
			OpLength = 1
			T = 15
		Case 123
			OpName = "LD SP,(NN)"
			SP = Memory.Read16(NN)
			OpLength = 3
			T = 20
		Case 160
			OpName = "LDI"
			LDI
			OpLength = 1
			T = 16
		Case 161
			OpName = "CPI"
			CPI_
			OpLength = 1
			T = 16
		Case 162
			OpName = "INI"
			'Reads the (C) port and writes the result to (HL), then increments HL and decrements B.
			Memory.Write(HL,In(c))
			'Increment HL
			Temp = INC16(HL)
			h = HighByte(Temp)
			l = LowByte(Temp)
			b = Bit.AND(b - 1,255) 'B = B - 1. Remove any carry bit 8 
			'C is preserved, the N flag is set. S, H and P/V are undefined.
			'Z is set if B becomes zero after decrementing, otherwise it is reset.
			SetZeroFlag(b)
			SubstractFlag = 1 'According to Zilog User's Manual
			OpLength = 1
			T = 16
		Case 163
			'TODO: Check this!
			OpName = "OUTI"
			'Reads from (HL) and writes to the (C) port. HL is then incremented, and B is decremented.
			OutPort.Process(c,Memory.Read(HL))
			Temp = INC16(HL)
			h = HighByte(Temp)
			l = LowByte(Temp)
			b = Bit.AND(b - 1,255) 'B = B - 1. Remove any carry bit 8 
			SetZeroFlag(b)
			SubstractFlag = 0
			OpLength = 1
			T = 16
		Case 168
			OpName = "LDD"
			LDD
			OpLength = 1
			T = 16
		Case 169
			OpName = "CPD"
			CPD
			OpLength = 1
			T = 16
		Case 170
			OpName = "IND"
			IND
			OpLength = 1
			T = 16
		Case 171
			OpName = "OUTD"
			'Writes the value from (HL) to the (C) port, then decrements B and HL.
			Temp = Memory.Read(HL)
			OutPort.Process(c,Temp)
			Temp = DEC16(HL)
			h = HighByte(Temp)
			l = LowByte(Temp)
			b = Bit.AND(b - 1,255) 'B = B - 1. Remove any carry bit 8 
			'C is preserved, the N flag is set. S, H and P/V are undefined.
			'Z is set if B becomes zero after decrementing, otherwise it is reset.
			SetZeroFlag(b)
			SubstractFlag = 1
			OpLength = 1
			T = 16
		Case 176
			OpName = "LDIR"
			'Repeats LDI (LD (DE),(HL), then increments DE, HL, and decrements BC) until BC=0.
			'Note that if BC=0 before this instruction is called, it will loop around until BC=0 again.
			LDI
			If BC > 0 Then
				T = 21
				OpLength = -1 'The operation is repeated until BC = 0. OpLength = -1 so that the ED prefix can be fetched.
			Else
				T = 16
				OpLength = 1
			End If
		Case 177
			'Repeats CPI until either BC = 0 or A = (HL)
			OpName = "CPIR" 'OK
			CPI_
			If BC > 0 AND ZeroFlag <> 1 Then
				T = 21
				OpLength = -1 'The operation is repeated until BC = 0 or A = (HL). OpLength = -1 so that the ED prefix can be fetched.
			Else
				T = 16
				OpLength = 1
			End If
		Case 178
			OpName = "INIR"
			'TODO
			OpLength = 1
			T = 4
		Case 179
			OpName = "OTIR"
			'TODO
			OpLength = 1
			T = 4
		Case 184
			OpName = "LDDR"
			'Repeats the instruction LDD (Does a LD (DE),(HL) and decrements each of DE, HL, and BC) until BC=0.
			'Note that if BC=0 before the start of the routine, it will try loop around until BC=0 again.
			LDD
			If BC > 0 Then
				T = 21
				OpLength = -1 'The operation is repeated until BC = 0. OpLength = -1 so that the ED prefix can be fetched.
			Else
				T = 16
				OpLength = 1
			End If
		Case 185
			OpName = "CPDR" 'OK
			'Repeats CPD until either BC = 0 or A = (HL)
			CPD
			If BC > 0 AND ZeroFlag <> 1 Then
				T = 21
				OpLength = -1 'The operation is repeated until BC = 0 or A = (HL). OpLength = -1 so that the ED prefix can be fetched.
			Else
				T = 16
				OpLength = 1
			End If
		Case 186
			OpName = "INDR"
			'Repeats IND until B = 0
			IND
			If B > 0 Then
				T = 21
				OpLength = -1 'The operation is repeated until B = 0. OpLength = -1 so that the ED prefix can be fetched.
			Else
				T = 16
				OpLength = 1
			End If
		Case 187
			OpName = "OTDR"
			'TODO
			OpLength = 1
			T = 4
		Case Else
			OpName = ""
			OpLength = 1
			Logger.logsWARN("Missing OpCode ED + " & OpCode)
	End Select
End Sub
Private Sub DDOpCodes
	IncR 'R is incremented twice for every DD opcode. This is the second increment.
	Select OpCode
		Case 9
			OpName = "ADD IX,BC"
			Temp = ADD16(IX, BC)
			i1 = HighByte(Temp)
			x = LowByte(Temp)
			OpLength = 1
			T = 15
		Case 25
			OpName = "ADD IX,DE"
			Temp = ADD16(IX, DE)
			i1 = HighByte(Temp)
			x = LowByte(Temp)
			OpLength = 1
			T = 15
		Case 33
			OpName = "LD IX,NN"
			i1 = Nhigh
			x = N
			OpLength = 3
			T = 14
		Case 34
			OpName = "LD (NN),IX"
			Memory.Write(NN + 1,i1)
			Memory.Write(NN,x)
			OpLength = 3
			T = 20
		Case 35
			OpName = "INC IX"
			Temp = INC16(IX)
			i1 = HighByte(Temp)
			x = LowByte(Temp)
			OpLength = 1
			T = 10
		Case 36
			OpName = "INC IXH"
			i1 = INC(i1)
			OpLength = 1
			T = 4
		Case 37
			OpName = "DEC IXH"
			i1 = DEC(i1)
			OpLength = 1
			T = 4
		Case 38
			OpName = "LD IXH,N"
			i1 = N
			OpLength = 2
			T = 7
		Case 41
			OpName = "ADD IX,IX"
			Temp = ADD16(IX,IX)
			i1 = HighByte(Temp)
			x = LowByte(Temp)
			OpLength = 1
			T = 15
		Case 42
			OpName = "LD IX,(NN)"
			i1 = Memory.Read(NN + 1)
			x = Memory.Read(NN)
			OpLength = 3
			T = 20
		Case 43
			OpName = "DEC IX"
			Temp = DEC16(IX)
			i1 = HighByte(Temp)
			x = LowByte(Temp)
			OpLength = 1
			T = 10
		Case 44
			OpName = "INC IXL"
			x = INC(x)
			OpLength = 1
			T = 4
		Case 45
			OpName = "DEC IXL"
			x = DEC(x)
			OpLength = 1
			T = 4
		Case 46
			OpName = "LD IXL,N"
			x = N
			OpLength = 2
			T = 7
		Case 52
			OpName = "INC (IX+d)"
			Memory.Write(IX + dIndex,INC(Memory.Read(IX + dIndex)))
			OpLength = 2
			T = 23
		Case 53
			OpName = "DEC (IX+d)"
			Memory.Write(IX + dIndex,DEC(Memory.Read(IX + dIndex)))
			OpLength = 2
			T = 23
		Case 54
			OpName = "LD (IX+d),N"
			Memory.Write(IX + dIndex,Memory.Read(PC + 2))
			OpLength = 3
			T = 19
		Case 57
			OpName = "ADD IX,SP"
			Temp = ADD16(IX, SP)
			i1 = HighByte(Temp)
			x = LowByte(Temp)
			OpLength = 1
			T = 15
		Case 68
			OpName = "LD B,IXH"
			b = i1
			OpLength = 1
			T = 4
		Case 69
			OpName = "LD B,IXL"
			b = x
			OpLength = 1
			T = 4
		Case 70
			OpName = "LD B,(IX+d)"
			b = Memory.Read(IX + dIndex)
			OpLength = 2
			T = 19
		Case 76
			OpName = "LD C,IXH"
			c = i1
			OpLength = 1
			T = 4
		Case 77
			OpName = "LD C,IXL"
			c = x
			OpLength = 1
			T = 4
		Case 78
			OpName = "LD C,(IX+d)"
			c = Memory.Read(IX + dIndex)
			OpLength = 2
			T = 19
		Case 84
			OpName = "LD D,IXH"
			d = i1
			OpLength = 1
			T = 4
		Case 85
			OpName = "LD D,IXL"
			d = x
			OpLength = 1
			T = 4
		Case 86
			OpName = "LD D,(IX+d)"
			d = Memory.Read(IX + dIndex)
			OpLength = 2
			T = 19
		Case 92
			OpName = "LD E,IXH"
			e = i1
			OpLength = 1
			T = 4
		Case 93
			OpName = "LD E,IXL"
			e = x
			OpLength = 1
			T = 4
		Case 94
			OpName = "LD E,(IX+d)"
			e = Memory.Read(IX + dIndex)
			OpLength = 2
			T = 19
		Case 96
			OpName = "LD IXH,B"
			i1 = b
			OpLength = 1
			T = 4
		Case 97
			OpName = "LD IXH,C"
			i1 = c
			OpLength = 1
			T = 4
		Case 98
			OpName = "LD IXH,D"
			i1 = d
			OpLength = 1
			T = 4
		Case 99
			OpName = "LD IXH,E"
			i1 = e
			OpLength = 1
			T = 4
		Case 100
			OpName = "LD IXH,IXH"
			i1 = i1
			OpLength = 1
			T = 4
		Case 101
			OpName = "LD IXH,IXL"
			i1 = x
			OpLength = 1
			T = 4
		Case 102
			OpName = "LD H,(IX+d)"
			h = Memory.Read(IX + dIndex)
			OpLength = 2
			T = 19
		Case 103
			OpName = "LD IXH,A"
			i1 = a
			OpLength = 1
			T = 4
		Case 104
			OpName = "LD IXL,B"
			x = b
			OpLength = 1
			T = 4
		Case 105
			OpName = "LD IXL,C"
			x = c
			OpLength = 1
			T = 4
		Case 106
			OpName = "LD IXL,D"
			x = d
			OpLength = 1
			T = 4
		Case 107
			OpName = "LD IXL,E"
			x = e
			OpLength = 1
			T = 4
		Case 108
			OpName = "LD IXL,IXH"
			x = i1
			OpLength = 1
			T = 4
		Case 109
			OpName = "LD IXL,IXL"
			x = x
			OpLength = 1
			T = 4
		Case 110
			OpName = "LD L,(IX+d)"
			l = Memory.Read(IX + dIndex)
			OpLength = 2
			T = 19
		Case 111
			OpName = "LD IXL,A"
			x = a
			OpLength = 1
			T = 4
		Case 112
			OpName = "LD (IX+d),B"
			Memory.Write(IX + dIndex,b)
			OpLength = 2
			T = 19
		Case 113
			OpName = "LD (IX+d),C"
			Memory.Write(IX + dIndex,c)
			OpLength = 2
			T = 19
		Case 114
			OpName = "LD (IX+d),D"
			Memory.Write(IX + dIndex,d)
			OpLength = 2
			T = 19
		Case 115
			OpName = "LD (IX+d),E"
			Memory.Write(IX + dIndex,e)
			OpLength = 2
			T = 19
		Case 116
			OpName = "LD (IX+d),H"
			Memory.Write(IX + dIndex,h)
			OpLength = 2
			T = 19
		Case 117
			OpName = "LD (IX+d),L"
			Memory.Write(IX + dIndex,l)
			OpLength = 2
			T = 19
		Case 119
			OpName = "LD (IX+d),A"
			Memory.Write(IX + dIndex,a)
			OpLength = 2
			T = 19
		Case 124
			OpName = "LD A,IXH"
			a = i1
			OpLength = 1
			T = 4
		Case 125
			OpName = "LD A,IXL"
			a = x
			OpLength = 1
			T = 4
		Case 126
			OpName = "LD A,(IX+d)"
			a = Memory.Read(IX + dIndex)
			OpLength = 2
			T = 19
		Case 132
			OpName = "ADD A,IXH"
			a = ADD(a,i1)
			OpLength = 1
			T = 4
		Case 133
			OpName = "ADD A,IXL"
			a = ADD(a,x)
			OpLength = 1
			T = 4
		Case 134
			OpName = "ADD A,(IX+d)"
			a = ADD(a,Memory.Read(IX + dIndex))
			OpLength = 2
			T = 4
		Case 140
			OpName = "ADC A,IXH"
			a = ADC(a,i1)
			OpLength = 1
			T = 4
		Case 141
			OpName = "ADC A,IXL"
			a = ADC(a,x)
			OpLength = 1
			T = 4
		Case 142
			OpName = "ADC A,(IX+d)"
			a = ADC(a,Memory.Read(IX + dIndex))
			OpLength = 2
			T = 19
		Case 148
			OpName = "SUB IXH"
			a = SUB_(a,i1)				
			OpLength = 1
			T = 4
		Case 149
			OpName = "SUB IXL"
			a = SUB_(a,x)				
			OpLength = 1
			T = 4
		Case 150
			OpName = "SUB (IX+d)"
			a = SUB_(a,Memory.Read(IX + dIndex))
			OpLength = 2
			T = 19
		Case 156
			OpName = "SBC A,IXH"
			a = SBC(a,i1)				
			OpLength = 1
			T = 4
		Case 157
			OpName = "SBC A,IXL"
			a = SBC(a,x)				
			OpLength = 1
			T = 4
		Case 158
			OpName = "SBC A,(IX+d)"
			a = SBC(a,Memory.Read(IX + dIndex))
			OpLength = 2
			T = 19
		Case 164
			OpName = "AND IXH"
			a = AND_(a,i1)				
			OpLength = 1
			T = 4
		Case 165
			OpName = "AND IXL"
			a = AND_(a,x)				
			OpLength = 1
			T = 4
		Case 166
			OpName = "AND (IX+d)"
			a = AND_(a,Memory.Read(IX + dIndex))
			OpLength = 2
			T = 19
		Case 172
			OpName = "XOR IXH"
			a = XOR_(a,i1)				
			OpLength = 1
			T = 4
		Case 173
			OpName = "XOR IXL"
			a = XOR_(a,x)				
			OpLength = 1
			T = 4
		Case 174
			OpName = "XOR (IX+d)"
			a = XOR_(a,Memory.Read(IX + dIndex))
			OpLength = 2
			T = 19
		Case 180
			OpName = "OR IXH"
			a = OR_(a,i1)				
			OpLength = 1
			T = 4
		Case 181
			OpName = "OR IXL"
			a = OR_(a,x)				
			OpLength = 1
			T = 4
		Case 182
			OpName = "OR (IX+d)"
			a = OR_(a,Memory.Read(IX + dIndex))
			OpLength = 2
			T = 19
		Case 188
			OpName = "CP IXH"
			SUB_(a,i1)				
			OpLength = 1
			T = 4
		Case 189
			OpName = "CP IXL"
			SUB_(a,x)				
			OpLength = 1
			T = 4
		Case 190
			OpName = "CP (IX+d)"
			SUB_(a,Memory.Read(IX + dIndex))
			OpLength = 2
			T = 19
		Case 203
			OpName = "CB"
			OpLength = 1
		Case 225
			OpName = "POP IX"
			Temp = Pop16
			i1 = HighByte(Temp)
			x = LowByte(Temp)
			OpLength = 1
			T = 14
		Case 227
			OpName = "EX (SP),IX"
			Temp = Memory.Read(SP)
			Memory.Write(SP,x)
			x = Temp
			Temp = Memory.Read(SP + 1)
			Memory.Write(SP + 1,i1)
			i1 = Temp
			OpLength = 1
			T = 23
		Case 229
			OpName = "PUSH IX"
			Push16(IX)
			OpLength = 1
			T = 15
		Case 233
			OpName = "JP (IX)"
			PC = IX
			OpLength = 0
			T = 8
		Case 237
			OpName = "ED"
			OpLength = 1
		Case 249
			OpName = "LD SP,IX"
			SP = IX
			OpLength = 1
			T = 10
		Case Else
			OpName = ""
			Logger.logsWARN("Missing OpCode DD + " & OpCode)
	End Select
End Sub
Private Sub FDOpCodes
	IncR 'R is incremented twice for every FD opcode. This is the second increment.
	Select OpCode
		Case 9 'OK
			OpName = "ADD IY,BC"
			Temp = ADD16(IY, BC)
			i2 = HighByte(Temp)
			y = LowByte(Temp)
			OpLength = 1
			T = 15
		Case 25 'OK
			OpName = "ADD IY,DE"
			Temp = ADD16(IY, DE)
			i2 = HighByte(Temp)
			y = LowByte(Temp)
			OpLength = 1
			T = 15
		Case 33 'OK
			OpName = "LD IY,NN"
			i2 = Nhigh
			y = N
			OpLength = 3
			T = 14
		Case 34
			OpName = "LD (NN),IY"
			Memory.Write(NN + 1,i2)
			Memory.Write(NN,y)
			OpLength = 3
			T = 20
		Case 35
			OpName = "INC IY"
			Temp = INC16(IY)
			i2 = HighByte(Temp)
			y = LowByte(Temp)
			OpLength = 1
			T = 10
		Case 36
			OpName = "INC IYH"
			i2 = INC(i2)
			OpLength = 1
			T = 4
		Case 37
			OpName = "DEC IYH"
			i2 = DEC(i2)
			OpLength = 1
			T = 4
		Case 38
			OpName = "LD IYH,N"
			i2 = N
			OpLength = 2
			T = 4
		Case 41
			OpName = "ADD IY,IY"
			Temp = ADD16(IY,IY)
			i2 = HighByte(Temp)
			y = LowByte(Temp)
			OpLength = 1
			T = 15
		Case 42
			OpName = "LD IY,(NN)"
			i2 = Memory.Read(NN + 1)
			y = Memory.Read(NN)
			OpLength = 3
			T = 20
		Case 43
			OpName = "DEC IY"
			Temp = DEC16(IY)
			i2 = HighByte(Temp)
			y = LowByte(Temp)
			OpLength = 1
			T = 10
		Case 44
			OpName = "INC IYL"
			y = INC(y)
			OpLength = 1
			T = 4
		Case 45
			OpName = "DEC IYL"
			y = DEC(y)
			OpLength = 1
			T = 4
		Case 46
			OpName = "LD IYL,N"
			y = N
			OpLength = 2
			T = 7
		Case 52
			OpName = "INC (IY+d)"
			Memory.Write(IY + dIndex,INC(Memory.Read(IY + dIndex)))
			OpLength = 2
			T = 23
		Case 53
			OpName = "DEC (IY+d)"
			Memory.Write(IY + dIndex,DEC(Memory.Read(IY + dIndex)))
			OpLength = 2
			T = 23
		Case 54
			OpName = "LD (IY+d),N"
			Memory.Write(IY + dIndex,Memory.Read(PC + 2))
			OpLength = 3
			T = 19
		Case 57
			OpName = "ADD IY,SP"
			Temp = ADD16(IY, SP)
			i2 = HighByte(Temp)
			y = LowByte(Temp)
			OpLength = 1
			T = 15
		Case 68
			OpName = "LD B,IYH"
			b = i2
			OpLength = 1
			T = 4
		Case 69
			OpName = "LD B,IYL"
			b = y
			OpLength = 1
			T = 4
		Case 70
			OpName = "LD B,(IY+d)"
			b = Memory.Read(IY + dIndex)
			OpLength = 2
			T = 19
		Case 76
			OpName = "LD C,IYH"
			c = i2
			OpLength = 1
			T = 4
		Case 77
			OpName = "LD C,IYL"
			c = y
			T = 4
		Case 78
			OpName = "LD C,(IY+d)"
			c = Memory.Read(IY + dIndex)
			OpLength = 2
			T = 19
		Case 84
			OpName = "LD D,IYH"
			d = i2
			OpLength = 1
			T = 4
		Case 85
			OpName = "LD D,IYL"
			d = y
			OpLength = 1
			T = 4
		Case 86
			OpName = "LD D,(IY+d)"
			d = Memory.Read(IY + dIndex)
			OpLength = 2
			T = 19
		Case 92
			OpName = "LD E,IYH"
			e = i2
			OpLength = 1
			T = 4
		Case 93
			OpName = "LD E,IYL"
			e = y
			OpLength = 1
			T = 4
		Case 94
			OpName = "LD E,(IY+d)"
			e = Memory.Read(IY + dIndex)
			OpLength = 2
			T = 19
		Case 96
			OpName = "LD IYH,B"
			i2 = b
			OpLength = 1
			T = 4
		Case 97
			OpName = "LD IYH,C"
			i2 = c
			OpLength = 1
			T = 4
		Case 98
			OpName = "LD IYH,D"
			i2 = d
			OpLength = 1
			T = 4
		Case 99
			OpName = "LD IYH,E"
			i2 = e
			OpLength = 1
			T = 4
		Case 100
			OpName = "LD IYH,IYH"
			i2 = i2
			OpLength = 1
			T = 4
		Case 101
			OpName = "LD IYH,IYL"
			i2 = y
			OpLength = 1
			T = 4
		Case 102
			OpName = "LD H,(IY+d)"
			h = Memory.Read(IY + dIndex)
			OpLength = 2
			T = 19
		Case 103
			OpName = "LD IYH,A"
			i2 = a
			OpLength = 1
			T = 4
		Case 104
			OpName = "LD IYL,B"
			y = b
			OpLength = 1
			T = 4
		Case 105
			OpName = "LD IYL,C"
			y = c
			OpLength = 1
			T = 4
		Case 106
			OpName = "LD IYL,D"
			y = d
			OpLength = 1
			T = 4
		Case 107
			OpName = "LD IYL,E"
			y = e
			OpLength = 1
			T = 4
		Case 108
			OpName = "LD IYL,IYH"
			y = i2
			OpLength = 1
			T = 4
		Case 109
			OpName = "LD IYL,IYL"
			y = y
			OpLength = 1
			T = 4
		Case 110
			OpName = "LD L,(IY+d)"
			l = Memory.Read(IY + dIndex)
			OpLength = 2
			T = 19
		Case 111
			OpName = "LD IYL,A"
			y = a
			OpLength = 1
			T = 4
		Case 112
			OpName = "LD (IY+d),B"
			Memory.Write(IY + dIndex,b)
			OpLength = 2
			T = 19
		Case 113
			OpName = "LD (IY+d),C"
			Memory.Write(IY + dIndex,c)
			OpLength = 2
			T = 19
		Case 114
			OpName = "LD (IY+d),D"
			Memory.Write(IY + dIndex,d)
			OpLength = 2
			T = 19
		Case 115
			OpName = "LD (IY+d),E"
			Memory.Write(IY + dIndex,e)
			OpLength = 2
			T = 19
		Case 116
			OpName = "LD (IY+d),H"
			Memory.Write(IY + dIndex,h)
			OpLength = 2
			T = 19
		Case 117
			OpName = "LD (IY+d),L"
			Memory.Write(IY + dIndex,l)
			OpLength = 2
			T = 19
		Case 119
			OpName = "LD (IY+d),A"
			Memory.Write(IY + dIndex,a)
			OpLength = 2
			T = 19
		Case 124
			OpName = "LD A,IYH"
			a = i2
			OpLength = 1
			T = 4
		Case 125
			OpName = "LD A,IYL"
			a = y
			OpLength = 1
			T = 4
		Case 126
			OpName = "LD A,(IY+d)"
			a = Memory.Read(IY + dIndex)
			OpLength = 2
			T = 19
		Case 132
			OpName = "ADD A,IYH"
			a = ADD(a,i2)
			OpLength = 1
			T = 4
		Case 133
			OpName = "ADD A,IYL"
			a = ADD(a,y)
			OpLength = 1
			T = 4
		Case 134
			OpName = "ADD A,(IY+d)"
			a = ADD(a,Memory.Read(IY + dIndex))
			OpLength = 2
			T = 19
		Case 140
			OpName = "ADC A,IYH"
			a = ADC(a,i2)
			OpLength = 1
			T = 4
		Case 141
			OpName = "ADC A,IYL"
			a = ADC(a,y)
			OpLength = 1
			T = 4
		Case 142
			OpName = "ADC A,(IY+d)"
			a = ADC(a,Memory.Read(IY + dIndex))
			OpLength = 2
			T = 4
		Case 148
			OpName = "SUB IYH"
			a = SUB_(a,i2)				
			OpLength = 1
			T = 4
		Case 149
			OpName = "SUB IYL"
			a = SUB_(a,y)				
			OpLength = 1
			T = 4
		Case 150
			OpName = "SUB (IY+d)"
			a = SUB_(a,Memory.Read(IY + dIndex))
			OpLength = 2
			T = 19
		Case 156
			OpName = "SBC A,IYH"
			a = SBC(a,i2)				
			OpLength = 1
			T = 4
		Case 157
			OpName = "SBC A,IYL"
			a = SBC(a,y)				
			OpLength = 1
			T = 4
		Case 158
			OpName = "SBC A,(IY+d)"
			a = SBC(a,Memory.Read(IY + dIndex))
			OpLength = 2
			T = 19
		Case 164
			OpName = "AND IYH"
			a = AND_(a,i2)				
			OpLength = 1
			T = 4
		Case 165
			OpName = "AND IYL"
			a = AND_(a,y)				
			OpLength = 1
			T = 4
		Case 166
			OpName = "AND (IY+d)"
			a = AND_(a,Memory.Read(IY + dIndex))
			OpLength = 2
			T = 19
		Case 172
			OpName = "XOR IYH"
			a = XOR_(a,i2)				
			OpLength = 1
			T = 4
		Case 173
			OpName = "XOR IYL"
			a = XOR_(a,y)				
			OpLength = 1
			T = 4
		Case 174
			OpName = "XOR (IY+d)"
			a = XOR_(a,Memory.Read(IY + dIndex))
			OpLength = 2
			T = 19
		Case 180
			OpName = "OR IYH"
			a = OR_(a,i2)				
			OpLength = 1
			T = 4
		Case 181
			OpName = "OR IYL"
			a = OR_(a,y)				
			OpLength = 1
			T = 4
		Case 182
			OpName = "OR (IY+d)"
			a = OR_(a,Memory.Read(IY + dIndex))
			OpLength = 2
			T = 19
		Case 188
			OpName = "CP IYH"
			SUB_(a,i2)				
			OpLength = 1
			T = 4
		Case 189
			OpName = "CP IYL"
			SUB_(a,y)				
			OpLength = 1
			T = 4
		Case 190
			OpName = "CP (IY+d)"
			SUB_(a,Memory.Read(IY + dIndex))
			OpLength = 2
			T = 19
		Case 203
			OpName = "CB"
			OpLength = 1
		Case 225
			OpName = "POP IY"
			Temp = Pop16
			i2 = HighByte(Temp)
			y = LowByte(Temp)
			OpLength = 1
			T = 10
		Case 227
			OpName = "EX (SP),IY"
			Temp = Memory.Read(SP)
			Memory.Write(SP,y)
			y = Temp
			Temp = Memory.Read(SP + 1)
			Memory.Write(SP + 1,i2)
			i2 = Temp
			OpLength = 1
			T = 23
		Case 229
			OpName = "PUSH IY"
			Push16(IY)
			OpLength = 1
			T = 15
		Case 233
			OpName = "JP (IY)"
			PC = IY
			OpLength = 0
			T = 8
		Case 237
			OpName = "ED"
			OpLength = 1
		Case 249
			OpName = "LD SP,IY"
			SP = IY
			OpLength = 1
			T = 10
		Case Else
			Logger.logsWARN("Missing OpCode FD + " & OpCode)
			OpName = ""
			OpLength = 1
	End Select
End Sub

Private Sub ResetFlags
	CarryFlag = 0
	ZeroFlag = 0
	Parity_OverflowFlag = 0
	SignFlag = 0
	SubstractFlag = 0
	HalfCarryFlag = 0
	CarryFlagPrime = 0
	ZeroFlagPrime = 0
	Parity_OverflowFlagPrime = 0
	SignFlagPrime = 0
	SubstractFlagPrime = 0
	HalfCarryFlagPrime = 0
End Sub
Private Sub ResetRegisters
	a = 0
	f = 0
	b = 0
	c = 0
	d = 0
	e = 0
	h = 0
	l = 0
	i1 = 0
	x = 0
	i2 = 0
	y = 0
	aPrime = 0
	fPrime = 0
	bPrime = 0
	cPrime = 0
	dPrime = 0
	ePrime = 0
	hPrime = 0
	lPrime = 0
	SP = 0
	i = 0
	r = 0
End Sub

Private Sub ADD(p1 As Int, p2 As Int) As Int
	Dim Sum  As Int
	Sum = p1 + p2
	SubstractFlag = 0
	SetCarryFlag(Sum)
	SetSignFlag(Sum)
	Sum = Bit.AND(Sum,255) 'Drop Carry bit (bit 8)
	SetZeroFlag(Sum)
	'Half carry is overflow from bit 3 to 4.
	'Test this by doing the operation only on bit 0-3 of
	'the operands and check if bit 4 is then set.
	HalfCarryFlag = Bit.AND(16,(Bit.AND(p1,15) + Bit.AND(p2,15)))/16
	
	'Overflow after an ADD is when the result has the wrong sign bit (bit 7)
	'The sum of two operands with the same sign must also have the same sign
	'There can't be overflow when adding two values with different signs
	If Bit.AND(128,(Bit.Xor(p1,p2))) = 0 Then 'There may be oveflow if both operands have the same value at bit 7
		If Bit.AND(128,(Bit.Xor(p1,Sum))) = 128 Then 'There is overflow if any operand and the sum have different values at bit 7
			Parity_OverflowFlag = 1
		Else
			Parity_OverflowFlag = 0
		End If
	Else
		Parity_OverflowFlag = 0
	End If
	Return Sum
End Sub
Private Sub ADC(p1 As Int, p2 As Int) As Int
	Dim Sum  As Int
	Sum = p1 + p2 + CarryFlag
	HalfCarryFlag = Bit.AND(16,(Bit.AND(p1,15) + Bit.AND(p2 + CarryFlag,15)))/16
	'Overflow after an ADD is when the result has the wrong sign bit (bit 7)
	'The sum of two operands with the same sign must also have the same sign
	'There can't be overflow when adding two values with different signs
	'NOTE: In this case, the carry flag is added to p2.
	If Bit.AND(128,(Bit.Xor(p1,p2 + CarryFlag))) = 0 Then 'There may be oveflow if both operands have the same value at bit 7
		If Bit.AND(128,(Bit.Xor(p1,Sum))) = 128 Then 'There is overflow if any operand and the sum have different values at bit 7
			Parity_OverflowFlag = 1
		Else
			Parity_OverflowFlag = 0
		End If
	Else
		Parity_OverflowFlag = 0
	End If
	SubstractFlag = 0
	SetCarryFlag(Sum)
	SetSignFlag(Sum)
	Sum = Bit.AND(Sum,255) 'Drop Carry bit (bit 8)
	SetZeroFlag(Sum)
	Return Sum
End Sub

Private Sub ADD16(p1 As Int, p2 As Int) As Int
	Dim Sum  As Int
	Sum = p1 + p2
	SetCarryFlag16(Sum)
	Sum = Bit.AND(Sum,65535) 'Drop Carry bit (bit 16)
	'SignFlag is preserved
	SubstractFlag = 0
	'ZeroFlag is preseved
	'HalfCarryFlag is undefiend
	'Parity_OverflowFlag is preserved
	Return Sum
End Sub
Private Sub ADC16(p1 As Int, p2 As Int) As Int
	Dim Sum  As Int
	Sum = p1 + p2 + CarryFlag
	'HalfCarryFlag is undefined
	
	'Overflow after an ADD is when the result has the wrong sign bit (bit 15)
	'The sum of two operands with the same sign must also have the same sign
	'There can't be overflow when adding two values with different signs
	'NOTE: In this case, the carry flag is added to p2.
	If Bit.AND(32768,(Bit.Xor(p1,p2 + CarryFlag))) = 0 Then 'There may be oveflow if both operands have the same value at bit 15
		If Bit.AND(32768,(Bit.Xor(p1,Sum))) = 32768 Then 'There is overflow if any operand and the sum have different values at bit 15
			Parity_OverflowFlag = 1
		Else
			Parity_OverflowFlag = 0
		End If
	Else
		Parity_OverflowFlag = 0
	End If
	SubstractFlag = 0
	SetCarryFlag16(Sum)
	SetSignFlag16(Sum)
	Sum = Bit.AND(Sum,65535) 'Drop Carry bit (bit 16)
	SetZeroFlag(Sum)
	Return Sum
End Sub
Private Sub INC(p1 As Int) As Int
	Dim Sum  As Int
	Sum = p1 + 1
	SubstractFlag = 0
	'CarryFlag is preserved
	SetSignFlag(Sum)
	Sum = Bit.AND(Sum,255) 'Drop Carry bit (bit 8)
	SetZeroFlag(Sum)
	'Half carry is overflow from bit 3 to 4.
	'Test this by doing the operation only on bit 0-3 of
	'the operands and check if bit 4 is then set.
	HalfCarryFlag = Bit.AND(16,(Bit.AND(p1,15) + Bit.AND(1,15)))/16
	
	'Overflow after an ADD is when the result has the wrong sign bit (bit 7)
	'The sum of two operands with the same sign must also have the same sign
	'There can't be overflow when adding two values with different signs
	If Bit.AND(128,(Bit.Xor(p1,1))) = 0 Then 'There may be oveflow if both operands have the same value at bit 7
		If Bit.AND(128,(Bit.Xor(p1,Sum))) = 128 Then 'There is overflow if any operand and the sum have different values at bit 7
			Parity_OverflowFlag = 1
		Else
			Parity_OverflowFlag = 0
		End If
	Else
		Parity_OverflowFlag = 0
	End If
	Return Sum
End Sub

Private Sub INC16(p1 As Int) As Int
	Return Bit.AND(p1 + 1,65535) 'Drop Carry bit (bit 16)
End Sub

Private Sub SUB_(p1 As Int, p2 As Int) As Int
	Dim Sum As Int
	Sum = p1 - p2
	SubstractFlag = 1
	SetCarryFlag(Sum)
	SetSignFlag(Sum)
	Sum = Bit.AND(Sum,255) 'Drop Carry bit (bit 8)
	SetZeroFlag(Sum)
	
	'Half carry is overflow from bit 3 to 4.
	'Test this by doing the operation only on bit 0-3 of
	'the operands and check if bit 4 is then set.
	HalfCarryFlag = Bit.AND(16,(Bit.AND(p1,15) - Bit.AND(p2,15)))/16
	
	'Overflow after a SUB is when the result has the wrong sign bit (bit 7)
	'A negative number substracted from a positive number can't give a negative result.
	'A positive number substracted from a negative number can't give a positive result.
	If Bit.AND(128,(Bit.Xor(p1,p2))) = 128 Then 'There may be oveflow if the operands have different values at Bit 7
		If Bit.AND(128,(Bit.Xor(p1,Sum))) = 128 Then 'There is overflow if the left operand and the result have different values at bit 7
			Parity_OverflowFlag = 1
		Else
			Parity_OverflowFlag = 0
		End If
	Else
		Parity_OverflowFlag = 0
	End If
	Return Sum
End Sub
Private Sub SBC(p1 As Int, p2 As Int) As Int
	Dim Sum As Int
	Sum = p1 - (p2 + CarryFlag)
	'Half carry is overflow from bit 3 to 4.
	'Test this by doing the operation only on bit 0-3 of
	'the operands and check if bit 4 is then set.
	'The formula is ((p1 & 0x0F) - (p2 & 0x0F)) & 0x10)
	HalfCarryFlag = Bit.AND(0x10,(Bit.AND(p1,0x0F) - Bit.AND(p2 + CarryFlag,0x0F)))/16
	'Overflow after a SUB is when the result has the wrong sign bit (bit 7)
	'A negative number substracted from a positive number can't give a negative result.
	'A positive number substracted from a negative number can't give a positive result.
	'NOTE: In this case, the carry flag is added to p2.
	If Bit.AND(128,(Bit.Xor(p1,p2 + CarryFlag))) = 128 Then 'There may be oveflow if the operands have different values at Bit 7
		If Bit.AND(128,(Bit.Xor(p1,Sum))) = 128 Then 'There is overflow if the left operand and the result have different values at bit 7
			Parity_OverflowFlag = 1
		Else
			Parity_OverflowFlag = 0
		End If
	Else
		Parity_OverflowFlag = 0
	End If
	SubstractFlag = 1
	SetCarryFlag(Sum)
	SetSignFlag(Sum)
	Sum = Bit.AND(Sum,255) 'Drop Carry bit (bit 8)
	SetZeroFlag(Sum)
	Return Sum
End Sub
Private Sub SBC16(p1 As Int, p2 As Int) As Int
	Dim Sum As Int
	Sum = p1 - (p2 + CarryFlag)
	'HalfCarryFlag is undefined
	'Overflow after a SUB is when the result has the wrong sign bit (bit 15)
	'A negative number substracted from a positive number can't give a negative result.
	'A positive number substracted from a negative number can't give a positive result.
	'NOTE: In this case, the carry flag is added to p2.
	If Bit.AND(32768,(Bit.Xor(p1,p2 + CarryFlag))) = 32768 Then 'There may be oveflow if the operands have different values at Bit 15
		If Bit.AND(32768,(Bit.Xor(p1,Sum))) = 32768 Then 'There is overflow if the left operand and the result have different values at bit 15
			Parity_OverflowFlag = 1
		Else
			Parity_OverflowFlag = 0
		End If
	Else
		Parity_OverflowFlag = 0
	End If
	SubstractFlag = 1
	SetCarryFlag16(Sum)
	SetSignFlag16(Sum)
	Sum = Bit.AND(Sum,65535) 'Drop Carry bit (bit 16)
	SetZeroFlag(Sum)
	Return Sum
End Sub
Private Sub DEC(p1 As Int) As Int
	Dim Sum As Int
	Sum = p1 - 1
	'CarryFlag is preserved
	Sum = Bit.AND(Sum,255) 'Drop Carry bit (bit 8)
	SignFlag = Bit.AND(Sum,128)/128 'Set Sign flag if bit 7 is set
	SubstractFlag = 1
	If Sum = 0 Then ZeroFlag = 1 Else ZeroFlag = 0
	
	'Half carry is overflow from bit 3 to 4.
	'Test this by doing the operation only on bit 0-3 of
	'the operands and check if bit 4 is then set.
	HalfCarryFlag = Bit.AND(16,(Bit.AND(p1,15) - Bit.AND(1,15)))/16
	
	'Overflow after a SUB is when the result has the wrong sign bit (bit 7)
	'A negative number substracted from a positive number can't give a negative result.
	'A positive number substracted from a negative number can't give a positive result.
	If Bit.AND(128,(Bit.Xor(p1,1))) = 128 Then 'There may be oveflow if the operands have different values at Bit 7
		If Bit.AND(128,(Bit.Xor(p1,Sum))) = 128 Then 'There is overflow if the left operand and the result have different values at bit 7
			Parity_OverflowFlag = 1
		Else
			Parity_OverflowFlag = 0
		End If
	Else
		Parity_OverflowFlag = 0
	End If
	Return Sum
End Sub
Private Sub DEC16(p1 As Int) As Int
	Return Bit.AND(p1 - 1,65535) 'Drop Carry bit (bit 16)
End Sub
Private Sub RLC(p1 As Int) As Int
	'The RLC instruction rotates the register left by one.  It can  be
	'represented symbolically as
	'C  <---   bbbbbbbb <-
	'       |             |
	'        -------------
	Dim Bit7, Result As Int
	Bit7 = Bit.AND(p1,128) / 128 'Extract bit 7
	Result = Bit.AND(254,Bit.ShiftLeft(p1,1)) + Bit7 'Shift left and set bit 0 to former bit 7
	CarryFlag = Bit7 'CarryFlag = bit former bit 7
	SetZeroFlag(Result)
	Parity_OverflowFlag = Parity(Result)
	SetSignFlag(Result)
	SubstractFlag = 0
	HalfCarryFlag = 0
	Return Result
End Sub
Private Sub RL(p1 As Int) As Int
	'The RL instruction rotates the register through the carry flag.
    ' --- C <---  bbbbbbbb  <--
    '|                         |
    '------------- > ----------
	'The  register  is rotated left by one,  the 7th bit goes  to  the
	'carry  flag,  and the carry flag goes yo the 0 bit.  Flags  apart
	'from  the  carry  flag  are as for RLC.  RL  works  on  the  same
	'registers as RLC.
	Dim Bit7, Result As Int
	Bit7 = Bit.AND(p1,128) / 128 'Extract bit 7
	Result = Bit.AND(254,Bit.ShiftLeft(p1,1)) + CarryFlag 'Shift left and set bit 0 to CarryFlag
	CarryFlag = Bit7 'CarryFlag = bit former bit 7
	SetZeroFlag(Result)
	Parity_OverflowFlag = Parity(Result)
	SetSignFlag(Result)
	SubstractFlag = 0
	HalfCarryFlag = 0
	Return Result
End Sub
Private Sub RRC(p1 As Int) As Int
	'The RRC instruction rotates the register right.
    ' ---> bbbbbbbb ---> C
    '|              |
    ' ------ < -----
	Dim Bit0, Result As Int
	Bit0 = Bit.AND(p1,1) 'Extract bit 0
	Result = Bit.AND(127,Bit.ShiftRight(p1,1)) + Bit0 * 128 'Shift Right and set bit 7 to former bit 0
	CarryFlag = Bit0 'CarryFlag = bit former bit 0
	SetZeroFlag(Result)
	Parity_OverflowFlag = Parity(Result)
	SetSignFlag(Result)
	SubstractFlag = 0
	HalfCarryFlag = 0
	Return Result
End Sub
Private Sub RR(p1 As Int) As Int
	'The  RR instruction rotates the register right through the carry flag.
    ' ---> bbbbbbbb ---> C --
    '|                       |
    ' ----------- < ---------
	'The register is rotated right by one, the 0 Bit goes to the carry
	'flag and the carry flag goes to the 7th Bit.
	Dim Bit0, Result As Int
	Bit0 = Bit.AND(p1,1) 'Extract bit 0
	Result = Bit.AND(127,Bit.ShiftRight(p1,1)) + CarryFlag * 128 'Shift Right and set bit 7 to CarryFlag
	CarryFlag = Bit0 'CarryFlag = bit former bit 0
	SetZeroFlag(Result)
	Parity_OverflowFlag = Parity(Result)
	SetSignFlag(Result)
	SubstractFlag = 0
	HalfCarryFlag = 0
	Return Result
End Sub
Private Sub SLA(p1 As Int) As Int
	'The SLA instruction shifts the register left by one.  It can  be
	'represented symbolically as
	'C  <---   bbbbbbbb <- 0
	Dim Bit7, Result As Int
	Bit7 = Bit.AND(p1,128) / 128 'Extract bit 7
	Result = Bit.AND(Bit.ShiftLeft(p1,1),255) 'Shift left and drop any overflow
	CarryFlag = Bit7 'Carry = former bit 7
	SetZeroFlag(Result)
	Parity_OverflowFlag = Parity(Result)
	SetSignFlag(Result)
	SubstractFlag = 0
	HalfCarryFlag = 0
	Return Result
End Sub
Private Sub SRA(p1 As Int) As Int
	'The SRA instruction shifts the register right by one.  It can  be
	'represented symbolically as
    ' --> bbbbbbbb ----> C
    ' -<--
	'The bits are all shifted right one.  The 0 Bit goes To the  carry
	'flag.  The  7th Bit however stays the same,  although the 6th Bit
	'equals the old 7th bit.
	Dim Bit7, Bit0, Result As Int
	Bit7 = Bit.AND(p1,128) 'Extract bit 7 value
	Bit0 = Bit.AND(p1,1) 'Extract bit 1
	Result = Bit.OR(Bit.ShiftRight(p1,1),Bit7) 'Shift right and add former bit 7
	CarryFlag = Bit0 'Carry = former bit 0
	SetZeroFlag(Result)
	Parity_OverflowFlag = Parity(Result)
	SetSignFlag(Result)
	SubstractFlag = 0
	HalfCarryFlag = 0
	Return Result
End Sub
Private Sub SRL(p1 As Int) As Int
	'The SRL instruction shifts the register right by one.  It can  be
	'represented symbolically as
    ' 0 --> bbbbbbbb ----> C
	'The bits are all shifted right one.  The 0 Bit goes To the  carry
	'flag.  The  7th Bit his replaced by 0.
	Dim Bit0, Result As Int
	Bit0 = Bit.AND(p1,1) 'Extract bit 1
	Result = Bit.ShiftRight(p1,1) 'Shift right
	CarryFlag = Bit0 'Carry = former bit 0
	SetZeroFlag(Result)
	Parity_OverflowFlag = Parity(Result)
	SetSignFlag(Result)
	SubstractFlag = 0
	HalfCarryFlag = 0
	Return Result
End Sub
Private Sub AND_(p1 As Int, p2 As Int) As Int
	Dim Result As Int
	Result = Bit.AND(p1,p2)
	CarryFlag = 0
	SetSignFlag(Result)
	SubstractFlag = 0
	SetZeroFlag(Result)
	HalfCarryFlag = 1
	Parity_OverflowFlag = Parity(Result)
	Return Result
End Sub
Private Sub XOR_(p1 As Int, p2 As Int) As Int
	Dim Result As Int
	Result = Bit.XOR(p1,p2)
	CarryFlag = 0
	SetSignFlag(Result)
	SubstractFlag = 0
	SetZeroFlag(Result)
	HalfCarryFlag = 0
	Parity_OverflowFlag = Parity(Result)
	Return Result
End Sub
Private Sub OR_(p1 As Int, p2 As Int) As Int
	Dim Result As Int
	Result = Bit.OR(p1,p2)
	CarryFlag = 0
	SetSignFlag(Result)
	SubstractFlag = 0
	SetZeroFlag(Result)
	HalfCarryFlag = 0
	Parity_OverflowFlag = Parity(Result)
	Return Result
End Sub
Private Sub BitCheck(p1 As Int, p2 As Int)
	'The  BitCheck instruction is used to test whether a specific bit is  a
	'zero or a one.
	If Bit.AND(Power(2,p1),p2) = 0 Then ZeroFlag = 1 Else ZeroFlag = 0
	'SignFlag is unaffected
	'ParityOverflowFlag is unaffected
	'CarryFlag is unaffected
	SubstractFlag = 0
	HalfCarryFlag = 1
End Sub
Private Sub BitReset(p1 As Int, p2 As Int) As Int
	'The  BitReset resets the appropiate bit to 0.
	'All flags are unaffected
	Return Bit.AND(255 - Power(2,p1), p2)
End Sub
Private Sub BitSet(p1 As Int, p2 As Int) As Int	
	'The  BitSet sets the appropiate bit to 1
	'All flags are unaffected
	Return Bit.OR(Power(2,p1), p2)
End Sub

Private Sub Parity(p1 As Int) As Int
	Dim ParityArray() As Int
	ParityArray = Array As Int (1,0,0,1,0,1,1,0,0,1,1,0,1,0,0,1,0,1,1,0,1,0,0,1,1,0,0,1,0,1,1,0,0,1,1,0,1,0,0,1,1,0,0,1,0,1,1,0,1,0,0,1,0,1,1,0,0,1,1,0,1,0,0,1,0,1,1,0,1,0,0,1,1,0,0,1,0,1,1,0,1,0,0,1,0,1,1,0,0,1,1,0,1,0,0,1,1,0,0,1,0,1,1,0,0,1,1,0,1,0,0,1,0,1,1,0,1,0,0,1,1,0,0,1,0,1,1,0,0,1,1,0,1,0,0,1,1,0,0,1,0,1,1,0,1,0,0,1,0,1,1,0,0,1,1,0,1,0,0,1,1,0,0,1,0,1,1,0,0,1,1,0,1,0,0,1,0,1,1,0,1,0,0,1,1,0,0,1,0,1,1,0,1,0,0,1,0,1,1,0,0,1,1,0,1,0,0,1,0,1,1,0,1,0,0,1,1,0,0,1,0,1,1,0,0,1,1,0,1,0,0,1,1,0,0,1,0,1,1,0,1,0,0,1,0,1,1,0,0,1,1,0,1,0,0,1)
	Return ParityArray(p1)
End Sub
Private Sub Push16(p1 As Int)
	SP = DEC16(SP)
	Memory.Write(SP,HighByte(p1))
	SP = DEC16(SP)
	Memory.Write(SP,LowByte(p1))
End Sub
Private Sub Pop16 As Int
	Dim Result As Int
	Result = Memory.Read16(SP)
	SP = INC16(SP)
	SP = INC16(SP)
	Return Result
End Sub

Private Sub IND
	'Reads the (C) port and writes the result to (HL), then decrements HL and decrements B.
	Memory.Write(HL,In(c))
	'Decrement HL
	Temp = DEC16(HL)
	h = HighByte(Temp)
	l = LowByte(Temp)
	b = Bit.AND(b - 1,255) 'B = B - 1. Remove any carry bit 8 
	'C is preserved, the N flag is set. S, H and P/V are undefined.
	'Z is set if B becomes zero after decrementing, otherwise it is reset.
	SetZeroFlag(b)
	SubstractFlag = 1
End Sub


Private Sub LDI
	Dim Temp As Int
	Memory.Write(DE,Memory.Read(HL))'memory contents (DE) = memory contents (HL)
	Temp = INC16(DE)
	d = HighByte(Temp)
	e = LowByte(Temp)
	Temp = INC16(HL)
	h = HighByte(Temp)
	l = LowByte(Temp)
	Temp = DEC16(BC)
	b = HighByte(Temp)
	c = LowByte(Temp)
	If BC = 0 Then Parity_OverflowFlag = 0 Else Parity_OverflowFlag = 1
	'SignFlag is unaffected
	'ZeroFlag is unaffected
	'CarryFlag is unaffected
	HalfCarryFlag = 0
	SubstractFlag = 0
End Sub
Private Sub LDD
	Dim Temp As Int
	Memory.Write(DE,Memory.Read(HL))'memory contents (DE) = memory contents (HL)
	Temp = DEC16(DE)
	d = HighByte(Temp)
	e = LowByte(Temp)
	Temp = DEC16(HL)
	h = HighByte(Temp)
	l = LowByte(Temp)
	Temp = DEC16(BC)
	b = HighByte(Temp)
	c = LowByte(Temp)
	If BC = 0 Then Parity_OverflowFlag = 0 Else Parity_OverflowFlag = 1		
	'SignFlag is unaffected
	'ZeroFlag is unaffected
	'CarryFlag is unaffected
	HalfCarryFlag = 0
	SubstractFlag = 0
End Sub
Private Sub CPI_
	Dim Result As Int
	Result = a - Memory.Read(HL) 'A - (HL)
	If Result = 0 Then ZeroFlag = 1 Else ZeroFlag = 0
	HalfCarryFlag = Bit.AND(16,(Bit.AND(a,15) - Bit.AND(Memory.Read(HL),0x15)))/16
	SubstractFlag = 0
	'CarryFlag is unaffected
	SetSignFlag(Result)
	Temp = INC16(HL)
	h = HighByte(Temp)
	l = LowByte(Temp)
	Temp = DEC16(BC)
	b = HighByte(Temp)
	c = LowByte(Temp)
	If BC = 0 Then Parity_OverflowFlag = 0 Else Parity_OverflowFlag = 1
End Sub

Private Sub CPD
	Dim Result As Int
	Dim Temp As Int
	Result = a - Memory.Read(HL) 'A - (HL)
	If Result = 0 Then ZeroFlag = 1 Else ZeroFlag = 0
	HalfCarryFlag = Bit.AND(16,(Bit.AND(a,15) - Bit.AND(Memory.Read(HL),0x15)))/16
	SubstractFlag = 0
	'CarryFlag is unaffected
	SetSignFlag(Result)
	Temp = DEC16(HL)
	h = HighByte(Temp)
	l = LowByte(Temp)
	Temp = DEC16(BC)
	b = HighByte(Temp)
	c = LowByte(Temp)
	If BC = 0 Then Parity_OverflowFlag = 0 Else Parity_OverflowFlag = 1		
End Sub
Private Sub dIndex As Int 'Used by DD and FD where d comes after the OpCode. 
	Dim Dis As Byte
	Dis = Memory.Read(PC + 1)
	Return Dis
End Sub
Private Sub dIndex_1 As Int 'Used by DDCB and FDCB where d comes before the OpCode. 
	Dim Dis As Byte
	Dis = Memory.Read(PC)
	Return Dis
End Sub

Private Sub SetCarryFlag(p1 As Int)
	CarryFlag = Bit.AND(p1,256)/256 'Set Carry flag if bit 8 is set
End Sub
Private Sub SetCarryFlag16(p1 As Int)
	CarryFlag = Bit.AND(p1,65536)/65536 'Set Carry flag if bit 16 is set
End Sub
Private Sub SetSignFlag(p1 As Int)
	SignFlag = Bit.AND(p1,128)/128 'Set Sign flag if bit 7 is set
End Sub
Private Sub SetSignFlag16(p1 As Int)
	SignFlag = Bit.AND(p1,32768)/32768 'Set Sign flag if bit 15 is set
End Sub
Private Sub SetZeroFlag(p1 As Int)
	If p1 = 0 Then ZeroFlag = 1 Else ZeroFlag = 0
End Sub

Private Sub N As Int 
	Return Memory.Read(PC + 1)
End Sub
Private Sub Nhigh As Int
	Return Memory.Read(PC + 2)
End Sub
Private Sub NN As Int
	Return Memory.Read16(PC + 1)
End Sub
Public Sub BC As Int
	Return b * 256 + c
End Sub
Public Sub DE As Int
	Return d * 256 + e
End Sub
Public Sub HL As Int
	Return h * 256 + l
End Sub
Public Sub IX As Int
	Return i1 * 256 + x
End Sub
Public Sub IY As Int
	Return i2 * 256 + y
End Sub
Private Sub In(p1 As Int) As Int
	'This is the standard IN function where the high byte equals the B register.
	Dim InValue As Int
	InValue = InPort.In(b * 256 + p1)
	'S is set if input data is negative; reset otherwise
	SetSignFlag(InValue)
	SetZeroFlag(InValue)
	HalfCarryFlag = 0
	Parity_OverflowFlag = Parity(InValue)
	SubstractFlag = 0
	'CarryFlag is not affected
	Return InValue
End Sub
Private Sub HighByte(p1 As Int) As Int
	Return p1 / 256
End Sub
Private Sub LowByte(p1 As Int) As Int
	Dim Temp As Int
	Temp = p1 / 256
	Return p1 - Temp * 256
End Sub
Private Sub CatchError(p1 As String)
	Logger.logsERROR(p1)
	Logger.logsERROR("PC = " & PC)
	Logger.logsERROR("A = " & a)
	Logger.logsERROR("BC = " & BC)
	Logger.logsERROR("DE = " & DE)
	Logger.logsERROR("HL = " & HL)
	Logger.logsERROR("InterruptEnabled = " & InterruptEnabled)
	For i = 0 To 9
		Logger.logsERROR("PC -" & i & " = " & PCHistory(i))
	Next
	Main.MainTimer.Enabled = False
End Sub

Private Sub IncR
	'The R register is a counter that is updated during every Z80 M1 cycle 
	'(approximately equivalent to every instruction), so long as DD, FD, ED 
	'and CB are to be regarded as separate instructions, so shifted instructions 
	'increase R by two. There's an interesting exception: doubly-shifted opcodes, 
	'the DD CB and FD CB ones, also increase R by two. LDI increases R by two, 
	'LDIR increases it by 2 times BC, as does LDDR etcetera. R is set to zero 
	'when the Z80 is reset.
	r = Bit.AND(r + 1,127) 
End Sub


